@@ -1,3 +1,5 @@
+#ifndef GDK_DISABLE_DEPRECATED
+
 #ifndef __GDK_FONT_H__
 #define __GDK_FONT_H__
 
@@ -8,6 +10,8 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#define GDK_TYPE_FONT gdk_font_get_type
+
 /* Types of font.
  *   GDK_FONT_FONT: the font is an XFontStruct.
  *   GDK_FONT_FONTSET: the font is an XFontSet used for I18N.
@@ -25,6 +29,8 @@ struct _GdkFont
   gint descent;
 };
 
+GType    gdk_font_get_type  (void);
+
 GdkFont* gdk_font_load	    (const gchar    *font_name);
 GdkFont* gdk_fontset_load   (const gchar    *fontset_name);
 
@@ -100,3 +106,5 @@ void	 gdk_font_full_name_free (gchar *name);
 #endif /* __cplusplus */
 
 #endif /* __GDK_FONT_H__ */
+
+#endif /* GDK_DISABLE_DEPRECATED */
