@@ -31,11 +31,7 @@
 #include "gdkdrawable.h"
 #include "gdkpixmap.h"
 
-#ifndef USE_BACKING_STORE
-#ifndef GDK_WINDOWING_WIN32
-#define USE_BACKING_STORE	/* Doesn't work yet on Win32 */
-#endif
-#endif
+#define USE_BACKING_STORE	/* Appears to work on Win32, too, now. */
 
 typedef struct _GdkWindowPaint GdkWindowPaint;
 
@@ -140,12 +136,14 @@ static void         gdk_window_real_set_colormap (GdkDrawable *drawable,
 static GdkColormap* gdk_window_real_get_colormap (GdkDrawable *drawable);
 
 static GdkDrawable* gdk_window_get_composite_drawable (GdkDrawable *drawable,
-                                                       gint         x,
-                                                       gint         y,
-                                                       gint         width,
-                                                       gint         height,
-                                                       gint        *composite_x_offset,
-                                                       gint        *composite_y_offset);
+						       gint         x,
+						       gint         y,
+						       gint         width,
+						       gint         height,
+						       gint        *composite_x_offset,
+						       gint        *composite_y_offset);
+static GdkRegion*   gdk_window_get_clip_region        (GdkDrawable *drawable);
+static GdkRegion*   gdk_window_get_visible_region     (GdkDrawable *drawable);
 
 static void gdk_window_free_paint_stack (GdkWindow *window);
 
@@ -190,6 +188,8 @@ gdk_window_init (GdkWindowObject *window)
 
   window->window_type = GDK_WINDOW_CHILD;
 
+  window->state = GDK_WINDOW_STATE_WITHDRAWN;
+  
   window->impl = g_object_new (_gdk_window_impl_get_type (), NULL);
 }
 
@@ -221,6 +221,8 @@ gdk_window_class_init (GdkWindowObjectClass *klass)
   drawable_class->get_colormap = gdk_window_real_get_colormap;
   drawable_class->get_visual = gdk_window_real_get_visual;
   drawable_class->get_image = gdk_window_get_image;
+  drawable_class->get_clip_region = gdk_window_get_clip_region;
+  drawable_class->get_visible_region = gdk_window_get_visible_region;
   drawable_class->get_composite_drawable = gdk_window_get_composite_drawable;
 }
 
@@ -290,7 +292,7 @@ _gdk_window_destroy_hierarchy (GdkWindow *window,
     case GDK_WINDOW_FOREIGN:
       if (!GDK_WINDOW_DESTROYED (window))
 	{
-	  private->mapped = FALSE;
+	  private->state |= GDK_WINDOW_STATE_WITHDRAWN;
 	  private->destroyed = TRUE;
 	  
 	  _gdk_windowing_window_destroy (window, recursing, foreign_destroy);
@@ -476,8 +478,7 @@ gdk_window_add_filter (GdkWindow     *window,
   GList *tmp_list;
   GdkEventFilter *filter;
   
-  g_return_if_fail (window != NULL);
-  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (window == NULL || GDK_IS_WINDOW (window));
 
   private = (GdkWindowObject*) window;
   if (private && GDK_WINDOW_DESTROYED (window))
@@ -515,8 +516,7 @@ gdk_window_remove_filter (GdkWindow     *window,
   GList *tmp_list, *node;
   GdkEventFilter *filter;
   
-  g_return_if_fail (window != NULL);
-  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (window == NULL || GDK_IS_WINDOW (window));
 
   private = (GdkWindowObject*) window;
   
@@ -573,12 +573,9 @@ gdk_window_get_toplevels (void)
 gboolean 
 gdk_window_is_visible (GdkWindow *window)
 {
-  GdkWindowObject *private = (GdkWindowObject *)window;
-  
-  g_return_val_if_fail (window != NULL, FALSE);
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
   
-  return private->mapped;
+  return GDK_WINDOW_IS_MAPPED (window);
 }
 
 /*************************************************************
@@ -605,7 +602,7 @@ gdk_window_is_viewable (GdkWindow *window)
 	 (private != (GdkWindowObject *)gdk_parent_root) &&
 	 (GDK_WINDOW_TYPE (private) != GDK_WINDOW_FOREIGN))
     {
-      if (!private->mapped)
+      if (!GDK_WINDOW_IS_MAPPED (window))
 	return FALSE;
       
       private = (GdkWindowObject *)private->parent;
@@ -614,6 +611,35 @@ gdk_window_is_viewable (GdkWindow *window)
   return TRUE;
 }
 
+/**
+ * gdk_window_get_state:
+ * @window: a #GdkWindow
+ * 
+ * Gets the bitwise OR of the currently active window state flags,
+ * from the #GdkWindowState enumeration.
+ * 
+ * Return value: window state bitfield
+ **/
+GdkWindowState
+gdk_window_get_state (GdkWindow *window)
+{
+  GdkWindowObject *private = (GdkWindowObject *)window;
+  
+  g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
+  
+  return private->state;
+}
+
+/**
+ * gdk_window_begin_paint_rect:
+ * @window: a #GdkWindow
+ * @rectangle: rectangle you intend to draw to
+ *
+ * A convenience wrapper around gdk_window_begin_paint_region() which
+ * creates a rectangular region for you. See
+ * gdk_window_begin_paint_region() for details.
+ * 
+ **/
 void
 gdk_window_begin_paint_rect (GdkWindow    *window,
 			     GdkRectangle *rectangle)
@@ -684,6 +710,51 @@ gdk_window_paint_init_bg (GdkWindow      *window,
 #include "x11/gdkx.h"
 #endif
 
+/**
+ * gdk_window_begin_paint_region:
+ * @window: a #GdkWindow
+ * @region: region you intend to draw to
+ *
+ * Indicates that you are beginning the process of redrawing @region.
+ * A backing store (offscreen buffer) large enough to contain @region
+ * will be created. The backing store will be initialized with the
+ * background color or background pixmap for @window. Then, all
+ * drawing operations performed on @window will be diverted to the
+ * backing store.  When you call gdk_window_end_paint(), the backing
+ * store will be copied to @window, making it visible onscreen. Only
+ * the part of window contained in @region will be modified; that is,
+ * drawing operations are clipped to @region.
+ *
+ * The net result of all this is to remove flicker, because the user
+ * sees the finished product appear all at once when you call
+ * gdk_window_end_paint(). If you draw to @window directly without
+ * calling gdk_window_begin_paint_region(), the user may see flicker
+ * as individual drawing operations are performed in sequence.  The
+ * clipping and background-initializing features of
+ * gdk_window_begin_paint_region() are conveniences for the
+ * programmer, so you can avoid doing that work yourself.
+ *
+ * When using GTK+, the widget system automatically places calls to
+ * gdk_window_begin_paint_region() and gdk_window_end_paint() around
+ * emissions of the expose_event signal. That is, if you're writing an
+ * expose event handler, you can assume that the exposed area in
+ * #GdkEventExpose has already been cleared to the window background,
+ * is already set as the clip region, and already has a backing store.
+ * Therefore in most cases, application code need not call
+ * gdk_window_begin_paint_region(). (You can disable the automatic
+ * calls around expose events on a widget-by-widget basis by calling
+ * gtk_widget_set_double_buffered().)
+ *
+ * If you call this function multiple times before calling the
+ * matching gdk_window_end_paint(), the backing stores are pushed onto
+ * a stack. gdk_window_end_paint() copies the topmost backing store
+ * onscreen, subtracts the topmost region from all other regions in
+ * the stack, and pops the stack. All drawing operations affect only
+ * the topmost backing store in the stack. One matching call to
+ * gdk_window_end_paint() is required for each call to
+ * gdk_window_begin_paint_region().
+ * 
+ **/
 void	      
 gdk_window_begin_paint_region (GdkWindow *window,
 			       GdkRegion *region)
@@ -783,6 +854,19 @@ gdk_window_begin_paint_region (GdkWindow *window,
 #endif /* USE_BACKING_STORE */
 }
 
+/**
+ * gdk_window_end_paint:
+ * @window: a #GdkWindow
+ *
+ * Indicates that the backing store created by the most recent call to
+ * gdk_window_begin_paint_region() should be copied onscreen and
+ * deleted, leaving the next-most-recent backing store or no backing
+ * store at all as the active paint region. See
+ * gdk_window_begin_paint_region() for full details. It is an error to
+ * call this function without a matching
+ * gdk_window_begin_paint_region() first.
+ * 
+ **/
 void
 gdk_window_end_paint (GdkWindow *window)
 {
@@ -798,8 +882,12 @@ gdk_window_end_paint (GdkWindow *window)
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
-  
-  g_return_if_fail (private->paint_stack != NULL);
+
+  if (private->paint_stack == NULL)
+    {
+      g_warning (G_STRLOC": no preceding call to gdk_window_begin_paint_region(), see documentation");
+      return;
+    }
 
   paint = private->paint_stack->data;
   private->paint_stack = g_slist_delete_link (private->paint_stack, private->paint_stack);
@@ -1211,6 +1299,41 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
   return tmp_pixmap;
 }
 
+static GdkRegion*
+gdk_window_get_clip_region (GdkDrawable *drawable)
+{
+  GdkWindowObject *private = (GdkWindowObject *)drawable;
+  GdkRegion *result;
+
+  result = gdk_drawable_get_clip_region (private->impl);
+
+  if (private->paint_stack)
+    {
+      GdkRegion *paint_region = gdk_region_new ();
+      GSList *tmp_list = private->paint_stack;
+
+      while (tmp_list)
+	{
+	  GdkWindowPaint *paint = tmp_list->data;
+	  
+	  gdk_region_union (paint_region, paint->region);
+	}
+
+      gdk_region_intersect (result, paint_region);
+      gdk_region_destroy (paint_region);
+    }
+
+  return result;
+}
+
+static GdkRegion*
+gdk_window_get_visible_region (GdkDrawable *drawable)
+{
+  GdkWindowObject *private = (GdkWindowObject*) drawable;
+  
+  return gdk_drawable_get_visible_region (private->impl);
+}
+
 static void
 gdk_window_draw_drawable (GdkDrawable *drawable,
 			  GdkGC       *gc,
@@ -1602,6 +1725,7 @@ gdk_window_get_image (GdkDrawable *drawable,
 
 static GSList *update_windows = NULL;
 static guint update_idle = 0;
+static gboolean debug_updates = FALSE;
 
 static void
 gdk_window_process_updates_internal (GdkWindow *window)
@@ -1622,8 +1746,17 @@ gdk_window_process_updates_internal (GdkWindow *window)
 	{
 	  GdkEvent event;
 	  GdkRectangle window_rect;
+	  GdkRegion *expose_region;
+	  GdkRegion *window_region;
           gint width, height;
 
+          if (debug_updates)
+            {
+              /* Make sure we see the red invalid area before redrawing. */
+              gdk_flush ();
+              g_usleep (70000);
+            }
+          
           gdk_drawable_get_size (GDK_DRAWABLE (private), &width, &height);
           
 	  window_rect.x = 0;
@@ -1636,21 +1769,40 @@ gdk_window_process_updates_internal (GdkWindow *window)
 	  event.expose.type = GDK_EXPOSE;
 	  event.expose.window = gdk_window_ref (window);
 	  event.expose.count = 0;
-      
-	  gdk_region_get_clipbox (update_area, &event.expose.area);
-	  if (gdk_rectangle_intersect (&event.expose.area, &window_rect, &event.expose.area))
+
+	  if (save_region)
+	    expose_region = gdk_region_copy (update_area);
+	  else
+	    expose_region = update_area;
+	  window_region = gdk_region_rectangle (&window_rect);
+	  gdk_region_intersect (expose_region,
+				window_region);
+	  gdk_region_destroy (window_region);
+	  
+	  event.expose.region = expose_region;
+	  gdk_region_get_clipbox (expose_region, &event.expose.area);
+	  
+	  if (!gdk_region_empty (expose_region))
 	    {
 	      (*gdk_event_func) (&event, gdk_event_data);
 	    }
 
+	  if (expose_region != update_area)
+	    gdk_region_destroy (expose_region);
 	  gdk_window_unref (window);
 	}
-      
       if (!save_region)
 	gdk_region_destroy (update_area);
     }
 }
 
+/**
+ * gdk_window_process_all_updates:
+ *
+ * Calls gdk_window_process_updates() for all windows (see #GdkWindow)
+ * in the application.
+ * 
+ **/
 void
 gdk_window_process_all_updates (void)
 {
@@ -1682,6 +1834,21 @@ gdk_window_update_idle (gpointer data)
   return FALSE;
 }
 
+/**
+ * gdk_window_process_updates:
+ * @window: a #GdkWindow
+ * @update_children: whether to also process updates for child windows
+ *
+ * Sends one or more expose events to @window. The areas in each 
+ * expose event will cover the entire update area for the window (see
+ * gdk_window_invalidate_region() for details). Normally GDK calls
+ * gdk_window_process_all_updates() on your behalf, so there's no
+ * need to call this function unless you want to force expose events
+ * to be delivered immediately and synchronously (vs. the usual
+ * case, where GDK delivers them in an idle handler). Occasionally
+ * this is useful to produce nicer scrolling behavior, for example.
+ * 
+ **/
 void
 gdk_window_process_updates (GdkWindow *window,
 			    gboolean   update_children)
@@ -1690,9 +1857,9 @@ gdk_window_process_updates (GdkWindow *window,
 
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
-
+  
   if (private->update_area)
-    {
+    {      
       gdk_window_process_updates_internal (window);
       update_windows = g_slist_remove (update_windows, window);
     }
@@ -1708,12 +1875,24 @@ gdk_window_process_updates (GdkWindow *window,
     }
 }
 
+/**
+ * gdk_window_invalidate_rect:
+ * @window: a #GdkWindow
+ * @rect: rectangle to invalidate
+ * @invalidate_children: whether to also invalidate child windows
+ *
+ * A convenience wrapper around gdk_window_invalidate_region() which
+ * invalidates a rectangular region. See
+ * gdk_window_invalidate_region() for details.
+ * 
+ **/
 void
 gdk_window_invalidate_rect   (GdkWindow    *window,
 			      GdkRectangle *rect,
 			      gboolean      invalidate_children)
 {
   GdkRectangle window_rect;
+  GdkRegion *region;
   GdkWindowObject *private = (GdkWindowObject *)window;
 
   g_return_if_fail (window != NULL);
@@ -1722,7 +1901,7 @@ gdk_window_invalidate_rect   (GdkWindow    *window,
   if (GDK_WINDOW_DESTROYED (window))
     return;
   
-  if (private->input_only || !private->mapped)
+  if (private->input_only || !GDK_WINDOW_IS_MAPPED (window))
     return;
 
   if (!rect)
@@ -1734,63 +1913,43 @@ gdk_window_invalidate_rect   (GdkWindow    *window,
                              &window_rect.height);
       rect = &window_rect;
     }
-  
-  if (private->update_area)
-    {
-      gdk_region_union_with_rect (private->update_area, rect);
-    }
-  else
-    {
-      update_windows = g_slist_prepend (update_windows, window);
-      private->update_area = gdk_region_rectangle (rect);
 
-      if (!private->update_freeze_count && !update_idle)
-	update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
-				       gdk_window_update_idle, NULL, NULL);
-    }
-
-
-  if (invalidate_children)
-    {
-      GList *tmp_list;
-      GdkRectangle child_rect, new_rect;
-
-      tmp_list = private->children;
-      while (tmp_list)
-	{
-	  GdkWindowObject *child = tmp_list->data;
-	  tmp_list = tmp_list->next;
-
-	  if (!child->input_only)
-	    {
-              gint width, height;
-
-              gdk_drawable_get_size (GDK_DRAWABLE (child),
-                                     &width, &height);
-              
-	      child_rect.x = child->x;
-	      child_rect.y = child->y;
-	      child_rect.width = width;
-	      child_rect.height = height;
-	      
-	      if (gdk_rectangle_intersect (rect, &child_rect, &new_rect))
-		{
-		  new_rect.x -= child_rect.x;
-		  new_rect.y -= child_rect.y;
-		  
-		  gdk_window_invalidate_rect ((GdkWindow *)child, &new_rect, TRUE);
-		}
-	    }
-	}
-    }
+  region = gdk_region_rectangle (rect);
+  gdk_window_invalidate_region (window, region, invalidate_children);
+  gdk_region_destroy (region);
 }
 
+/**
+ * gdk_window_invalidate_region:
+ * @window: a #GdkWindow
+ * @region: a #GdkRegion
+ * @invalidate_children: %TRUE to also invalidate child windows 
+ *
+ * Adds @region to the update area for @window. The update area is the
+ * region that needs to be redrawn, or "dirty region." The call
+ * gdk_window_process_updates() sends one or more expose events to the
+ * window, which together cover the entire update area. An
+ * application would normally redraw the contents of @window in
+ * response to those expose events.
+ *
+ * GDK will call gdk_window_process_all_updates() on your behalf
+ * whenever your program returns to the main loop and becomes idle, so
+ * normally there's no need to do that manually, you just need to
+ * invalidate regions that you know should be redrawn.
+ *
+ * The @invalidate_children parameter controls whether the region of
+ * each child window that intersects @region will also be invalidated.
+ * If %FALSE, then the update area for child windows will remain
+ * unaffected.
+ *
+ **/
 void
 gdk_window_invalidate_region (GdkWindow *window,
 			      GdkRegion *region,
 			      gboolean   invalidate_children)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkRegion *visible_region;
 
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -1798,62 +1957,95 @@ gdk_window_invalidate_region (GdkWindow *window,
   if (GDK_WINDOW_DESTROYED (window))
     return;
   
-  if (private->input_only || !private->mapped)
+  if (private->input_only || !GDK_WINDOW_IS_MAPPED (window))
     return;
-  
-  if (private->update_area)
-    {
-      gdk_region_union (private->update_area, region);
-    }
-  else
-    {
-      update_windows = g_slist_prepend (update_windows, window);
-      private->update_area = gdk_region_copy (region);
 
-      if (!private->update_freeze_count && !update_idle)
-	update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
-				       gdk_window_update_idle, NULL, NULL);
-    }
+  visible_region = gdk_drawable_get_visible_region (window);
+  gdk_region_intersect (visible_region, region);
 
-  if (invalidate_children)
+  if (!gdk_region_empty (visible_region))
     {
-      GList *tmp_list;
-      GdkRectangle child_rect;
-      GdkRegion *child_region;
+      if (debug_updates)
+        {
+          /* Draw ugly color all over the newly-invalid region */
+          GdkRectangle ugly_rect;
+          GdkGC *ugly_gc;
+          GdkColor ugly_color = { 0, 60000, 10000, 10000 };
+          
+          ugly_gc = gdk_gc_new (window);
 
-      tmp_list = private->children;
-      while (tmp_list)
+          gdk_gc_set_rgb_fg_color (ugly_gc, &ugly_color);
+          
+	  gdk_region_get_clipbox (visible_region, &ugly_rect);
+
+          gdk_draw_rectangle (window,
+                              ugly_gc,
+                              TRUE,
+                              ugly_rect.x, ugly_rect.y,
+                              ugly_rect.width, ugly_rect.height);
+          
+          g_object_unref (G_OBJECT (ugly_gc));
+        }
+      
+      if (private->update_area)
 	{
-	  GdkWindowObject *child = tmp_list->data;
-	  tmp_list = tmp_list->next;
-
-	  if (!child->input_only)
+	  gdk_region_union (private->update_area, visible_region);
+	}
+      else
+	{
+	  update_windows = g_slist_prepend (update_windows, window);
+	  private->update_area = gdk_region_copy (visible_region);
+	  
+	  if (!private->update_freeze_count && !update_idle)
+	    update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
+					   gdk_window_update_idle, NULL, NULL);
+	}
+      
+      if (invalidate_children)
+	{
+	  GList *tmp_list;
+	  
+	  tmp_list = private->children;
+	  while (tmp_list)
 	    {
-              gint width, height;
-
-              gdk_drawable_get_size (GDK_DRAWABLE (child),
-                                     &width, &height);
-              
-	      child_rect.x = child->x;
-	      child_rect.y = child->y;
-	      child_rect.width = width;
-	      child_rect.height = height;
+	      GdkWindowObject *child = tmp_list->data;
+	      tmp_list = tmp_list->next;
 	      
-	      child_region = gdk_region_rectangle (&child_rect);
-	      gdk_region_intersect (child_region, region);
-	      
-	      if (!gdk_region_empty (child_region))
+	      if (!child->input_only)
 		{
-		  gdk_region_offset (child_region, - child_rect.x, - child_rect.y);
+		  GdkRegion *child_region;
+		  gint x, y;
+
+		  gdk_window_get_position ((GdkWindow *)child, &x, &y);
+
+		  /* This copy could be saved with a little more complexity */
+		  child_region = gdk_region_copy (visible_region);
+		  gdk_region_offset (child_region, -x, -y);
+		  
 		  gdk_window_invalidate_region ((GdkWindow *)child, child_region, TRUE);
+		  
+		  gdk_region_destroy (child_region);
 		}
-	      
-	      gdk_region_destroy (child_region);
 	    }
 	}
     }
+  
+  gdk_region_destroy (visible_region);
 }
 
+/**
+ * gdk_window_get_update_area:
+ * @window: a #GdkWindow
+ * 
+ * Transfers ownership of the update area from @window to the caller
+ * of the function. That is, after calling this function, @window will
+ * no longer have an invalid/dirty region; the update area is removed
+ * from @window and handed to you. If a window has no update area,
+ * gdk_window_get_update_area() returns %NULL. You are responsible for
+ * calling gdk_region_destroy() on the returned region if it's non-%NULL.
+ * 
+ * Return value: the update area for @window
+ **/
 GdkRegion *
 gdk_window_get_update_area (GdkWindow *window)
 {
@@ -1926,3 +2118,162 @@ gdk_window_thaw_updates (GdkWindow *window)
 				   gdk_window_update_idle, NULL, NULL);
 }
 
+/**
+ * gdk_window_set_debug_updates:
+ * @setting: %TRUE to turn on update debugging
+ *
+ * With update debugging enabled, calls to
+ * gdk_window_invalidate_region() clear the invalidated region of the
+ * screen to a noticeable color, and GDK pauses for a short time
+ * before sending exposes to windows during
+ * gdk_window_process_updates().  The net effect is that you can see
+ * the invalid region for each window and watch redraws as they
+ * occur. This allows you to diagnose inefficiencies in your application.
+ *
+ * In essence, because the GDK rendering model prevents all flicker,
+ * if you are redrawing the same region 400 times you may never
+ * notice, aside from noticing a speed problem. Enabling update
+ * debugging causes GTK to flicker slowly and noticeably, so you can
+ * see exactly what's being redrawn when, in what order.
+ *
+ * The --gtk-debug=updates command line option passed to GTK+ programs
+ * enables this debug option at application startup time. That's
+ * usually more useful than calling gdk_window_set_debug_updates()
+ * yourself, though you might want to use this function to enable
+ * updates sometime after application startup time.
+ * 
+ **/
+void
+gdk_window_set_debug_updates (gboolean setting)
+{
+  debug_updates = setting;
+}
+
+/**
+ * gdk_window_constrain_size:
+ * @geometry: a #GdkGeometry structure
+ * @flags: a mask indicating what portions of @geometry are set
+ * @width: desired width of window
+ * @height: desired height of the window
+ * @new_width: location to store resulting width
+ * @new_height: location to store resulting height
+ * 
+ * Constrains a desired width and height according to a 
+ * set of geometry hints (such as minimum and maximum size).
+ */
+void
+gdk_window_constrain_size (GdkGeometry *geometry,
+			   guint        flags,
+			   gint         width,
+			   gint         height,
+			   gint        *new_width,
+			   gint        *new_height)
+{
+  /* This routine is partially borrowed from fvwm.
+   *
+   * Copyright 1993, Robert Nation
+   *     You may use this code for any purpose, as long as the original
+   *     copyright remains in the source code and all documentation
+   *
+   * which in turn borrows parts of the algorithm from uwm
+   */
+  gint min_width = 0;
+  gint min_height = 0;
+  gint base_width = 0;
+  gint base_height = 0;
+  gint xinc = 1;
+  gint yinc = 1;
+  gint max_width = G_MAXINT;
+  gint max_height = G_MAXINT;
+  
+#define FLOOR(value, base)	( ((gint) ((value) / (base))) * (base) )
+
+  if ((flags & GDK_HINT_BASE_SIZE) && (flags & GDK_HINT_MIN_SIZE))
+    {
+      base_width = geometry->base_width;
+      base_height = geometry->base_height;
+      min_width = geometry->min_width;
+      min_height = geometry->min_height;
+    }
+  else if (flags & GDK_HINT_BASE_SIZE)
+    {
+      base_width = geometry->base_width;
+      base_height = geometry->base_height;
+      min_width = geometry->base_width;
+      min_height = geometry->base_height;
+    }
+  else if (flags & GDK_HINT_MIN_SIZE)
+    {
+      base_width = geometry->min_width;
+      base_height = geometry->min_height;
+      min_width = geometry->min_width;
+      min_height = geometry->min_height;
+    }
+
+  if (flags & GDK_HINT_MAX_SIZE)
+    {
+      max_width = geometry->max_width ;
+      max_height = geometry->max_height;
+    }
+
+  if (flags & GDK_HINT_RESIZE_INC)
+    {
+      xinc = MAX (xinc, geometry->width_inc);
+      yinc = MAX (yinc, geometry->height_inc);
+    }
+  
+  /* clamp width and height to min and max values
+   */
+  width = CLAMP (width, min_width, max_width);
+  height = CLAMP (height, min_height, max_height);
+  
+  /* shrink to base + N * inc
+   */
+  width = base_width + FLOOR (width - base_width, xinc);
+  height = base_height + FLOOR (height - base_height, yinc);
+
+  /* constrain aspect ratio, according to:
+   *
+   *                width     
+   * min_aspect <= -------- <= max_aspect
+   *                height    
+   */
+  
+  if (flags & GDK_HINT_ASPECT &&
+      geometry->min_aspect > 0 &&
+      geometry->max_aspect > 0)
+    {
+      gint delta;
+
+      if (geometry->min_aspect * height > width)
+	{
+	  delta = FLOOR (height - width * geometry->min_aspect, yinc);
+	  if (height - delta >= min_height)
+	    height -= delta;
+	  else
+	    { 
+	      delta = FLOOR (height * geometry->min_aspect - width, xinc);
+	      if (width + delta <= max_width) 
+		width += delta;
+	    }
+	}
+      
+      if (geometry->max_aspect * height < width)
+	{
+	  delta = FLOOR (width - height * geometry->max_aspect, xinc);
+	  if (width - delta >= min_width) 
+	    width -= delta;
+	  else
+	    {
+	      delta = FLOOR (width / geometry->max_aspect - height, yinc);
+	      if (height + delta <= max_height)
+		height += delta;
+	    }
+	}
+    }
+
+#undef FLOOR
+  
+  *new_width = width;
+  *new_height = height;
+}
