@@ -37,6 +37,7 @@
 static gint  gdk_colormap_match_color (GdkColormap *cmap,
 				       GdkColor    *color,
 				       const gchar *available);
+static void  gdk_fb_color_round_to_hw (GdkColor *color);
 
 static gpointer parent_class;
 
@@ -112,6 +113,7 @@ gdk_colormap_new (GdkVisual *visual,
 		  gint       private_cmap)
 {
   GdkColormap *colormap;
+  GdkColormap *system;
   GdkColormapPrivateFB *private;
   GdkFBDisplay *fbd;
   int i;
@@ -140,6 +142,9 @@ gdk_colormap_new (GdkVisual *visual,
       
       private->hash = g_hash_table_new ((GHashFunc) gdk_color_hash,
 					(GEqualFunc) gdk_color_equal);
+
+      system = gdk_colormap_get_system ();
+      memcpy (colormap->colors, system->colors, colormap->size * sizeof (GdkColor));
       
       if (private_cmap)
 	{
@@ -150,7 +155,7 @@ gdk_colormap_new (GdkVisual *visual,
 	  fbc.green = green;
 	  fbc.blue = blue;
 
-	  if (ioctl (fbd->fd, FBIOGETCMAP, &fbc))
+	  if (ioctl (fbd->fb_fd, FBIOGETCMAP, &fbc))
 	    g_error("ioctl(FBIOGETCMAP) failed");
 
 	  for (i = 0; i < colormap->size; i++)
@@ -198,7 +203,7 @@ gdk_colormap_new (GdkVisual *visual,
 
 #define MIN_SYNC_TIME 2
 
-void
+static void
 gdk_colormap_sync (GdkColormap *colormap,
 		   gboolean     force)
 {
@@ -212,24 +217,44 @@ gdk_colormap_get_system (void)
 
   if (!colormap)
     {
-      guint16 red[256], green[256], blue[256];
-      struct fb_cmap fbc = {0, 256};
-      int i, r, g, b;
+      GdkColormapPrivateFB *private;
       GdkVisual *visual = gdk_visual_get_system ();
+      int i, r, g, b;
+      
+      colormap = g_object_new (gdk_colormap_get_type (), NULL);
+      private = GDK_COLORMAP_PRIVATE_DATA (colormap);
 
-      if (visual->type == GDK_VISUAL_GRAYSCALE
-	 || visual->type ==  GDK_VISUAL_PSEUDO_COLOR)
+      colormap->visual = visual;
+      private->hash = NULL;
+      
+      colormap->size = visual->colormap_size;
+      colormap->colors = NULL;
+      
+      switch (visual->type)
 	{
-	  fbc.red = red;
-	  fbc.green = green;
-	  fbc.blue = blue;
+	case GDK_VISUAL_STATIC_GRAY:
+	case GDK_VISUAL_STATIC_COLOR:
+	case GDK_VISUAL_GRAYSCALE:
+	case GDK_VISUAL_PSEUDO_COLOR:
+	  private->info = g_new0 (GdkColorInfo, colormap->size);
+	  colormap->colors = g_new (GdkColor, colormap->size);
+	  
+	  private->hash = g_hash_table_new ((GHashFunc) gdk_color_hash,
+					    (GEqualFunc) gdk_color_equal);
 	  switch(visual->type)
 	    {
 	    case GDK_VISUAL_GRAYSCALE:
-	      for(i = 0; i < 256; i++)
-		red[i] = green[i] = blue[i] = i << 8;
+	      for(i = 0; i < 256; i++) {
+		colormap->colors[i].red = 
+		  colormap->colors[i].green =
+		  colormap->colors[i].blue = i << 8;
+		gdk_fb_color_round_to_hw (&colormap->colors[i]);
+	      }
 	      i--;
-	      red[i] = green[i] = blue[i] = 65535; /* Make it a true white */
+	      colormap->colors[i].red = 
+		colormap->colors[i].green =
+		colormap->colors[i].blue = 65535; /* Make it a true white */
+	      gdk_fb_color_round_to_hw (&colormap->colors[i]);
 	      break;
 	    case GDK_VISUAL_PSEUDO_COLOR:
 	      /* Color cube stolen from gdkrgb upon advice from Owen */
@@ -237,30 +262,58 @@ gdk_colormap_get_system (void)
 		for(g = 0; g < 6; g++)
 		  for(b = 0; b < 6; b++)
 		    {
-		      red[i] = r * 65535 / 5;
-		      green[i] = g * 65535 / 5;
-		      blue[i] = b * 65535 / 5;
+		      colormap->colors[i].red = r * 65535 / 5;
+		      colormap->colors[i].green = g * 65535 / 5;
+		      colormap->colors[i].blue = b * 65535 / 5;
+		      gdk_fb_color_round_to_hw (&colormap->colors[i]);
 		      i++;
 		    }
 	      g_assert (i == 216);
-
-	  /* Fill in remaining space with grays */
+	      /* Fill in remaining space with grays */
 	      for(i = 216; i < 256; i++)
 		{
-		  red[i] = green[i] = blue[i] =
-		    (i - 216) * 40;
+		  colormap->colors[i].red = 
+		    colormap->colors[i].green =
+		    colormap->colors[i].blue = (i - 216) * 40;
+		  gdk_fb_color_round_to_hw (&colormap->colors[i]);
 		}
 	      /* Real white */
-	      red[255] = green[255] = blue[255] = 65535;
+	      colormap->colors[255].red = 
+		  colormap->colors[255].green =
+		  colormap->colors[255].blue = 65535;
+	      gdk_fb_color_round_to_hw (&colormap->colors[255]);
+
 	      break;
 	    default:
 	      break;
 	    }
-
-	  ioctl (gdk_display->fd, FBIOPUTCMAP, &fbc);
+	  break;
+	case GDK_VISUAL_DIRECT_COLOR:
+	  g_error("NYI");
+	  break;
+	default:
+	  g_assert_not_reached ();
+	case GDK_VISUAL_TRUE_COLOR:
+	  break;
 	}
-
-      colormap = gdk_colormap_new (visual, TRUE);
+      
+      /* Lock all colors for the system colormap
+       * on pseudocolor visuals. The AA text rendering
+       * takes to many colors otherwise.
+       */
+      if ((visual->type == GDK_VISUAL_GRAYSCALE) ||
+	  (visual->type == GDK_VISUAL_PSEUDO_COLOR))
+	{
+	  for(i = 0; i < 256; i++)
+	    {
+	      colormap->colors[i].pixel = i;
+	      private->info[i].ref_count = 1;
+	      g_hash_table_insert (private->hash,
+				   &colormap->colors[i],
+				   &colormap->colors[i]);
+	    }
+	}
+      gdk_colormap_change (colormap, colormap->size);
     }
 
   return colormap;
@@ -298,7 +351,7 @@ gdk_colormap_change (GdkColormap *colormap,
 	     colormap->colors[i].green +
 	     colormap->colors[i].blue)/3;
 	}
-      ioctl (gdk_display->fd, FBIOPUTCMAP, &fbc);
+      ioctl (gdk_display->fb_fd, FBIOPUTCMAP, &fbc);
       break;
 
     case GDK_VISUAL_PSEUDO_COLOR:
@@ -308,7 +361,7 @@ gdk_colormap_change (GdkColormap *colormap,
 	  green[i] = colormap->colors[i].green;
 	  blue[i] = colormap->colors[i].blue;
 	}
-      ioctl (gdk_display->fd, FBIOPUTCMAP, &fbc);
+      ioctl (gdk_display->fb_fd, FBIOPUTCMAP, &fbc);
       break;
 
     default:
@@ -421,7 +474,7 @@ gdk_colormap_free_colors (GdkColormap *colormap,
       (colormap->visual->type != GDK_VISUAL_GRAYSCALE))
     return;
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       gulong pixel = colors[i].pixel;
       
@@ -443,6 +496,23 @@ gdk_colormap_free_colors (GdkColormap *colormap,
  * Color allocation *
  ********************/
 
+static void
+gdk_fb_color_round_to_hw (GdkColor *color)
+{
+  guint rmask, gmask, bmask, len;
+
+  len = gdk_display->modeinfo.red.length;
+  rmask = ((1 << len) - 1) << (16-len);
+  len = gdk_display->modeinfo.green.length;
+  gmask = ((1 << len) - 1) << (16-len);
+  len = gdk_display->modeinfo.blue.length;
+  bmask = ((1 << len) - 1) << (16-len);
+  
+  color->red &=rmask;
+  color->green &=gmask;
+  color->blue &=bmask;
+}
+
 /* Try to allocate a single color using XAllocColor. If it succeeds,
  * cache the result in our colormap, and store in ret.
  */
@@ -461,23 +531,30 @@ gdk_colormap_alloc1 (GdkColormap *colormap,
     return FALSE;
 
   *ret = *color;
-  if (!color->red && !color->green && !color->blue) /* black */
+  
+  gdk_fb_color_round_to_hw (ret);
+
+  for (i = 0; i<colormap->size; i++)
     {
-      ret->pixel = 0;
-      private->info[ret->pixel].ref_count++;
-      return TRUE;
+      if (!(private->info[i].flags && GDK_COLOR_WRITEABLE) &&
+	  (ret->red == colormap->colors[i].red) &&
+	  (ret->green == colormap->colors[i].green) &&
+	  (ret->blue == colormap->colors[i].blue))
+	{
+	  ret->pixel = i;
+	  colormap->colors[i].pixel = i;
+	  if (private->info[i].ref_count == 0)
+	    g_hash_table_insert (private->hash,
+				 &colormap->colors[ret->pixel],
+				 &colormap->colors[ret->pixel]);
+	  private->info[i].ref_count++;
+	  return TRUE;
+	}
     }
 
-  if (color->red == 65535 && color->green == 65535 && color->blue == 65535) /* white */
+  for (i = 0; i<colormap->size; i++)
     {
-      ret->pixel = 255;
-      private->info[ret->pixel].ref_count++;
-      return TRUE;
-    }
-
-  for (i = 1; i < (colormap->size - 1); i++)
-    {
-      if (!private->info[i].ref_count)
+      if (private->info[i].ref_count==0)
 	{
 	  guint16 red = color->red, green = color->green, blue = color->blue;
 	  struct fb_cmap fbc;
@@ -488,7 +565,7 @@ gdk_colormap_alloc1 (GdkColormap *colormap,
 	  fbc.green = &green;
 	  fbc.blue = &blue;
 
-	  ioctl (gdk_display->fd, FBIOPUTCMAP, &fbc);
+	  ioctl (gdk_display->fb_fd, FBIOPUTCMAP, &fbc);
 
 	  ret->pixel = i;
 	  colormap->colors[ret->pixel] = *ret;
@@ -499,7 +576,7 @@ gdk_colormap_alloc1 (GdkColormap *colormap,
 	  return TRUE;
 	}
     }
-
+      
   return FALSE;
 }
 
@@ -519,7 +596,7 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
   index = -1;
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       if (!success[i])
 	{
@@ -541,7 +618,7 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
       
       while (nremaining > 0)
 	{
-	  for (i=0; i<ncolors; i++)
+	  for (i = 0; i < ncolors; i++)
 	    {
 	      if (!success[i])
 		{
@@ -586,7 +663,7 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
   /* Change back the values we flagged as permanent failures */
   if (nfailed > 0)
     {
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	if (success[i] == 2)
 	  success[i] = FALSE;
       nremaining = nfailed;
@@ -612,7 +689,7 @@ gdk_colormap_alloc_colors_pseudocolor (GdkColormap *colormap,
 
   /* Check for an exact match among previously allocated colors */
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     {
       if (!success[i])
 	{
@@ -655,7 +732,7 @@ gdk_colormap_alloc_colors (GdkColormap *colormap,
 
   private = GDK_COLORMAP_PRIVATE_DATA (colormap);
 
-  for (i=0; i<ncolors; i++)
+  for (i = 0; i < ncolors; i++)
     success[i] = FALSE;
 
   visual = colormap->visual;
@@ -671,7 +748,7 @@ gdk_colormap_alloc_colors (GdkColormap *colormap,
 
     case GDK_VISUAL_DIRECT_COLOR:
     case GDK_VISUAL_TRUE_COLOR:
-      for (i=0; i<ncolors; i++)
+      for (i = 0; i < ncolors; i++)
 	{
 	  colors[i].pixel = (((colors[i].red >> (16 - visual->red_prec)) << visual->red_shift) +
 			     ((colors[i].green >> (16 - visual->green_prec)) << visual->green_shift) +
@@ -700,12 +777,15 @@ gdk_color_change (GdkColormap *colormap,
     case GDK_VISUAL_GRAYSCALE:
       color->red = color->green = color->blue = (color->red + color->green + color->blue)/3;
 
+      /* Fall through */
     case GDK_VISUAL_PSEUDO_COLOR:
+      colormap->colors[color->pixel] = *color;
+      
       fbc.start = color->pixel;
       fbc.red = &color->red;
       fbc.green = &color->green;
       fbc.blue = &color->blue;
-      ioctl (gdk_display->fd, FBIOPUTCMAP, &fbc);
+      ioctl (gdk_display->fb_fd, FBIOPUTCMAP, &fbc);
       break;
 
     default:
@@ -753,7 +833,7 @@ gdk_colormap_match_color (GdkColormap *cmap,
   return index;
 }
 
-gint
+gboolean
 gdk_colors_alloc (GdkColormap	*colormap,
 		  gboolean 	 contiguous,
 		  gulong	*planes,
@@ -761,7 +841,45 @@ gdk_colors_alloc (GdkColormap	*colormap,
 		  gulong	*pixels,
 		  gint		 npixels)
 {
-  return 0;
+  GdkColormapPrivateFB *private;
+  gint found, i, col;
+
+  g_return_val_if_fail (colormap != NULL, FALSE);
+
+  private = GDK_COLORMAP_PRIVATE_DATA (colormap);
+  
+  if (nplanes > 0)
+    return 0;
+
+  found = 0;
+  for (i = 1; i < colormap->size; i++)
+    {
+      if (private->info[i].ref_count == 0)
+	{
+	  found++;
+	  if (found >= npixels)
+	    break;
+	}
+    }
+
+  if (found < npixels)
+    return FALSE;
+
+  col = 0;
+  for (i = 1; i < colormap->size; i++)
+    {
+      if (private->info[i].ref_count == 0)
+	{
+	  pixels[col++] = i;
+	  private->info[i].ref_count++;
+	  private->info[i].flags |= GDK_COLOR_WRITEABLE;
+	  if (col == npixels)
+	    return TRUE;
+	}
+    }
+
+  g_assert_not_reached ();
+  return FALSE;
 }
 
 void
@@ -770,4 +888,66 @@ gdk_colors_free	(GdkColormap	*colormap,
 		 gint		 npixels,
 		 gulong	 	 planes)
 {
+  GdkColormapPrivateFB *private;
+  gint i, pixel;
+
+  g_return_if_fail (colormap != NULL);
+
+  if ((colormap->visual->type != GDK_VISUAL_PSEUDO_COLOR) &&
+      (colormap->visual->type != GDK_VISUAL_GRAYSCALE))
+    return;
+  
+  private = GDK_COLORMAP_PRIVATE_DATA (colormap);
+  
+  for (i = 0; i < npixels; i++)
+    {
+      pixel = pixels[i];
+      
+      if (private->info[pixel].ref_count)
+	{
+	  private->info[pixel].ref_count--;
+
+	  if (private->info[pixel].ref_count == 0)
+	    {
+	      if (!(private->info[pixel].flags & GDK_COLOR_WRITEABLE))
+		g_hash_table_remove (private->hash, &colormap->colors[pixel]);
+	      private->info[pixel].flags = 0;
+	    }
+	}
+    }
+}
+
+
+void
+gdk_colormap_query_color (GdkColormap *colormap,
+			  gulong       pixel,
+			  GdkColor    *result)
+{
+  GdkVisual *visual;
+
+  g_return_if_fail (GDK_IS_COLORMAP (colormap));
+  
+  visual = gdk_colormap_get_visual (colormap);
+
+  switch (visual->type)
+    {
+    case GDK_VISUAL_DIRECT_COLOR:
+    case GDK_VISUAL_TRUE_COLOR:
+      result->red = 65535. * (gdouble) ((pixel & visual->red_mask) >> visual->red_shift) / ((1 << visual->red_prec) - 1);
+      result->green = 65535. * (gdouble) ((pixel & visual->green_mask) >> visual->green_shift) / ((1 << visual->green_prec) - 1);
+      result->blue = 65535. * (gdouble) ((pixel & visual->blue_mask) >> visual->blue_shift) / ((1 << visual->blue_prec) - 1);
+      break;
+    case GDK_VISUAL_STATIC_GRAY:
+    case GDK_VISUAL_GRAYSCALE:
+      result->red = result->green = result->blue = 65535. * (double)pixel/((1<<visual->depth) - 1);
+      break;
+    case GDK_VISUAL_PSEUDO_COLOR:
+      result->red = colormap->colors[pixel].red;
+      result->green = colormap->colors[pixel].green;
+      result->blue = colormap->colors[pixel].blue;
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+    }
 }
