@@ -2,23 +2,23 @@
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
@@ -31,13 +31,13 @@
 #include "gdkprivate-x11.h"
 #include "gdkinternals.h"
 
-static void  gdk_visual_add            (GdkVisual *visual);
-static void  gdk_visual_decompose_mask (gulong     mask,
-					gint      *shift,
-					gint      *prec);
-static guint gdk_visual_hash           (Visual    *key);
-static gint  gdk_visual_compare        (Visual    *a,
-					Visual    *b);
+static void     gdk_visual_add            (GdkVisual *visual);
+static void     gdk_visual_decompose_mask (gulong     mask,
+					   gint      *shift,
+					   gint      *prec);
+static guint    gdk_visual_hash           (Visual    *key);
+static gboolean gdk_visual_equal          (Visual    *a,
+					   Visual    *b);
 
 
 static GdkVisualPrivate *system_visual;
@@ -411,7 +411,7 @@ gdk_visual_add (GdkVisual *visual)
 
   if (!visual_hash)
     visual_hash = g_hash_table_new ((GHashFunc) gdk_visual_hash,
-				    (GCompareFunc) gdk_visual_compare);
+				    (GEqualFunc) gdk_visual_equal);
 
   private = (GdkVisualPrivate*) visual;
 
@@ -445,9 +445,9 @@ gdk_visual_hash (Visual *key)
   return key->visualid;
 }
 
-static gint
-gdk_visual_compare (Visual *a,
-		    Visual *b)
+static gboolean
+gdk_visual_equal (Visual *a,
+		  Visual *b)
 {
   return (a->visualid == b->visualid);
 }
