@@ -36,17 +36,12 @@
 #include <gdk/x11/gdkwindow-x11.h>
 #include <gdk/x11/gdkpixmap-x11.h>
 
-typedef struct _GdkGCXData             GdkGCXData;
 typedef struct _GdkColormapPrivateX11 GdkColormapPrivateX11;
 typedef struct _GdkCursorPrivate       GdkCursorPrivate;
 typedef struct _GdkFontPrivateX        GdkFontPrivateX;
 typedef struct _GdkImagePrivateX11    GdkImagePrivateX11;
 typedef struct _GdkVisualPrivate       GdkVisualPrivate;
 
-#ifdef USE_XIM
-typedef struct _GdkICPrivate        GdkICPrivate;
-#endif /* USE_XIM */
-
 struct _GdkCursorPrivate
 {
   GdkCursor cursor;
@@ -90,18 +85,6 @@ struct _GdkImagePrivateX11
 };
 
 
-#ifdef USE_XIM
-
-struct _GdkICPrivate
-{
-  XIC xic;
-  GdkICAttr *attr;
-  GdkICAttributesType mask;
-};
-
-#endif /* USE_XIM */
-
-
 typedef struct _GdkGCX11      GdkGCX11;
 typedef struct _GdkGCX11Class GdkGCX11Class;
 
@@ -120,6 +103,12 @@ struct _GdkGCX11
   Display *xdisplay;
   GdkRegion *clip_region;
   guint dirty_mask;
+
+  /* We can't conditionalize on HAVE_XFT here, so we simply always
+   * have this here as a gpointer.
+   */
+  gpointer xft_draw;
+  gulong fg_pixel;
 };
 
 struct _GdkGCX11Class
@@ -180,6 +169,9 @@ gpointer      gdk_xid_table_lookup     (XID              xid);
 
 guint32       gdk_x11_get_server_time  (GdkWindow       *window);
 
+/* returns TRUE if we support the given WM spec feature */
+gboolean      gdk_net_wm_supports      (GdkAtom property);
+
 #define gdk_window_lookup(xid)	   ((GdkWindow*) gdk_xid_table_lookup (xid))
 #define gdk_pixmap_lookup(xid)	   ((GdkPixmap*) gdk_xid_table_lookup (xid))
 #define gdk_font_lookup(xid)	   ((GdkFont*) gdk_xid_table_lookup (xid))
