@@ -58,7 +58,6 @@
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtkctree.h>
 #include <gtk/gtkcurve.h>
-#include <gtk/gtkdata.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkdrawingarea.h>
@@ -82,6 +81,7 @@
 #include <gtk/gtkhseparator.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkimcontext.h>
 #include <gtk/gtkimmulticontext.h>
 #include <gtk/gtkinputdialog.h>
@@ -96,12 +96,10 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenubar.h>
-#include <gtk/gtkmenufactory.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkmenushell.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkmisc.h>
-#include <gtk/gtkmodelsimple.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkobject.h>
 #include <gtk/gtkoldeditable.h>
@@ -123,6 +121,8 @@
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkseparator.h>
+#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtksettings.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtksocket.h>
 #include <gtk/gtkspinbutton.h>
@@ -139,6 +139,7 @@
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtktooltips.h>
 #include <gtk/gtktree.h>
+#include <gtk/gtktreednd.h>
 #include <gtk/gtktreeitem.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtktreemodelsort.h>
