@@ -18,21 +18,24 @@
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
 #include "gtkarrow.h"
-
+#include "gtkintl.h"
 
 #define MIN_ARROW_SIZE  11
 
 enum {
-  ARG_0,
-  ARG_ARROW_TYPE,
-  ARG_SHADOW_TYPE
+  PROP_0,
+
+  PROP_ARROW_TYPE,
+  PROP_SHADOW_TYPE,
+  
+  PROP_LAST
 };
 
 
@@ -40,13 +43,14 @@ static void gtk_arrow_class_init (GtkArrowClass  *klass);
 static void gtk_arrow_init       (GtkArrow       *arrow);
 static gint gtk_arrow_expose     (GtkWidget      *widget,
 				  GdkEventExpose *event);
-static void gtk_arrow_set_arg    (GtkObject      *object,
-				  GtkArg         *arg,
-				  guint           arg_id);
-static void gtk_arrow_get_arg    (GtkObject      *object,
-				  GtkArg         *arg,
-				  guint           arg_id);
-
+static void gtk_arrow_set_property (GObject         *object,
+				    guint            prop_id,
+				    const GValue    *value,
+				    GParamSpec      *pspec);
+static void gtk_arrow_get_property (GObject         *object,
+				    guint            prop_id,
+				    GValue          *value,
+				    GParamSpec      *pspec);
 
 GtkType
 gtk_arrow_get_type (void)
@@ -76,65 +80,85 @@ gtk_arrow_get_type (void)
 static void
 gtk_arrow_class_init (GtkArrowClass *class)
 {
+  GObjectClass *gobject_class;
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
+  gobject_class = (GObjectClass*) class;
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
 
-  gtk_object_add_arg_type ("GtkArrow::arrow_type", GTK_TYPE_ARROW_TYPE, GTK_ARG_READWRITE, ARG_ARROW_TYPE);
-  gtk_object_add_arg_type ("GtkArrow::shadow_type", GTK_TYPE_SHADOW_TYPE, GTK_ARG_READWRITE, ARG_SHADOW_TYPE);
-
-  object_class->set_arg = gtk_arrow_set_arg;
-  object_class->get_arg = gtk_arrow_get_arg;
-
+  gobject_class->set_property = gtk_arrow_set_property;
+  gobject_class->get_property = gtk_arrow_get_property;
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_ARROW_TYPE,
+                                   g_param_spec_enum ("arrow_type",
+                                                      _("Arrow direction"),
+                                                      _("The direction the arrow should point"),
+						      GTK_TYPE_ARROW_TYPE,
+						      GTK_ARROW_RIGHT,
+                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_SHADOW_TYPE,
+                                   g_param_spec_enum ("shadow_type",
+                                                      _("Arrow shadow"),
+                                                      _("Appearance of the shadow surrounding the arrow"),
+						      GTK_TYPE_SHADOW_TYPE,
+						      GTK_SHADOW_OUT,
+                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
   widget_class->expose_event = gtk_arrow_expose;
 }
 
 static void
-gtk_arrow_set_arg (GtkObject      *object,
-		   GtkArg         *arg,
-		   guint           arg_id)
+gtk_arrow_set_property (GObject         *object,
+			guint            prop_id,
+			const GValue    *value,
+			GParamSpec      *pspec)
 {
   GtkArrow *arrow;
-
+  
   arrow = GTK_ARROW (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_ARROW_TYPE:
+    case PROP_ARROW_TYPE:
       gtk_arrow_set (arrow,
-		     GTK_VALUE_ENUM (*arg),
+		     g_value_get_enum (value),
 		     arrow->shadow_type);
       break;
-    case ARG_SHADOW_TYPE:
+    case PROP_SHADOW_TYPE:
       gtk_arrow_set (arrow,
 		     arrow->arrow_type,
-		     GTK_VALUE_ENUM (*arg));
+		     g_value_get_enum (value));
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
+
 static void
-gtk_arrow_get_arg (GtkObject      *object,
-		   GtkArg         *arg,
-		   guint           arg_id)
+gtk_arrow_get_property (GObject         *object,
+			guint            prop_id,
+			GValue          *value,
+			GParamSpec      *pspec)
 {
   GtkArrow *arrow;
-
+  
   arrow = GTK_ARROW (object);
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_ARROW_TYPE:
-      GTK_VALUE_ENUM (*arg) = arrow->arrow_type;
+    case PROP_ARROW_TYPE:
+      g_value_set_enum (value, arrow->arrow_type);
       break;
-    case ARG_SHADOW_TYPE:
-      GTK_VALUE_ENUM (*arg) = arrow->shadow_type;
+    case PROP_SHADOW_TYPE:
+      g_value_set_enum (value, arrow->shadow_type);
       break;
     default:
-      arg->type = GTK_TYPE_INVALID;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -170,22 +194,34 @@ gtk_arrow_set (GtkArrow      *arrow,
 	       GtkArrowType   arrow_type,
 	       GtkShadowType  shadow_type)
 {
+  gboolean changed = FALSE;
+   
   g_return_if_fail (arrow != NULL);
   g_return_if_fail (GTK_IS_ARROW (arrow));
 
-  if (((GtkArrowType) arrow->arrow_type != arrow_type) ||
-      ((GtkShadowType) arrow->shadow_type != shadow_type))
+  if ((GtkArrowType) arrow->arrow_type != arrow_type)
     {
       arrow->arrow_type = arrow_type;
-      arrow->shadow_type = shadow_type;
+      g_object_notify (G_OBJECT (arrow), "arrow_type");
+      changed = TRUE;
+    }
 
+  if ((GtkShadowType) arrow->shadow_type != shadow_type)
+    {
+      arrow->shadow_type = shadow_type;
+      g_object_notify (G_OBJECT (arrow), "shadow_type");
+      changed = TRUE;
+    }
+
+  if (changed == TRUE)
+    {
       if (GTK_WIDGET_DRAWABLE (arrow))
 	gtk_widget_queue_clear (GTK_WIDGET (arrow));
     }
 }
 
 
-static gint
+static gboolean 
 gtk_arrow_expose (GtkWidget      *widget,
 		  GdkEventExpose *event)
 {
