@@ -18,7 +18,7 @@
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
@@ -33,12 +33,15 @@
 #include "gtkhbox.h"
 #include "gtkstock.h"
 #include "gtkiconfactory.h"
+#include "gtkintl.h"
 
 #define CHILD_SPACING     1
-#define DEFAULT_LEFT_POS  4
-#define DEFAULT_TOP_POS   4
 #define DEFAULT_SPACING   7
 
+/* Time out before giving up on getting a key release when animatng
+ * the close button.
+ */
+#define ACTIVATE_TIMEOUT 250
 
 enum {
   PRESSED,
@@ -46,50 +49,46 @@ enum {
   CLICKED,
   ENTER,
   LEAVE,
+  ACTIVATE,
   LAST_SIGNAL
 };
 
 enum {
-  ARG_0,
-  ARG_LABEL,
-  ARG_RELIEF
+  PROP_0,
+  PROP_LABEL,
+  PROP_RELIEF
 };
 
-
-
 static void gtk_button_class_init     (GtkButtonClass   *klass);
 static void gtk_button_init           (GtkButton        *button);
-static void gtk_button_set_arg        (GtkObject        *object,
-				       GtkArg           *arg,
-				       guint		 arg_id);
-static void gtk_button_get_arg        (GtkObject        *object,
-				       GtkArg           *arg,
-				       guint		 arg_id);
+static void gtk_button_set_property   (GObject         *object,
+                                       guint            prop_id,
+                                       const GValue    *value,
+                                       GParamSpec      *pspec);
+static void gtk_button_get_property   (GObject         *object,
+                                       guint            prop_id,
+                                       GValue          *value,
+                                       GParamSpec      *pspec);
 static void gtk_button_realize        (GtkWidget        *widget);
+static void gtk_button_unrealize      (GtkWidget        *widget);
 static void gtk_button_size_request   (GtkWidget        *widget,
 				       GtkRequisition   *requisition);
 static void gtk_button_size_allocate  (GtkWidget        *widget,
 				       GtkAllocation    *allocation);
 static void gtk_button_paint          (GtkWidget        *widget,
 				       GdkRectangle     *area);
-static void gtk_button_draw           (GtkWidget        *widget,
-				       GdkRectangle     *area);
-static void gtk_button_draw_focus     (GtkWidget        *widget);
-static void gtk_button_draw_default   (GtkWidget        *widget);
 static gint gtk_button_expose         (GtkWidget        *widget,
 				       GdkEventExpose   *event);
 static gint gtk_button_button_press   (GtkWidget        *widget,
 				       GdkEventButton   *event);
 static gint gtk_button_button_release (GtkWidget        *widget,
 				       GdkEventButton   *event);
+static gint gtk_button_key_release    (GtkWidget        *widget,
+				       GdkEventKey      *event);
 static gint gtk_button_enter_notify   (GtkWidget        *widget,
 				       GdkEventCrossing *event);
 static gint gtk_button_leave_notify   (GtkWidget        *widget,
 				       GdkEventCrossing *event);
-static gint gtk_button_focus_in       (GtkWidget        *widget,
-				       GdkEventFocus    *event);
-static gint gtk_button_focus_out      (GtkWidget        *widget,
-				       GdkEventFocus    *event);
 static void gtk_button_add            (GtkContainer     *container,
 				       GtkWidget        *widget);
 static void gtk_button_remove         (GtkContainer     *container,
@@ -98,8 +97,11 @@ static void gtk_real_button_pressed   (GtkButton        *button);
 static void gtk_real_button_released  (GtkButton        *button);
 static void gtk_real_button_enter     (GtkButton        *button);
 static void gtk_real_button_leave     (GtkButton        *button);
+static void gtk_real_button_activate (GtkButton         *button);
 static GtkType gtk_button_child_type  (GtkContainer     *container);
 
+static void gtk_button_finish_activate (GtkButton *button,
+					gboolean   do_it);
 
 static GtkBinClass *parent_class = NULL;
 static guint button_signals[LAST_SIGNAL] = { 0 };
@@ -141,11 +143,50 @@ gtk_button_class_init (GtkButtonClass *klass)
   object_class = (GtkObjectClass*) klass;
   widget_class = (GtkWidgetClass*) klass;
   container_class = (GtkContainerClass*) klass;
+  
+  parent_class = g_type_class_peek_parent (klass);
 
-  parent_class = gtk_type_class (GTK_TYPE_BIN);
+  G_OBJECT_CLASS(object_class)->set_property = gtk_button_set_property;
+  G_OBJECT_CLASS(object_class)->get_property = gtk_button_get_property;
 
-  gtk_object_add_arg_type ("GtkButton::label", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_LABEL);
-  gtk_object_add_arg_type ("GtkButton::relief", GTK_TYPE_RELIEF_STYLE, GTK_ARG_READWRITE, ARG_RELIEF);
+  widget_class->realize = gtk_button_realize;
+  widget_class->unrealize = gtk_button_unrealize;
+  widget_class->size_request = gtk_button_size_request;
+  widget_class->size_allocate = gtk_button_size_allocate;
+  widget_class->expose_event = gtk_button_expose;
+  widget_class->button_press_event = gtk_button_button_press;
+  widget_class->button_release_event = gtk_button_button_release;
+  widget_class->key_release_event = gtk_button_key_release;
+  widget_class->enter_notify_event = gtk_button_enter_notify;
+  widget_class->leave_notify_event = gtk_button_leave_notify;
+
+  container_class->add = gtk_button_add;
+  container_class->remove = gtk_button_remove;
+  container_class->child_type = gtk_button_child_type;
+
+  klass->pressed = gtk_real_button_pressed;
+  klass->released = gtk_real_button_released;
+  klass->clicked = NULL;
+  klass->enter = gtk_real_button_enter;
+  klass->leave = gtk_real_button_leave;
+  klass->activate = gtk_real_button_activate;
+
+  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+                                   PROP_LABEL,
+                                   g_param_spec_string ("label",
+                                                        _("Label"),
+                                                        _("Text of the label widget inside the button, if the button contains a label widget."),
+                                                        NULL,
+                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
+  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+                                   PROP_RELIEF,
+                                   g_param_spec_enum ("relief",
+                                                      _("Border relief"),
+                                                      _("The border relief style."),
+                                                      GTK_TYPE_RELIEF_STYLE,
+                                                      GTK_RELIEF_NORMAL,
+                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
 
   button_signals[PRESSED] =
     gtk_signal_new ("pressed",
@@ -182,36 +223,23 @@ gtk_button_class_init (GtkButtonClass *klass)
                     GTK_SIGNAL_OFFSET (GtkButtonClass, leave),
                     gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
+  button_signals[ACTIVATE] =
+    gtk_signal_new ("activate",
+                    GTK_RUN_FIRST,
+                    GTK_CLASS_TYPE (object_class),
+                    GTK_SIGNAL_OFFSET (GtkButtonClass, activate),
+                    gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
+  widget_class->activate_signal = button_signals[ACTIVATE];
 
-  gtk_object_class_add_signals (object_class, button_signals, LAST_SIGNAL);
-
-  object_class->set_arg = gtk_button_set_arg;
-  object_class->get_arg = gtk_button_get_arg;
-
-  widget_class->activate_signal = button_signals[CLICKED];
-  widget_class->realize = gtk_button_realize;
-  widget_class->draw = gtk_button_draw;
-  widget_class->draw_focus = gtk_button_draw_focus;
-  widget_class->draw_default = gtk_button_draw_default;
-  widget_class->size_request = gtk_button_size_request;
-  widget_class->size_allocate = gtk_button_size_allocate;
-  widget_class->expose_event = gtk_button_expose;
-  widget_class->button_press_event = gtk_button_button_press;
-  widget_class->button_release_event = gtk_button_button_release;
-  widget_class->enter_notify_event = gtk_button_enter_notify;
-  widget_class->leave_notify_event = gtk_button_leave_notify;
-  widget_class->focus_in_event = gtk_button_focus_in;
-  widget_class->focus_out_event = gtk_button_focus_out;
-
-  container_class->add = gtk_button_add;
-  container_class->remove = gtk_button_remove;
-  container_class->child_type = gtk_button_child_type;
-
-  klass->pressed = gtk_real_button_pressed;
-  klass->released = gtk_real_button_released;
-  klass->clicked = NULL;
-  klass->enter = gtk_real_button_enter;
-  klass->leave = gtk_real_button_leave;
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("default_spacing",
+							     _("Default Spacing"),
+							     _("Extra space to add for CAN_DEFAULT buttons"),
+							     0,
+							     G_MAXINT,
+							     DEFAULT_SPACING,
+							     G_PARAM_READABLE));
 }
 
 static void
@@ -236,19 +264,20 @@ gtk_button_child_type  (GtkContainer     *container)
 }
 
 static void
-gtk_button_set_arg (GtkObject *object,
-		    GtkArg    *arg,
-		    guint      arg_id)
+gtk_button_set_property (GObject         *object,
+                         guint            prop_id,
+                         const GValue    *value,
+                         GParamSpec      *pspec)
 {
   GtkButton *button;
 
   button = GTK_BUTTON (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
       GtkWidget *child;
 
-    case ARG_LABEL:
+    case PROP_LABEL:
       child = GTK_BIN (button)->child;
       if (!child)
 	child = gtk_widget_new (GTK_TYPE_LABEL,
@@ -256,39 +285,43 @@ gtk_button_set_arg (GtkObject *object,
 				"parent", button,
 				NULL);
       if (GTK_IS_LABEL (child))
-	gtk_label_set_text (GTK_LABEL (child),
-		       GTK_VALUE_STRING (*arg) ? GTK_VALUE_STRING (*arg) : "");
+        {
+          gtk_label_set_text (GTK_LABEL (child),
+                              g_value_get_string (value) ? g_value_get_string (value) : "");
+        }
       break;
-    case ARG_RELIEF:
-      gtk_button_set_relief (button, GTK_VALUE_ENUM (*arg));
+    case PROP_RELIEF:
+      gtk_button_set_relief (button, g_value_get_enum (value));
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
 static void
-gtk_button_get_arg (GtkObject *object,
-		    GtkArg    *arg,
-		    guint      arg_id)
+gtk_button_get_property (GObject         *object,
+                         guint            prop_id,
+                         GValue          *value,
+                         GParamSpec      *pspec)
 {
   GtkButton *button;
 
   button = GTK_BUTTON (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_LABEL:
+    case PROP_LABEL:
       if (GTK_BIN (button)->child && GTK_IS_LABEL (GTK_BIN (button)->child))
-	GTK_VALUE_STRING (*arg) = g_strdup (GTK_LABEL (GTK_BIN (button)->child)->label);
+	 g_value_set_string (value, GTK_LABEL (GTK_BIN (button)->child)->label); 
       else
-	GTK_VALUE_STRING (*arg) = NULL;
+	 g_value_set_string (value, NULL);
       break;
-    case ARG_RELIEF:
-      GTK_VALUE_ENUM (*arg) = gtk_button_get_relief (button);
+    case PROP_RELIEF:
+      g_value_set_enum (value, gtk_button_get_relief (button));
       break;
     default:
-      arg->type = GTK_TYPE_INVALID;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -315,9 +348,17 @@ gtk_button_new_with_label (const gchar *label)
   return button;
 }
 
+/**
+ * gtk_button_new_from_stock:
+ * @stock_id: the name of the stock item 
+ * @returns: a new #GtkButton
+ *
+ * Creates a new #GtkButton containing the image and text from a stock item.
+ * Some stock ids have preprocessor macros like #GTK_STOCK_BUTTON_OK and
+ * #GTK_STOCK_BUTTON_APPLY.
+ **/
 GtkWidget*
-gtk_button_new_stock (const gchar   *stock_id,
-                      GtkAccelGroup *accel_group)
+gtk_button_new_from_stock (const gchar   *stock_id)
 {
   GtkWidget *button;
   GtkStockItem item;
@@ -327,34 +368,12 @@ gtk_button_new_stock (const gchar   *stock_id,
       GtkWidget *label;
       GtkWidget *image;
       GtkWidget *hbox;
-      guint keyval;
       
       button = gtk_button_new ();
 
-      label = gtk_label_new (NULL);
-      keyval = gtk_label_parse_uline (GTK_LABEL (label),
-                                      item.label);
+      label = gtk_label_new_with_mnemonic (item.label);
 
-      if (keyval && accel_group)
-        {
-          gtk_widget_add_accelerator (button,
-                                      "clicked",
-                                      accel_group,
-                                      keyval,
-                                      GDK_MOD1_MASK,
-                                      GTK_ACCEL_LOCKED);
-        }
-
-      /* Also add the stock accelerator if one was specified. */
-      if (item.keyval && accel_group)
-        {
-          gtk_widget_add_accelerator (button,
-                                      "clicked",
-                                      accel_group,
-                                      item.keyval,
-                                      item.modifier,
-                                      GTK_ACCEL_LOCKED);
-        }
+      gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
 
       image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
       hbox = gtk_hbox_new (FALSE, 0);
@@ -367,37 +386,37 @@ gtk_button_new_stock (const gchar   *stock_id,
     }
   else
     {
-      button = gtk_button_new_accel (stock_id, accel_group);
+      button = gtk_button_new_with_mnemonic (stock_id);
     }
   
   return button;
 }
 
+/**
+ * gtk_button_new_with_mnemonic:
+ * @label: The text of the button, with an underscore in front of the
+ *         mnemonic character
+ * @returns: a new #GtkButton
+ *
+ * Creates a new #GtkButton containing a label.
+ * If characters in @label are preceded by an underscore, they are underlined
+ * indicating that they represent a keyboard accelerator called a mnemonic.
+ * Pressing Alt and that key activates the button.
+ **/
 GtkWidget*
-gtk_button_new_accel (const gchar   *uline_label,
-                      GtkAccelGroup *accel_group)
+gtk_button_new_with_mnemonic (const gchar *label)
 {
   GtkWidget *button;
-  GtkWidget *label;
-  guint keyval;
+  GtkWidget *label_widget;
 
   button = gtk_button_new ();
   
-  label = gtk_label_new (NULL);
-  keyval = gtk_label_parse_uline (GTK_LABEL (label), uline_label);
+  label_widget = gtk_label_new_with_mnemonic (label);
 
-  if (keyval && accel_group)
-    {
-      gtk_widget_add_accelerator (button,
-                                  "clicked",
-                                  accel_group,
-                                  keyval,
-                                  GDK_MOD1_MASK,
-                                  GTK_ACCEL_LOCKED);
-    }
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label_widget), button);
   
-  gtk_container_add (GTK_CONTAINER (button), label);
-  gtk_widget_show (label);
+  gtk_container_add (GTK_CONTAINER (button), label_widget);
+  gtk_widget_show (label_widget);
 
   return button;
 }
@@ -455,6 +474,7 @@ gtk_button_set_relief (GtkButton *button,
   g_return_if_fail (GTK_IS_BUTTON (button));
 
   button->relief = newrelief;
+  g_object_notify(G_OBJECT(button), "relief");
   gtk_widget_queue_draw (GTK_WIDGET (button));
 }
 
@@ -507,18 +527,41 @@ gtk_button_realize (GtkWidget *widget)
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 }
 
+static void
+gtk_button_unrealize (GtkWidget *widget)
+{
+  GtkButton *button = GTK_BUTTON (widget);
+
+  if (button->activate_timeout)
+    gtk_button_finish_activate (button, FALSE);
+    
+  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
+static void
+gtk_button_get_props (GtkButton *button,
+		      gint      *default_spacing,
+		      gboolean  *interior_focus)
+{
+  GtkWidget *widget =  GTK_WIDGET (button);
+
+  if (default_spacing)
+    gtk_widget_style_get (widget, "default_spacing", default_spacing, NULL);
+
+  if (interior_focus)
+    gtk_widget_style_get (widget, "interior_focus", interior_focus, NULL);
+}
+	
 static void
 gtk_button_size_request (GtkWidget      *widget,
 			 GtkRequisition *requisition)
 {
-  GtkButton *button;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_BUTTON (widget));
-  g_return_if_fail (requisition != NULL);
-
-  button = GTK_BUTTON (widget);
+  GtkButton *button = GTK_BUTTON (widget);
+  gint default_spacing;
+  gboolean interior_focus;
 
+  gtk_button_get_props (button, &default_spacing, &interior_focus);
+  
   requisition->width = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
 			GTK_WIDGET (widget)->style->xthickness) * 2;
   requisition->height = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
@@ -527,9 +570,9 @@ gtk_button_size_request (GtkWidget      *widget,
   if (GTK_WIDGET_CAN_DEFAULT (widget))
     {
       requisition->width += (GTK_WIDGET (widget)->style->xthickness * 2 +
-			     DEFAULT_SPACING);
+			     default_spacing);
       requisition->height += (GTK_WIDGET (widget)->style->ythickness * 2 +
-			      DEFAULT_SPACING);
+			      default_spacing);
     }
 
   if (GTK_BIN (button)->child && GTK_WIDGET_VISIBLE (GTK_BIN (button)->child))
@@ -541,22 +584,29 @@ gtk_button_size_request (GtkWidget      *widget,
       requisition->width += child_requisition.width;
       requisition->height += child_requisition.height;
     }
+
+  if (interior_focus)
+    {
+      requisition->width += 2;
+      requisition->height += 2;
+    }
 }
 
 static void
 gtk_button_size_allocate (GtkWidget     *widget,
 			  GtkAllocation *allocation)
 {
-  GtkButton *button;
+  GtkButton *button = GTK_BUTTON (widget);
   GtkAllocation child_allocation;
-  gint border_width;
 
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_BUTTON (widget));
-  g_return_if_fail (allocation != NULL);
+  gint border_width = GTK_CONTAINER (widget)->border_width;
+  gint xthickness = GTK_WIDGET (widget)->style->xthickness;
+  gint ythickness = GTK_WIDGET (widget)->style->ythickness;
+  gint default_spacing;
 
+  gtk_button_get_props (button, &default_spacing, NULL);
+  
   widget->allocation = *allocation;
-  border_width = GTK_CONTAINER (widget)->border_width;
 
   if (GTK_WIDGET_REALIZED (widget))
     gdk_window_move_resize (widget->window,
@@ -565,12 +615,10 @@ gtk_button_size_allocate (GtkWidget     *widget,
 			    widget->allocation.width - border_width * 2,
 			    widget->allocation.height - border_width * 2);
 
-  button = GTK_BUTTON (widget);
-
   if (GTK_BIN (button)->child && GTK_WIDGET_VISIBLE (GTK_BIN (button)->child))
     {
-      child_allocation.x = (CHILD_SPACING + GTK_WIDGET (widget)->style->xthickness);
-      child_allocation.y = (CHILD_SPACING + GTK_WIDGET (widget)->style->ythickness);
+      child_allocation.x = (CHILD_SPACING + xthickness);
+      child_allocation.y = (CHILD_SPACING + ythickness);
 
       child_allocation.width = MAX (1, (gint)widget->allocation.width - child_allocation.x * 2 -
 	                         border_width * 2);
@@ -580,13 +628,13 @@ gtk_button_size_allocate (GtkWidget     *widget,
       if (GTK_WIDGET_CAN_DEFAULT (button))
 	{
 	  child_allocation.x += (GTK_WIDGET (widget)->style->xthickness +
-				 DEFAULT_LEFT_POS);
+				 (1 + default_spacing) / 2);
 	  child_allocation.y += (GTK_WIDGET (widget)->style->ythickness +
-				 DEFAULT_TOP_POS);
+				 (1 + default_spacing) / 2);
 	  child_allocation.width =  MAX (1, (gint)child_allocation.width -
-					 (gint)(GTK_WIDGET (widget)->style->xthickness * 2 + DEFAULT_SPACING));
+					 (gint)(GTK_WIDGET (widget)->style->xthickness * 2 + default_spacing));
 	  child_allocation.height = MAX (1, (gint)child_allocation.height -
-					 (gint)(GTK_WIDGET (widget)->style->xthickness * 2 + DEFAULT_SPACING));
+					 (gint)(GTK_WIDGET (widget)->style->xthickness * 2 + default_spacing));
 	}
 
       gtk_widget_size_allocate (GTK_BIN (button)->child, &child_allocation);
@@ -627,10 +675,14 @@ gtk_button_paint (GtkWidget    *widget,
   GtkShadowType shadow_type;
   gint width, height;
   gint x, y;
+  gint default_spacing;
+  gboolean interior_focus;
    
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       button = GTK_BUTTON (widget);
+
+      gtk_button_get_props (button, &default_spacing, &interior_focus);
 	
       x = 0;
       y = 0;
@@ -653,13 +705,13 @@ gtk_button_paint (GtkWidget    *widget,
 	{
 	  x += widget->style->xthickness;
 	  y += widget->style->ythickness;
-	  width -= 2 * x + DEFAULT_SPACING;
-	  height -= 2 * y + DEFAULT_SPACING;
-	  x += DEFAULT_LEFT_POS;
-	  y += DEFAULT_TOP_POS;
+	  width -= 2 * x + default_spacing;
+	  height -= 2 * y + default_spacing;
+ 	  x += (1 + default_spacing) / 2;
+ 	  y += (1 + default_spacing) / 2;
 	}
        
-      if (GTK_WIDGET_HAS_FOCUS (widget))
+      if (!interior_focus && GTK_WIDGET_HAS_FOCUS (widget))
 	{
 	  x += 1;
 	  y += 1;
@@ -682,10 +734,20 @@ gtk_button_paint (GtkWidget    *widget,
        
       if (GTK_WIDGET_HAS_FOCUS (widget))
 	{
-	  x -= 1;
-	  y -= 1;
-	  width += 2;
-	  height += 2;
+	  if (interior_focus)
+	    {
+	      x += widget->style->xthickness + 1;
+	      y += widget->style->ythickness + 1;
+	      width -= 2 * (widget->style->xthickness + 1);
+	      height -=  2 * (widget->style->xthickness + 1);
+	    }
+	  else
+	    {
+	      x -= 1;
+	      y -= 1;
+	      width += 2;
+	      height += 2;
+	    }
 
 	  gtk_paint_focus (widget->style, widget->window,
 			   area, widget, "button",
@@ -694,51 +756,11 @@ gtk_button_paint (GtkWidget    *widget,
     }
 }
 
-static void
-gtk_button_draw (GtkWidget    *widget,
-		 GdkRectangle *area)
-{
-  GtkButton *button;
-  GdkRectangle child_area;
-  GdkRectangle tmp_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_BUTTON (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      button = GTK_BUTTON (widget);
-
-      tmp_area = *area;
-      tmp_area.x -= GTK_CONTAINER (button)->border_width;
-      tmp_area.y -= GTK_CONTAINER (button)->border_width;
-
-      gtk_button_paint (widget, &tmp_area);
-
-      if (GTK_BIN (button)->child && gtk_widget_intersect (GTK_BIN (button)->child, &tmp_area, &child_area))
-	gtk_widget_draw (GTK_BIN (button)->child, &child_area);
-    }
-}
-
-static void
-gtk_button_draw_focus (GtkWidget *widget)
-{
-  gtk_widget_draw (widget, NULL);
-}
-
-static void
-gtk_button_draw_default (GtkWidget *widget)
-{
-  gtk_widget_draw (widget, NULL);
-}
-
-static gint
+static gboolean
 gtk_button_expose (GtkWidget      *widget,
 		   GdkEventExpose *event)
 {
   GtkBin *bin;
-  GdkEventExpose child_event;
 
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
@@ -749,17 +771,14 @@ gtk_button_expose (GtkWidget      *widget,
       bin = GTK_BIN (widget);
       
       gtk_button_paint (widget, &event->area);
-
-      child_event = *event;
-      if (bin->child && GTK_WIDGET_NO_WINDOW (bin->child) &&
-	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
+      
+      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
     }
-
+  
   return FALSE;
 }
 
-static gint
+static gboolean
 gtk_button_button_press (GtkWidget      *widget,
 			 GdkEventButton *event)
 {
@@ -777,16 +796,13 @@ gtk_button_button_press (GtkWidget      *widget,
 	gtk_widget_grab_focus (widget);
 
       if (event->button == 1)
-	{
-	  gtk_grab_add (GTK_WIDGET (button));
-	  gtk_button_pressed (button);
-	}
+	gtk_button_pressed (button);
     }
 
   return TRUE;
 }
 
-static gint
+static gboolean
 gtk_button_button_release (GtkWidget      *widget,
 			   GdkEventButton *event)
 {
@@ -799,14 +815,30 @@ gtk_button_button_release (GtkWidget      *widget,
   if (event->button == 1)
     {
       button = GTK_BUTTON (widget);
-      gtk_grab_remove (GTK_WIDGET (button));
       gtk_button_released (button);
     }
 
   return TRUE;
 }
 
-static gint
+static gboolean
+gtk_button_key_release (GtkWidget   *widget,
+			GdkEventKey *event)
+{
+  GtkButton *button = GTK_BUTTON (widget);
+
+  if (button->activate_timeout)
+    {
+      gtk_button_finish_activate (button, TRUE);
+      return TRUE;
+    }
+  else if (GTK_WIDGET_CLASS (parent_class)->key_release_event)
+    return GTK_WIDGET_CLASS (parent_class)->key_release_event (widget, event);
+  else
+    return FALSE;
+}
+
+static gboolean
 gtk_button_enter_notify (GtkWidget        *widget,
 			 GdkEventCrossing *event)
 {
@@ -830,7 +862,7 @@ gtk_button_enter_notify (GtkWidget        *widget,
   return FALSE;
 }
 
-static gint
+static gboolean
 gtk_button_leave_notify (GtkWidget        *widget,
 			 GdkEventCrossing *event)
 {
@@ -854,34 +886,6 @@ gtk_button_leave_notify (GtkWidget        *widget,
   return FALSE;
 }
 
-static gint
-gtk_button_focus_in (GtkWidget     *widget,
-		     GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-  return FALSE;
-}
-
-static gint
-gtk_button_focus_out (GtkWidget     *widget,
-		      GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_BUTTON (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-  return FALSE;
-}
-
 static void
 gtk_button_add (GtkContainer *container,
 		GtkWidget    *widget)
@@ -916,6 +920,9 @@ gtk_real_button_pressed (GtkButton *button)
   g_return_if_fail (button != NULL);
   g_return_if_fail (GTK_IS_BUTTON (button));
 
+  if (button->activate_timeout)
+    return;
+  
   button->button_down = TRUE;
 
   new_state = (button->in_button ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL);
@@ -939,6 +946,9 @@ gtk_real_button_released (GtkButton *button)
     {
       button->button_down = FALSE;
 
+      if (button->activate_timeout)
+	return;
+  
       if (button->in_button)
 	gtk_button_clicked (button);
 
@@ -965,6 +975,9 @@ gtk_real_button_enter (GtkButton *button)
 
   new_state = (button->button_down ? GTK_STATE_ACTIVE : GTK_STATE_PRELIGHT);
 
+  if (button->activate_timeout)
+    return;
+  
   if (GTK_WIDGET_STATE (button) != new_state)
     {
       gtk_widget_set_state (GTK_WIDGET (button), new_state);
@@ -977,10 +990,66 @@ gtk_real_button_leave (GtkButton *button)
 {
   g_return_if_fail (button != NULL);
   g_return_if_fail (GTK_IS_BUTTON (button));
-
+  
+  if (button->activate_timeout)
+    return;
+  
   if (GTK_WIDGET_STATE (button) != GTK_STATE_NORMAL)
     {
       gtk_widget_set_state (GTK_WIDGET (button), GTK_STATE_NORMAL);
       gtk_widget_queue_draw (GTK_WIDGET (button));
     }
 }
+
+static gboolean
+button_activate_timeout (gpointer data)
+{
+  gtk_button_finish_activate (data, TRUE);
+
+  return FALSE;
+}
+
+static void
+gtk_real_button_activate (GtkButton *button)
+{
+  GtkWidget *widget = GTK_WIDGET (button);
+  
+  g_return_if_fail (button != NULL);
+  g_return_if_fail (GTK_IS_BUTTON (button));
+
+  if (GTK_WIDGET_REALIZED (button) && !button->activate_timeout)
+    {
+      if (gdk_keyboard_grab (widget->window, TRUE,
+			     gtk_get_current_event_time ()) == 0)
+	{
+	  gtk_grab_add (widget);
+	  
+	  button->activate_timeout = g_timeout_add (ACTIVATE_TIMEOUT,
+						    button_activate_timeout,
+						    button);
+	  button->button_down = TRUE;
+	  gtk_widget_set_state (widget, GTK_STATE_ACTIVE);
+	}
+    }
+}
+
+static void
+gtk_button_finish_activate (GtkButton *button,
+			    gboolean   do_it)
+{
+  GtkWidget *widget = GTK_WIDGET (button);
+  
+  g_source_remove (button->activate_timeout);
+  button->activate_timeout = 0;
+
+  gdk_keyboard_ungrab (gtk_get_current_event_time ());
+  gtk_grab_remove (widget);
+
+  button->button_down = FALSE;
+  gtk_widget_set_state (GTK_WIDGET (button),
+			button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);
+
+  if (do_it)
+    gtk_button_clicked (button);
+}
+
