@@ -18,7 +18,7 @@
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
@@ -56,6 +56,8 @@ struct _GtkButton
 		    * use GTK_BIN (button)->child instead
 		    */;
 
+  guint activate_timeout;
+
   guint in_button : 1;
   guint button_down : 1;
   guint relief : 2;
@@ -70,24 +72,23 @@ struct _GtkButtonClass
   void (* clicked)  (GtkButton *button);
   void (* enter)    (GtkButton *button);
   void (* leave)    (GtkButton *button);
+  void (* activate) (GtkButton *button);
 };
 
 
 GtkType        gtk_button_get_type       (void) G_GNUC_CONST;
-GtkWidget*     gtk_button_new            (void);
-GtkWidget*     gtk_button_new_with_label (const gchar   *label);
-GtkWidget*     gtk_button_new_stock      (const gchar   *stock_id,
-                                          GtkAccelGroup *accel_group);
-GtkWidget*     gtk_button_new_accel      (const gchar   *uline_label,
-                                          GtkAccelGroup *accel_group);
-void           gtk_button_pressed        (GtkButton *button);
-void           gtk_button_released       (GtkButton *button);
-void           gtk_button_clicked        (GtkButton *button);
-void           gtk_button_enter          (GtkButton *button);
-void           gtk_button_leave          (GtkButton *button);
-void           gtk_button_set_relief     (GtkButton *button,
-					  GtkReliefStyle newstyle);
-GtkReliefStyle gtk_button_get_relief      (GtkButton *button);
+GtkWidget*     gtk_button_new               (void);
+GtkWidget*     gtk_button_new_with_label    (const gchar    *label);
+GtkWidget*     gtk_button_new_from_stock    (const gchar    *stock_id);
+GtkWidget*     gtk_button_new_with_mnemonic (const gchar    *label);
+void           gtk_button_pressed           (GtkButton      *button);
+void           gtk_button_released          (GtkButton      *button);
+void           gtk_button_clicked           (GtkButton      *button);
+void           gtk_button_enter             (GtkButton      *button);
+void           gtk_button_leave             (GtkButton      *button);
+void           gtk_button_set_relief        (GtkButton      *button,
+					     GtkReliefStyle  newstyle);
+GtkReliefStyle gtk_button_get_relief        (GtkButton      *button);
 
 
 #ifdef __cplusplus
