@@ -262,6 +262,7 @@ typedef void (*GtkCalendarSignalDate) (GtkObject *object, guint arg1, guint arg2
 
 static void gtk_calendar_class_init	(GtkCalendarClass *class);
 static void gtk_calendar_init		(GtkCalendar *calendar);
+static void gtk_calendar_destroy	(GtkObject *calendar);
 static void gtk_calendar_realize	(GtkWidget *widget);
 static void gtk_calendar_unrealize	(GtkWidget *widget);
 static void gtk_calendar_draw_focus	(GtkWidget *widget);
@@ -283,10 +284,6 @@ static gint gtk_calendar_leave_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
 static gint gtk_calendar_key_press	(GtkWidget	   *widget,
 					 GdkEventKey	   *event);
-static gint gtk_calendar_focus_in	(GtkWidget *widget,
-					 GdkEventFocus *event);
-static gint gtk_calendar_focus_out	(GtkWidget *widget,
-					 GdkEventFocus *event);
 static void gtk_calendar_state_changed	(GtkWidget *widget,
 					 GtkStateType previous_state);
 static void gtk_calendar_style_set	(GtkWidget *widget,
@@ -306,8 +303,6 @@ static void gtk_calendar_paint_day_num	(GtkWidget    *widget,
 static void gtk_calendar_paint_day	(GtkWidget    *widget,
 					 gint	       row,
 					 gint	       col);
-static void gtk_calendar_draw		(GtkWidget    *widget,
-					 GdkRectangle *area);
 static void gtk_calendar_compute_days	(GtkCalendar  *calendar);
 static gint left_x_for_column		(GtkCalendar  *calendar,
 					 gint	       column);
@@ -354,11 +349,11 @@ gtk_calendar_class_init (GtkCalendarClass *class)
   
   parent_class = gtk_type_class (GTK_TYPE_WIDGET);
   
+  object_class->destroy = gtk_calendar_destroy;
+
   widget_class->realize = gtk_calendar_realize;
   widget_class->unrealize = gtk_calendar_unrealize;
   widget_class->expose_event = gtk_calendar_expose;
-  widget_class->draw = gtk_calendar_draw;
-  widget_class->draw_focus = gtk_calendar_draw_focus;
   widget_class->size_request = gtk_calendar_size_request;
   widget_class->size_allocate = gtk_calendar_size_allocate;
   widget_class->button_press_event = gtk_calendar_button_press;
@@ -366,11 +361,17 @@ gtk_calendar_class_init (GtkCalendarClass *class)
   widget_class->enter_notify_event = gtk_calendar_enter_notify;
   widget_class->leave_notify_event = gtk_calendar_leave_notify;
   widget_class->key_press_event = gtk_calendar_key_press;
-  widget_class->focus_in_event = gtk_calendar_focus_in;
-  widget_class->focus_out_event = gtk_calendar_focus_out;
   widget_class->style_set = gtk_calendar_style_set;
   widget_class->state_changed = gtk_calendar_state_changed;
   
+  class->month_changed = NULL;
+  class->day_selected = NULL;
+  class->day_selected_double_click = NULL;
+  class->prev_month = NULL;
+  class->next_month = NULL;
+  class->prev_year = NULL;
+  class->next_year = NULL;
+
   gtk_calendar_signals[MONTH_CHANGED_SIGNAL] =
     gtk_signal_new ("month_changed",
 		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
@@ -406,16 +407,6 @@ gtk_calendar_class_init (GtkCalendarClass *class)
 		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkCalendarClass, next_year),
 		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-  
-  gtk_object_class_add_signals (object_class, gtk_calendar_signals, LAST_SIGNAL);
-  
-  class->month_changed = NULL;
-  class->day_selected = NULL;
-  class->day_selected_double_click = NULL;
-  class->prev_month = NULL;
-  class->next_month = NULL;
-  class->prev_year = NULL;
-  class->next_year = NULL;
 }
 
 static void
@@ -432,7 +423,7 @@ gtk_calendar_init (GtkCalendar *calendar)
   widget = GTK_WIDGET (calendar);
   GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
   
-  calendar->private_data = (gpointer) malloc (sizeof (GtkCalendarPrivateData));
+  calendar->private_data = (gpointer) g_malloc (sizeof (GtkCalendarPrivateData));
   private_data = GTK_CALENDAR_PRIVATE_DATA (calendar);
 
   if (!default_abbreviated_dayname[0])
@@ -440,7 +431,7 @@ gtk_calendar_init (GtkCalendar *calendar)
       {
 	tmp_time= (i+3)*86400;
 	strftime ( buffer, sizeof (buffer), "%a", gmtime (&tmp_time));
-	default_abbreviated_dayname[i] = g_locale_to_utf8 (buffer, NULL);
+	default_abbreviated_dayname[i] = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
       }
   
   if (!default_monthname[0])
@@ -448,7 +439,7 @@ gtk_calendar_init (GtkCalendar *calendar)
       {
 	tmp_time=i*2764800;
 	strftime ( buffer, sizeof (buffer), "%B", gmtime (&tmp_time));
-	default_monthname[i] = g_locale_to_utf8 (buffer, NULL);
+	default_monthname[i] = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
       }
   
   /* Set defaults */
@@ -1147,6 +1138,16 @@ gtk_calendar_unrealize (GtkWidget *widget)
       gdk_window_destroy (private_data->main_win);
       private_data->main_win = NULL;      
     }
+  if (private_data->day_name_win)
+    {
+      gdk_window_set_user_data (private_data->day_name_win, NULL);
+      gdk_window_destroy (private_data->day_name_win);
+      private_data->day_name_win = NULL;      
+    }
+  if (calendar->xor_gc)
+    gdk_gc_unref (calendar->xor_gc);
+  if (calendar->gc)
+    gdk_gc_unref (calendar->gc);
   
   if (GTK_WIDGET_CLASS (parent_class)->unrealize)
     (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
@@ -1448,7 +1449,7 @@ gtk_calendar_draw_focus (GtkWidget *widget)
     }
 }
 
-static gint
+static gboolean
 gtk_calendar_expose (GtkWidget	    *widget,
 		     GdkEventExpose *event)
 {
@@ -1476,25 +1477,12 @@ gtk_calendar_expose (GtkWidget	    *widget,
       if (event->window == private_data->week_win)
 	gtk_calendar_paint_week_numbers (widget);
       if (event->window == widget->window)
-	gtk_widget_draw_focus (widget);
+	gtk_calendar_draw_focus (widget);
     }
   
   return FALSE;
 }
 
-static void
-gtk_calendar_draw (GtkWidget	*widget,
-		   GdkRectangle *area)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_CALENDAR (widget));
-  g_return_if_fail (area != NULL);
-  
-  if (GTK_WIDGET_DRAWABLE (widget))
-    gtk_calendar_paint (widget, area);
-  
-}
-
 static void
 gtk_calendar_paint (GtkWidget	 *widget,
 		    GdkRectangle *area)
@@ -1521,7 +1509,7 @@ gtk_calendar_paint (GtkWidget	 *widget,
   if (private_data->week_win != NULL)
     gtk_calendar_paint_week_numbers (widget);
   
-  gtk_widget_draw_focus (widget);
+  gtk_calendar_draw_focus (widget);
 }
 
 static void
@@ -2173,7 +2161,7 @@ gtk_calendar_display_options (GtkCalendar	       *calendar,
   
 }
 
-gint
+gboolean
 gtk_calendar_select_month (GtkCalendar *calendar,
 			   guint	month,
 			   guint	year)
@@ -2250,7 +2238,7 @@ gtk_calendar_clear_marks (GtkCalendar *calendar)
     }
 }
 
-gint
+gboolean
 gtk_calendar_mark_day (GtkCalendar *calendar,
 		       guint	    day)
 {
@@ -2270,7 +2258,7 @@ gtk_calendar_mark_day (GtkCalendar *calendar,
   return TRUE;
 }
 
-gint
+gboolean
 gtk_calendar_unmark_day (GtkCalendar *calendar,
 			 guint	      day)
 {
@@ -2310,13 +2298,14 @@ gtk_calendar_get_date (GtkCalendar *calendar,
     *day = calendar->selected_day;
 }
 
-static gint
+static gboolean
 gtk_calendar_button_press (GtkWidget	  *widget,
 			   GdkEventButton *event)
 {
   GtkCalendar *calendar;
   GtkCalendarPrivateData *private_data;
   gint x, y;
+  void (* action_func) (GtkCalendar *);
   
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_CALENDAR (widget), FALSE);
@@ -2330,26 +2319,26 @@ gtk_calendar_button_press (GtkWidget	  *widget,
   
   if (event->window == private_data->main_win)
     gtk_calendar_main_button (widget, event);
-  
-  if (event->type != GDK_BUTTON_PRESS)
-    return FALSE; /* Double-clicks? Triple-clicks? No thanks! */
+
+  action_func = NULL;  
 
   if (event->window == private_data->arrow_win[ARROW_MONTH_LEFT])
-    gtk_calendar_set_month_prev (calendar);
+    action_func = gtk_calendar_set_month_prev;
+  else if (event->window == private_data->arrow_win[ARROW_MONTH_RIGHT])
+    action_func = gtk_calendar_set_month_next;
+  else if (event->window == private_data->arrow_win[ARROW_YEAR_LEFT])
+    action_func = gtk_calendar_set_year_prev;
+  else if (event->window == private_data->arrow_win[ARROW_YEAR_RIGHT])
+    action_func = gtk_calendar_set_year_next;
+
+  /* only call the action on single click, not double */
+  if (event->type == GDK_BUTTON_PRESS)
+    (* action_func) (calendar);
   
-  if (event->window == private_data->arrow_win[ARROW_MONTH_RIGHT])
-    gtk_calendar_set_month_next (calendar);
-  
-  if (event->window == private_data->arrow_win[ARROW_YEAR_LEFT])
-    gtk_calendar_set_year_prev (calendar);
-  
-  if (event->window == private_data->arrow_win[ARROW_YEAR_RIGHT])
-    gtk_calendar_set_year_next (calendar);
-  
-  return FALSE;
+  return action_func != NULL;
 }
 
-static gint
+static gboolean
 gtk_calendar_motion_notify (GtkWidget	   *widget,
 			    GdkEventMotion *event)
 {
@@ -2391,7 +2380,7 @@ gtk_calendar_motion_notify (GtkWidget	   *widget,
   return TRUE;
 }
 
-static gint
+static gboolean
 gtk_calendar_enter_notify (GtkWidget	    *widget,
 			   GdkEventCrossing *event)
 {
@@ -2431,7 +2420,7 @@ gtk_calendar_enter_notify (GtkWidget	    *widget,
   return TRUE;
 }
 
-static gint
+static gboolean
 gtk_calendar_leave_notify (GtkWidget	    *widget,
 			   GdkEventCrossing *event)
 {
@@ -2627,45 +2616,17 @@ gtk_calendar_state_changed (GtkWidget	   *widget,
   gtk_calendar_set_background (widget);
 }
 
-static gint
-gtk_calendar_focus_in (GtkWidget	 *widget,
-		       GdkEventFocus	 *event)
+static void
+gtk_calendar_destroy (GtkObject *object)
 {
-  GtkCalendar *calendar;
-  
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_CALENDAR (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
-  calendar = GTK_CALENDAR (widget);
-  
-  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-  gtk_calendar_paint_day (widget, calendar->focus_row, calendar->focus_col);
-  
-  return FALSE;
+  GtkCalendarPrivateData *private_data;
+  private_data = GTK_CALENDAR_PRIVATE_DATA (object);
+  g_free (private_data);
+  if (GTK_OBJECT_CLASS (parent_class)->destroy)
+    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-static gint
-gtk_calendar_focus_out (GtkWidget	  *widget,
-			GdkEventFocus	  *event)
-{
-  GtkCalendar *calendar;
-  
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_CALENDAR (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
-  calendar = GTK_CALENDAR (widget);
-  
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-  gtk_calendar_paint_day (widget, calendar->focus_row, calendar->focus_col);
-  
-  return FALSE;
-}
-
-static gint
+static gboolean
 gtk_calendar_key_press (GtkWidget   *widget,
 			GdkEventKey *event)
 {
@@ -2772,6 +2733,7 @@ gtk_calendar_key_press (GtkWidget   *widget,
       
       if (row > -1 && col > -1)
 	{
+	  return_val = TRUE;
 	  gtk_calendar_freeze (calendar);	  
 
 	  if (calendar->day_month[row][col] == MONTH_PREV)
