@@ -40,70 +40,116 @@
 #include "gdk/gdkkeysyms.h"
 #include "gtkcombo.h"
 #include "gtkframe.h"
+#include "gtkintl.h"
 
 const gchar *gtk_combo_string_key = "gtk-combo-string-value";
 
 #define COMBO_LIST_MAX_HEIGHT	(400)
 #define	EMPTY_LIST_HEIGHT	(15)
 
-static void         gtk_combo_class_init      (GtkComboClass *klass);
-static void         gtk_combo_init            (GtkCombo      *combo);
-static void         gtk_combo_destroy         (GtkObject     *combo);
-static GtkListItem *gtk_combo_find            (GtkCombo      *combo);
-static gchar *      gtk_combo_func            (GtkListItem  *li);
-static gint         gtk_combo_focus_idle      (GtkCombo      *combo);
-static gint         gtk_combo_entry_focus_out (GtkEntry      *entry, 
-                                               GdkEventFocus *event, 
-                                               GtkCombo      *combo);
-static void         gtk_combo_get_pos         (GtkCombo      *combo, 
-                                               gint          *x, 
-                                               gint          *y, 
-                                               gint          *height, 
-                                               gint          *width);
-static void         gtk_combo_popup_list      (GtkCombo      *combo);
-static void         gtk_combo_activate        (GtkWidget        *widget,
-					       GtkCombo         *combo);
-static void         gtk_combo_popup_button_press (GtkWidget        *button,
+enum {
+  PROP_0,
+  PROP_ENABLE_ARROW_KEYS,
+  PROP_ENABLE_ARROWS_ALWAYS,
+  PROP_CASE_SENSITIVE
+};
+
+static void         gtk_combo_class_init         (GtkComboClass    *klass);
+static void         gtk_combo_init               (GtkCombo         *combo);
+static void         gtk_combo_destroy            (GtkObject        *combo);
+static GtkListItem *gtk_combo_find               (GtkCombo         *combo);
+static gchar *      gtk_combo_func               (GtkListItem      *li);
+static gint         gtk_combo_focus_idle         (GtkCombo         *combo);
+static gint         gtk_combo_entry_focus_out    (GtkEntry         *entry,
+						  GdkEventFocus    *event,
+						  GtkCombo         *combo);
+static void         gtk_combo_get_pos            (GtkCombo         *combo,
+						  gint             *x,
+						  gint             *y,
+						  gint             *height,
+						  gint             *width);
+static void         gtk_combo_popup_list         (GtkCombo         *combo);
+static void         gtk_combo_activate           (GtkWidget        *widget,
+						  GtkCombo         *combo);
+static gboolean     gtk_combo_popup_button_press (GtkWidget        *button,
 						  GdkEventButton   *event,
 						  GtkCombo         *combo);
 static void         gtk_combo_popup_button_leave (GtkWidget        *button,
 						  GdkEventCrossing *event,
 						  GtkCombo         *combo);
-static void         gtk_combo_update_entry    (GtkList       *list, 
-                                               GtkCombo      *combo);
-static void         gtk_combo_update_list     (GtkEntry      *entry, 
-                                               GtkCombo      *combo);
-static gint         gtk_combo_button_press    (GtkWidget     *widget,
-				               GdkEvent      *event,
-				               GtkCombo      *combo);
-static gint         gtk_combo_button_release  (GtkWidget     *widget,
-				               GdkEvent      *event,
-				               GtkCombo      *combo);
-static gint         gtk_combo_list_enter      (GtkWidget        *widget,
-				               GdkEventCrossing *event,
-				               GtkCombo         *combo);
-static gint         gtk_combo_list_key_press  (GtkWidget     *widget, 
-                                               GdkEventKey   *event, 
-                                               GtkCombo      *combo);
-static gint         gtk_combo_entry_key_press (GtkEntry      *widget, 
-                                               GdkEventKey   *event, 
-                                               GtkCombo      *combo);
-static void         gtk_combo_item_destroy    (GtkObject     *object);
-static void         gtk_combo_size_allocate   (GtkWidget     *widget,
-					       GtkAllocation *allocation);
-
+static void         gtk_combo_update_entry       (GtkList          *list,
+						  GtkCombo         *combo);
+static void         gtk_combo_update_list        (GtkEntry         *entry,
+						  GtkCombo         *combo);
+static gint         gtk_combo_button_press       (GtkWidget        *widget,
+						  GdkEvent         *event,
+						  GtkCombo         *combo);
+static gint         gtk_combo_button_release     (GtkWidget        *widget,
+						  GdkEvent         *event,
+						  GtkCombo         *combo);
+static gint         gtk_combo_list_enter         (GtkWidget        *widget,
+						  GdkEventCrossing *event,
+						  GtkCombo         *combo);
+static gint         gtk_combo_list_key_press     (GtkWidget        *widget,
+						  GdkEventKey      *event,
+						  GtkCombo         *combo);
+static gint         gtk_combo_entry_key_press    (GtkEntry         *widget,
+						  GdkEventKey      *event,
+						  GtkCombo         *combo);
+static gint         gtk_combo_window_key_press   (GtkWidget        *window,
+						  GdkEventKey      *event,
+						  GtkCombo         *combo);
+static void         gtk_combo_item_destroy       (GtkObject        *object);
+static void         gtk_combo_size_allocate      (GtkWidget        *widget,
+						  GtkAllocation   *allocation);
+static void         gtk_combo_set_property       (GObject         *object,
+						  guint            prop_id,
+						  const GValue    *value,
+						  GParamSpec      *pspec);
+static void         gtk_combo_get_property       (GObject         *object,
+						  guint            prop_id,
+						  GValue          *value,
+						  GParamSpec      *pspec);
 static GtkHBoxClass *parent_class = NULL;
 
 static void
 gtk_combo_class_init (GtkComboClass * klass)
 {
+  GObjectClass *gobject_class;
   GtkObjectClass *oclass;
   GtkWidgetClass *widget_class;
 
+  gobject_class = (GObjectClass *) klass;
   parent_class = gtk_type_class (GTK_TYPE_HBOX);
   oclass = (GtkObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
 
+  gobject_class->set_property = gtk_combo_set_property; 
+  gobject_class->get_property = gtk_combo_get_property; 
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_ENABLE_ARROW_KEYS,
+                                   g_param_spec_boolean ("enable_arrow_keys",
+                                                         _("Enable arrow keys"),
+                                                         _("Whether the arrow keys move through the list of items"),
+                                                         TRUE,
+                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_ENABLE_ARROWS_ALWAYS,
+                                   g_param_spec_boolean ("enable_arrows_always",
+                                                         _("Always enable arrows"),
+                                                         _("Whether the arrow keys work, even if the entry contents are not in the list"),
+                                                         TRUE,
+                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_CASE_SENSITIVE,
+                                   g_param_spec_boolean ("case_sensitive",
+                                                         _("Case sensitive"),
+                                                         _("Whether list item matching is case sensitive"),
+                                                         FALSE,
+                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
+  
+   
   oclass->destroy = gtk_combo_destroy;
   
   widget_class->size_allocate = gtk_combo_size_allocate;
@@ -208,6 +254,32 @@ gtk_combo_entry_key_press (GtkEntry * entry, GdkEventKey * event, GtkCombo * com
   return FALSE;
 }
 
+static int
+gtk_combo_window_key_press (GtkWidget   *window,
+			    GdkEventKey *event,
+			    GtkCombo    *combo)
+{
+  if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
+    {
+      if (GTK_WIDGET_VISIBLE (combo->popwin))
+	{
+	  gtk_widget_hide (combo->popwin);
+	  
+	  if (GTK_WIDGET_HAS_GRAB (combo->popwin))
+	    {
+	      gtk_grab_remove (combo->popwin);
+	      gdk_pointer_ungrab (event->time);
+	    }
+	}
+
+      gtk_signal_emit_stop_by_name (GTK_OBJECT (window), "key_press_event");
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static GtkListItem *
 gtk_combo_find (GtkCombo * combo)
 {
@@ -419,7 +491,7 @@ gtk_combo_activate (GtkWidget        *widget,
 		    NULL, NULL, GDK_CURRENT_TIME);
 }
 
-static void        
+static gboolean
 gtk_combo_popup_button_press (GtkWidget        *button,
 			      GdkEventButton   *event,
 			      GtkCombo         *combo)
@@ -438,6 +510,7 @@ gtk_combo_popup_button_press (GtkWidget        *button,
 		    GDK_BUTTON_RELEASE_MASK,
 		    NULL, NULL, event->time);
   gtk_grab_add (combo->list);
+  return TRUE;
 }
 
 static void         
@@ -556,7 +629,7 @@ gtk_combo_button_release (GtkWidget * widget, GdkEvent * event, GtkCombo * combo
 			    GDK_BUTTON_RELEASE_MASK |
 			    GDK_POINTER_MOTION_MASK, 
 			    NULL, NULL, GDK_CURRENT_TIME);
-	  return FALSE;
+	  return TRUE;
 	}
     }
   else
@@ -650,11 +723,11 @@ gtk_combo_init (GtkCombo * combo)
   GtkWidget *event_box;
   GdkCursor *cursor;
 
-  combo->case_sensitive = 0;
-  combo->value_in_list = 0;
-  combo->ok_if_empty = 1;
-  combo->use_arrows = 1;
-  combo->use_arrows_always = 0;
+  combo->case_sensitive = FALSE;
+  combo->value_in_list = FALSE;
+  combo->ok_if_empty = TRUE;
+  combo->use_arrows = TRUE;
+  combo->use_arrows_always = FALSE;
   combo->entry = gtk_entry_new ();
   combo->button = gtk_button_new ();
   combo->current_button = 0;
@@ -686,6 +759,9 @@ gtk_combo_init (GtkCombo * combo)
   combo->popwin = gtk_window_new (GTK_WINDOW_POPUP);
   gtk_widget_ref (combo->popwin);
   gtk_window_set_policy (GTK_WINDOW (combo->popwin), 1, 1, 0);
+
+  gtk_signal_connect (GTK_OBJECT (combo->popwin), "key_press_event",
+		      GTK_SIGNAL_FUNC (gtk_combo_window_key_press), combo);
   
   gtk_widget_set_events (combo->popwin, GDK_KEY_PRESS_MASK);
 
@@ -772,7 +848,7 @@ gtk_combo_new (void)
 }
 
 void
-gtk_combo_set_value_in_list (GtkCombo * combo, gint val, gint ok_if_empty)
+gtk_combo_set_value_in_list (GtkCombo * combo, gboolean val, gboolean ok_if_empty)
 {
   g_return_if_fail (combo != NULL);
   g_return_if_fail (GTK_IS_COMBO (combo));
@@ -782,31 +858,35 @@ gtk_combo_set_value_in_list (GtkCombo * combo, gint val, gint ok_if_empty)
 }
 
 void
-gtk_combo_set_case_sensitive (GtkCombo * combo, gint val)
+gtk_combo_set_case_sensitive (GtkCombo * combo, gboolean val)
 {
   g_return_if_fail (combo != NULL);
   g_return_if_fail (GTK_IS_COMBO (combo));
 
   combo->case_sensitive = val;
+  g_object_notify (G_OBJECT (combo), "case_sensitive");
 }
 
 void
-gtk_combo_set_use_arrows (GtkCombo * combo, gint val)
+gtk_combo_set_use_arrows (GtkCombo * combo, gboolean val)
 {
   g_return_if_fail (combo != NULL);
   g_return_if_fail (GTK_IS_COMBO (combo));
 
   combo->use_arrows = val;
+  g_object_notify (G_OBJECT (combo), "enable_arrows");
 }
 
 void
-gtk_combo_set_use_arrows_always (GtkCombo * combo, gint val)
+gtk_combo_set_use_arrows_always (GtkCombo * combo, gboolean val)
 {
   g_return_if_fail (combo != NULL);
   g_return_if_fail (GTK_IS_COMBO (combo));
 
   combo->use_arrows_always = val;
-  combo->use_arrows = 1;
+  combo->use_arrows = TRUE;
+  g_object_notify (G_OBJECT (combo), "enable_arrows");
+  g_object_notify (G_OBJECT (combo), "enable_arrows_always");
 }
 
 void
@@ -913,3 +993,58 @@ gtk_combo_disable_activate (GtkCombo* combo)
     combo->activate_id = 0;
   }
 }
+
+static void
+gtk_combo_set_property (GObject         *object,
+			guint            prop_id,
+			const GValue    *value,
+			GParamSpec      *pspec)
+{
+  GtkCombo *combo = GTK_COMBO (object);
+  
+  switch (prop_id)
+    {
+    case PROP_ENABLE_ARROW_KEYS:
+      /* This call does the notification */
+      gtk_combo_set_use_arrows (combo, g_value_get_boolean (value));
+      break;
+    case PROP_ENABLE_ARROWS_ALWAYS:
+      /* This call does the notification */
+      gtk_combo_set_use_arrows_always (combo, g_value_get_boolean (value));
+      break;
+    case PROP_CASE_SENSITIVE:
+      /* This call does the notification */
+      gtk_combo_set_case_sensitive (combo, g_value_get_boolean (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+  
+}
+
+static void
+gtk_combo_get_property (GObject         *object,
+			guint            prop_id,
+			GValue          *value,
+			GParamSpec      *pspec)
+{
+  GtkCombo *combo = GTK_COMBO (object);
+  
+  switch (prop_id)
+    {
+    case PROP_ENABLE_ARROW_KEYS:
+      g_value_set_boolean (value, combo->use_arrows);
+      break;
+    case PROP_ENABLE_ARROWS_ALWAYS:
+      g_value_set_boolean (value, combo->use_arrows_always);
+      break;
+    case PROP_CASE_SENSITIVE:
+      g_value_set_boolean (value, combo->case_sensitive);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+   
+}
