@@ -183,39 +183,40 @@ static GdkCursor *   gtk_drag_get_cursor         (GdkDragAction action);
 static GtkWidget    *gtk_drag_get_ipc_widget     (void);
 static void          gtk_drag_release_ipc_widget (GtkWidget      *widget);
 
-static void          gtk_drag_highlight_paint    (GtkWidget      *widget);
 static gboolean      gtk_drag_highlight_expose   (GtkWidget      *widget,
 					  	  GdkEventExpose *event,
 						  gpointer        data);
 
+static void     gtk_drag_selection_received (GtkWidget        *widget,
+					     GtkSelectionData *selection_data,
+					     guint32           time,
+					     gpointer          data);
+static void     gtk_drag_find_widget        (GtkWidget        *widget,
+					     GtkDragFindData  *data);
+static void     gtk_drag_proxy_begin        (GtkWidget        *widget,
+					     GtkDragDestInfo  *dest_info,
+					     guint32           time);
+static void     gtk_drag_dest_realized      (GtkWidget        *widget);
+static void     gtk_drag_dest_site_destroy  (gpointer          data);
+static void     gtk_drag_dest_leave         (GtkWidget        *widget,
+					     GdkDragContext   *context,
+					     guint             time);
+static gboolean gtk_drag_dest_motion        (GtkWidget        *widget,
+					     GdkDragContext   *context,
+					     gint              x,
+					     gint              y,
+					     guint             time);
+static gboolean gtk_drag_dest_drop          (GtkWidget        *widget,
+					     GdkDragContext   *context,
+					     gint              x,
+					     gint              y,
+					     guint             time);
 
-static GdkAtom   gtk_drag_dest_find_target    (GtkWidget          *widget,
-					       GtkDragDestSite    *site,
-			                       GdkDragContext     *context);
-static void      gtk_drag_selection_received  (GtkWidget          *widget,
-					       GtkSelectionData   *selection_data,
-					       guint32             time,
-					       gpointer            data);
-static void      gtk_drag_find_widget         (GtkWidget          *widget,
-					       GtkDragFindData    *data);
-static void      gtk_drag_proxy_begin         (GtkWidget          *widget,
-					       GtkDragDestInfo    *dest_info);
-static void      gtk_drag_dest_info_destroy   (gpointer            data);
-static void      gtk_drag_dest_realized       (GtkWidget          *widget);
-static void      gtk_drag_dest_site_destroy   (gpointer            data);
-static void      gtk_drag_dest_leave          (GtkWidget          *widget,
-					       GdkDragContext     *context,
-					       guint               time);
-static gboolean  gtk_drag_dest_motion         (GtkWidget	  *widget,
-				               GdkDragContext     *context,
-					       gint                x,
-					       gint                y,
-					       guint               time);
-static gboolean  gtk_drag_dest_drop           (GtkWidget	  *widget,
-					       GdkDragContext     *context,
-					       gint                x,
-					       gint                y,
-					       guint               time);
+static GtkDragDestInfo *  gtk_drag_get_dest_info     (GdkDragContext *context,
+						      gboolean        create);
+static GtkDragSourceInfo *gtk_drag_get_source_info   (GdkDragContext *context,
+						      gboolean        create);
+static void               gtk_drag_clear_source_info (GdkDragContext *context);
 
 static void gtk_drag_source_check_selection    (GtkDragSourceInfo *info, 
 					        GdkAtom            selection,
@@ -239,7 +240,7 @@ static void gtk_drag_selection_get             (GtkWidget         *widget,
 						gpointer           data);
 static gint gtk_drag_anim_timeout              (gpointer           data);
 static void gtk_drag_remove_icon               (GtkDragSourceInfo *info);
-static void gtk_drag_source_info_destroy       (gpointer           data);
+static void gtk_drag_source_info_destroy       (GtkDragSourceInfo *info);
 static void gtk_drag_update                    (GtkDragSourceInfo *info,
 						gint               x_root,
 						gint               y_root,
@@ -739,20 +740,22 @@ gtk_drag_finish (GdkDragContext *context,
 }
 
 /*************************************************************
- * gtk_drag_highlight_paint:
- *     Paint a highlight indicating drag status onto the widget.
+ * gtk_drag_highlight_expose:
+ *     Callback for expose_event for highlighted widgets.
  *   arguments:
  *     widget:
+ *     event:
+ *     data:
  *   results:
  *************************************************************/
 
-static void 
-gtk_drag_highlight_paint (GtkWidget  *widget)
+static gboolean
+gtk_drag_highlight_expose (GtkWidget      *widget,
+			   GdkEventExpose *event,
+			   gpointer        data)
 {
   gint x, y, width, height;
-
-  g_return_if_fail (widget != NULL);
-
+  
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       if (GTK_WIDGET_NO_WINDOW (widget))
@@ -778,25 +781,8 @@ gtk_drag_highlight_paint (GtkWidget  *widget)
 			  FALSE,
 			  x, y, width - 1, height - 1);
     }
-}
 
-/*************************************************************
- * gtk_drag_highlight_expose:
- *     Callback for expose_event for highlighted widgets.
- *   arguments:
- *     widget:
- *     event:
- *     data:
- *   results:
- *************************************************************/
-
-static gboolean
-gtk_drag_highlight_expose (GtkWidget      *widget,
-			   GdkEventExpose *event,
-			   gpointer        data)
-{
-  gtk_drag_highlight_paint (widget);
-  return TRUE;
+  return FALSE;
 }
 
 /*************************************************************
@@ -810,9 +796,6 @@ gtk_drag_highlight_expose (GtkWidget      *widget,
 void 
 gtk_drag_highlight (GtkWidget  *widget)
 {
-  gtk_signal_connect_after (GTK_OBJECT (widget), "draw",
-			    GTK_SIGNAL_FUNC (gtk_drag_highlight_paint),
-			    NULL);
   gtk_signal_connect (GTK_OBJECT (widget), "expose_event",
 		      GTK_SIGNAL_FUNC (gtk_drag_highlight_expose),
 		      NULL);
@@ -833,9 +816,6 @@ gtk_drag_unhighlight (GtkWidget *widget)
 {
   g_return_if_fail (widget != NULL);
 
-  gtk_signal_disconnect_by_func (GTK_OBJECT (widget),
-				 GTK_SIGNAL_FUNC (gtk_drag_highlight_paint),
-				 NULL);
   gtk_signal_disconnect_by_func (GTK_OBJECT (widget),
 				 GTK_SIGNAL_FUNC (gtk_drag_highlight_expose),
 				 NULL);
@@ -960,6 +940,58 @@ gtk_drag_dest_unset (GtkWidget *widget)
   gtk_object_set_data (GTK_OBJECT (widget), "gtk-drag-dest", NULL);
 }
 
+/**
+ * gtk_drag_dest_get_target_list:
+ * @widget: a #GtkWidget
+ * 
+ * Returns the list of targets this widget can accept from
+ * drag-and-drop.
+ * 
+ * Return value: the #GtkTargetList, or %NULL if none
+ **/
+GtkTargetList*
+gtk_drag_dest_get_target_list (GtkWidget *widget)
+{
+  GtkDragDestSite *site;
+  
+  site = gtk_object_get_data (GTK_OBJECT (widget), "gtk-drag-dest");
+
+  return site ? site->target_list : NULL;  
+}
+
+/**
+ * gtk_drag_dest_set_target_list:
+ * @widget: a #GtkWidget that's a drag destination
+ * @target_list: list of droppable targets, or %NULL for none
+ * 
+ * Sets the target types that this widget can accept from drag-and-drop.
+ * The widget must first be made into a drag destination with
+ * gtk_drag_dest_set().
+ **/
+void
+gtk_drag_dest_set_target_list (GtkWidget      *widget,
+                               GtkTargetList  *target_list)
+{
+  GtkDragDestSite *site;
+  
+  site = gtk_object_get_data (GTK_OBJECT (widget), "gtk-drag-dest");
+  
+  if (site == NULL)
+    {
+      g_warning ("can't set a target list on a widget until you've called gtk_drag_dest_set() to make the widget into a drag destination");
+      return;
+    }
+
+  if (target_list)
+    gtk_target_list_ref (site->target_list);
+  
+  if (site->target_list)
+    gtk_target_list_unref (site->target_list);
+
+  site->target_list = target_list;
+}
+
+
 /*************************************************************
  * gtk_drag_dest_handle_event:
  *     Called from widget event handling code on Drag events
@@ -983,21 +1015,7 @@ gtk_drag_dest_handle_event (GtkWidget *toplevel,
 
   context = event->dnd.context;
 
-  info = g_dataset_get_data (context, "gtk-info");
-  if (!info)
-    {
-      info = g_new (GtkDragDestInfo, 1);
-      info->widget = NULL;
-      info->context = event->dnd.context;
-      info->proxy_source = NULL;
-      info->proxy_data = NULL;
-      info->dropped = FALSE;
-      info->proxy_drop_wait = FALSE;
-      g_object_set_qdata_full (G_OBJECT (context),
-			       g_quark_from_static_string ("gtk-info"),
-			       info,
-			       gtk_drag_dest_info_destroy);
-    }
+  info = gtk_drag_get_dest_info (context, TRUE);
 
   /* Find the widget for the event */
   switch (event->type)
@@ -1073,25 +1091,35 @@ gtk_drag_dest_handle_event (GtkWidget *toplevel,
     }
 }
 
-/*************************************************************
+/**
  * gtk_drag_dest_find_target:
- *     Decide on a target for the drag.
- *   arguments:
- *     site:
- *     context:
- *   results:
- *************************************************************/
-
-static GdkAtom
-gtk_drag_dest_find_target (GtkWidget       *widget,
-			   GtkDragDestSite *site,
-			   GdkDragContext  *context)
+ * @widget: drag destination widget
+ * @context: drag context
+ * @dest_target_list: list of droppable targets
+ * 
+ * Looks for a match between @context->targets and the
+ * @dest_target_list, returning the first matching target, otherwise
+ * returning %GDK_NONE. @dest_target_list should usually be the return
+ * value from gtk_drag_dest_get_target_list(), but some widgets may
+ * have different valid targets for different parts of the widget; in
+ * that case, they will have to implement a drag_motion handler that
+ * passes the correct target list to this function.
+ * 
+ * Return value: first target that the source offers and the dest can accept, or %GDK_NONE
+ **/
+GdkAtom
+gtk_drag_dest_find_target (GtkWidget      *widget,
+                           GdkDragContext *context,
+                           GtkTargetList  *dest_target_list)
 {
   GList *tmp_target;
   GList *tmp_source = NULL;
   GtkWidget *source_widget = gtk_drag_get_source_widget (context);
 
-  tmp_target = site->target_list->list;
+  if (dest_target_list == NULL)
+    return GDK_NONE;
+  
+  tmp_target = dest_target_list->list;
   while (tmp_target)
     {
       GtkTargetPair *pair = tmp_target->data;
@@ -1127,7 +1155,7 @@ gtk_drag_selection_received (GtkWidget        *widget,
   drop_widget = data;
 
   context = gtk_object_get_data (GTK_OBJECT (widget), "drag-context");
-  info = g_object_get_qdata (G_OBJECT (context), g_quark_from_static_string ("gtk-info"));
+  info = gtk_drag_get_dest_info (context, FALSE);
 
   if (info->proxy_data && 
       info->proxy_data->target == selection_data->target)
@@ -1321,18 +1349,29 @@ gtk_drag_find_widget (GtkWidget       *widget,
 
 static void
 gtk_drag_proxy_begin (GtkWidget       *widget, 
-		      GtkDragDestInfo *dest_info)
+		      GtkDragDestInfo *dest_info,
+		      guint32          time)
 {
   GtkDragSourceInfo *source_info;
   GList *tmp_list;
+  GdkDragContext *context;
+  GtkWidget *ipc_widget;
+
+  if (dest_info->proxy_source)
+    {
+      gdk_drag_abort (dest_info->proxy_source->context, time);
+      gtk_drag_source_info_destroy (dest_info->proxy_source);
+      dest_info->proxy_source = NULL;
+    }
   
-  source_info = g_new0 (GtkDragSourceInfo, 1);
-  source_info->ipc_widget = gtk_drag_get_ipc_widget ();
-  
-  source_info->widget = widget;
-  gtk_widget_ref (source_info->widget);
-  source_info->context = gdk_drag_begin (source_info->ipc_widget->window,
-					 dest_info->context->targets);
+  ipc_widget = gtk_drag_get_ipc_widget ();
+  context = gdk_drag_begin (ipc_widget->window,
+			    dest_info->context->targets);
+
+  source_info = gtk_drag_get_source_info (context, TRUE);
+
+  source_info->ipc_widget = ipc_widget;
+  source_info->widget = gtk_widget_ref (widget);
 
   source_info->target_list = gtk_target_list_new (NULL, 0);
   tmp_list = dest_info->context->targets;
@@ -1345,11 +1384,7 @@ gtk_drag_proxy_begin (GtkWidget       *widget,
 
   source_info->proxy_dest = dest_info;
   
-  g_object_set_qdata (G_OBJECT (source_info->context),
-                      g_quark_from_static_string ("gtk-info"),
-                      source_info);
-  
-  gtk_signal_connect (GTK_OBJECT (source_info->ipc_widget), 
+  gtk_signal_connect (GTK_OBJECT (ipc_widget), 
 		      "selection_get",
 		      GTK_SIGNAL_FUNC (gtk_drag_selection_get), 
 		      source_info);
@@ -1365,6 +1400,59 @@ gtk_drag_dest_info_destroy (gpointer data)
   g_free (info);
 }
 
+static GtkDragDestInfo *
+gtk_drag_get_dest_info (GdkDragContext *context,
+			gboolean        create)
+{
+  GtkDragDestInfo *info;
+  static GQuark info_quark = 0;
+  if (!info_quark)
+    info_quark = g_quark_from_static_string ("gtk-dest-info");
+  
+  info = g_object_get_qdata (G_OBJECT (context), info_quark);
+  if (!info && create)
+    {
+      info = g_new (GtkDragDestInfo, 1);
+      info->widget = NULL;
+      info->context = context;
+      info->proxy_source = NULL;
+      info->proxy_data = NULL;
+      info->dropped = FALSE;
+      info->proxy_drop_wait = FALSE;
+      g_object_set_qdata_full (G_OBJECT (context), info_quark,
+			       info, gtk_drag_dest_info_destroy);
+    }
+
+  return info;
+}
+
+static GQuark dest_info_quark = 0;
+
+static GtkDragSourceInfo *
+gtk_drag_get_source_info (GdkDragContext *context,
+			  gboolean        create)
+{
+  GtkDragSourceInfo *info;
+  if (!dest_info_quark)
+    dest_info_quark = g_quark_from_static_string ("gtk-source-info");
+  
+  info = g_object_get_qdata (G_OBJECT (context), dest_info_quark);
+  if (!info && create)
+    {
+      info = g_new0 (GtkDragSourceInfo, 1);
+      info->context = context;
+      g_object_set_qdata (G_OBJECT (context), dest_info_quark, info);
+    }
+
+  return info;
+}
+
+static void
+gtk_drag_clear_source_info (GdkDragContext *context)
+{
+  g_object_set_qdata (G_OBJECT (context), dest_info_quark, NULL);
+}
+
 static void
 gtk_drag_dest_realized (GtkWidget *widget)
 {
@@ -1398,11 +1486,14 @@ gtk_drag_dest_leave (GtkWidget      *widget,
 
   if (site->do_proxy)
     {
-      GtkDragDestInfo *info = g_object_get_qdata (G_OBJECT (context),
-                                                  g_quark_from_static_string ("gtk-info"));
+      GtkDragDestInfo *info = gtk_drag_get_dest_info (context, FALSE);
 
-      if (info->proxy_source && !info->dropped)
-	gdk_drag_abort (info->proxy_source->context, time);
+      if (info->proxy_source && info->proxy_source->widget == widget && !info->dropped)
+	{
+	  gdk_drag_abort (info->proxy_source->context, time);
+	  gtk_drag_source_info_destroy (info->proxy_source);
+	  info->proxy_source = NULL;
+	}
       
       return;
     }
@@ -1440,11 +1531,10 @@ gtk_drag_dest_motion (GtkWidget	     *widget,
       GdkWindow *dest_window;
       GdkDragProtocol proto;
 	
-      GtkDragDestInfo *info = g_object_get_qdata (G_OBJECT (context),
-                                                 g_quark_from_static_string ("gtk-info"));
+      GtkDragDestInfo *info = gtk_drag_get_dest_info (context, FALSE);
 
-      if (!info->proxy_source)
-	gtk_drag_proxy_begin (widget, info);
+      if (!info->proxy_source || info->proxy_source->widget != widget)
+	gtk_drag_proxy_begin (widget, info, time);
 
       current_event = gtk_get_current_event ();
 
@@ -1501,7 +1591,7 @@ gtk_drag_dest_motion (GtkWidget	     *widget,
 	    }
 	}
       
-      if (action && gtk_drag_dest_find_target (widget, site, context))
+      if (action && gtk_drag_dest_find_target (widget, context, site->target_list))
 	{
 	  if (!site->have_drag)
 	    {
@@ -1538,8 +1628,7 @@ gtk_drag_dest_drop (GtkWidget	     *widget,
   site = gtk_object_get_data (GTK_OBJECT (widget), "gtk-drag-dest");
   g_return_val_if_fail (site != NULL, FALSE);
 
-  info = g_object_get_qdata (G_OBJECT (context),
-                             g_quark_from_static_string ("gtk-info"));
+  info = gtk_drag_get_dest_info (context, FALSE);
   g_return_val_if_fail (info != NULL, FALSE);
 
   info->drop_x = x;
@@ -1563,7 +1652,7 @@ gtk_drag_dest_drop (GtkWidget	     *widget,
 	  GdkWindow *dest_window;
 	  GdkDragProtocol proto;
 	  
-	  gtk_drag_proxy_begin (widget, info);
+	  gtk_drag_proxy_begin (widget, info, time);
 	  info->proxy_drop_wait = TRUE;
 	  info->proxy_drop_time = time;
 	  
@@ -1599,7 +1688,6 @@ gtk_drag_dest_drop (GtkWidget	     *widget,
 	    gtk_drag_source_check_selection (info->proxy_source, selection, time);
 
 	  gdk_event_free (current_event);
-      
 	}
 
       return TRUE;
@@ -1610,7 +1698,7 @@ gtk_drag_dest_drop (GtkWidget	     *widget,
 
       if (site->flags & GTK_DEST_DEFAULT_DROP)
 	{
-	  GdkAtom target = gtk_drag_dest_find_target (widget, site, context);
+	  GdkAtom target = gtk_drag_dest_find_target (widget, context, site->target_list);
       
 	  if (target == GDK_NONE)
 	    return FALSE;
@@ -1653,6 +1741,8 @@ gtk_drag_begin (GtkWidget         *widget,
   GList *tmp_list;
   guint32 time = GDK_CURRENT_TIME;
   GdkDragAction possible_actions, suggested_action;
+  GdkDragContext *context;
+  GtkWidget *ipc_widget;
 
   g_return_val_if_fail (widget != NULL, NULL);
   g_return_val_if_fail (GTK_WIDGET_REALIZED (widget), NULL);
@@ -1661,12 +1751,6 @@ gtk_drag_begin (GtkWidget         *widget,
   if (event)
     time = gdk_event_get_time (event);
 
-  info = g_new0 (GtkDragSourceInfo, 1);
-  info->ipc_widget = gtk_drag_get_ipc_widget ();
-  source_widgets = g_slist_prepend (source_widgets, info->ipc_widget);
-
-  gtk_object_set_data (GTK_OBJECT (info->ipc_widget), "gtk-info", info);
-
   tmp_list = g_list_last (target_list->list);
   while (tmp_list)
     {
@@ -1676,15 +1760,20 @@ gtk_drag_begin (GtkWidget         *widget,
       tmp_list = tmp_list->prev;
     }
 
-  info->widget = widget;
-  gtk_widget_ref (info->widget);
-  
-  info->context = gdk_drag_begin (info->ipc_widget->window, targets);
+  ipc_widget = gtk_drag_get_ipc_widget ();
+  source_widgets = g_slist_prepend (source_widgets, ipc_widget);
+
+  context = gdk_drag_begin (ipc_widget->window, targets);
   g_list_free (targets);
   
-  g_object_set_qdata (G_OBJECT (info->context),
-                     g_quark_from_static_string ("gtk-info"), info);
+  info = gtk_drag_get_source_info (context, TRUE);
+  
+  info->ipc_widget = ipc_widget;
+  gtk_object_set_data (GTK_OBJECT (info->ipc_widget), "gtk-info", info);
 
+  info->widget = gtk_widget_ref (widget);
+
+  
   info->button = button;
   info->target_list = target_list;
   gtk_target_list_ref (target_list);
@@ -1919,8 +2008,7 @@ gtk_drag_set_icon_window (GdkDragContext *context,
   g_return_if_fail (context != NULL);
   g_return_if_fail (widget != NULL);
 
-  info = g_object_get_qdata (G_OBJECT (context),
-                            g_quark_from_static_string ("gtk-info"));
+  info = gtk_drag_get_source_info (context, FALSE);
   gtk_drag_remove_icon (info);
 
   info->icon_window = widget;
@@ -2111,8 +2199,7 @@ gtk_drag_source_handle_event (GtkWidget *widget,
   g_return_if_fail (event != NULL);
 
   context = event->dnd.context;
-  info = g_object_get_qdata (G_OBJECT (context),
-                            g_quark_from_static_string ("gtk-info"));
+  info = gtk_drag_get_source_info (context, FALSE);
   if (!info)
     return;
 
@@ -2291,8 +2378,7 @@ gtk_drag_drop_finished (GtkDragSourceInfo *info,
 	  /* Mark the context as dead, so if the destination decides
 	   * to respond really late, we still are OK.
 	   */
-	  g_object_set_qdata (G_OBJECT (info->context),
-                             g_quark_from_static_string ("gtk-info"), NULL);
+	  gtk_drag_clear_source_info (info->context);
 	  gtk_timeout_add (ANIM_STEP_TIME, gtk_drag_anim_timeout, anim);
 	}
     }
@@ -2380,6 +2466,7 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
 			  gpointer        data)
 {
   GtkDragSourceSite *site;
+  gboolean retval = FALSE;
   site = (GtkDragSourceSite *)data;
 
   switch (event->type)
@@ -2390,6 +2477,7 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
 	  site->state |= (GDK_BUTTON1_MASK << (event->button.button - 1));
 	  site->x = event->button.x;
 	  site->y = event->button.y;
+	  retval = TRUE;
 	}
       break;
       
@@ -2397,6 +2485,7 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
       if ((GDK_BUTTON1_MASK << (event->button.button - 1)) & site->start_button_mask)
 	{
 	  site->state &= ~(GDK_BUTTON1_MASK << (event->button.button - 1));
+	  retval = TRUE;
 	}
       break;
       
@@ -2413,9 +2502,9 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
 		  GDK_BUTTON1_MASK << (i - 1))
 		break;
 	    }
-	  
-	  if (MAX (ABS (site->x - event->motion.x),
-		   ABS (site->y - event->motion.y)) > 3)
+
+	  if (gtk_drag_check_threshold (widget, site->x, site->y,
+					event->motion.x, event->motion.y))
 	    {
 	      GtkDragSourceInfo *info;
 	      GdkDragContext *context;
@@ -2425,9 +2514,7 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
 					site->actions, 
 					i, event);
 
-	      
-	      info = g_object_get_qdata (G_OBJECT (context),
-                                         g_quark_from_static_string ("gtk-info"));
+	      info = gtk_drag_get_source_info (context, FALSE);
 
 	      if (!info->icon_window)
 		{
@@ -2440,7 +2527,7 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
 		    gtk_drag_set_icon_default (context);
 		}
 
-	      return TRUE;
+	      retval = TRUE;
 	    }
 	}
       break;
@@ -2448,7 +2535,8 @@ gtk_drag_source_event_cb (GtkWidget      *widget,
     default:			/* hit for 2/3BUTTON_PRESS */
       break;
     }
-  return FALSE;
+  
+  return retval;
 }
 
 static void 
@@ -2581,11 +2669,9 @@ gtk_drag_remove_icon (GtkDragSourceInfo *info)
 }
 
 static void
-gtk_drag_source_info_destroy (gpointer data)
+gtk_drag_source_info_destroy (GtkDragSourceInfo *info)
 {
-  GtkDragSourceInfo *info = data;
-
-  gtk_drag_remove_icon (data);
+  gtk_drag_remove_icon (info);
 
   if (!info->proxy_dest)
     gtk_signal_emit_by_name (GTK_OBJECT (info->widget), "drag_end", 
@@ -2602,7 +2688,7 @@ gtk_drag_source_info_destroy (gpointer data)
 
   gtk_target_list_unref (info->target_list);
 
-  g_object_set_qdata (G_OBJECT (info->context), g_quark_from_static_string ("gtk-info"), NULL);
+  gtk_drag_clear_source_info (info->context);
   gdk_drag_context_unref (info->context);
 
   if (info->drop_timeout)
@@ -2661,12 +2747,22 @@ gtk_drag_update (GtkDragSourceInfo *info,
 		       possible_actions,
 		       time))
     {
-      if (info->last_event)
-	gdk_event_free ((GdkEvent *)info->last_event);
-      
-      info->last_event = gdk_event_copy ((GdkEvent *)event);
+      if (info->last_event != event) /* Paranoia, should not happen */
+	{
+	  if (info->last_event)
+	    gdk_event_free ((GdkEvent *)info->last_event);
+	  info->last_event = gdk_event_copy ((GdkEvent *)event);
+	}
     }
-
+  else
+    {
+      if (info->last_event)
+	{
+	  gdk_event_free ((GdkEvent *)info->last_event);
+	  info->last_event = NULL;
+	}
+    }
+  
   if (dest_window)
     gdk_window_unref (dest_window);
 
@@ -2847,3 +2943,30 @@ gtk_drag_abort_timeout (gpointer data)
   
   return FALSE;
 }
+
+/**
+ * gtk_drag_check_threshold:
+ * @widget: a #GtkWidget
+ * @start_x: X coordinate of start of drag
+ * @start_y: Y coordinate of start of drag
+ * @current_x: current X coordinate
+ * @current_y: current Y coordinate
+ * 
+ * Checks to see if a mouse drag starting at (start_x, start_y) and ending
+ * at (current_x, current_y) has passed the GTK drag threshhold, and thus
+ * should trigger the beginning of a drag-and-drop operation.
+ *
+ * Return Value: If the drag threshold has been passed.
+ **/
+gboolean
+gtk_drag_check_threshold (GtkWidget *widget,
+			  gint       start_x,
+			  gint       start_y,
+			  gint       current_x,
+			  gint       current_y)
+{
+#define DRAG_THRESHOLD 8
+
+  return (ABS (current_x - start_x) > DRAG_THRESHOLD ||
+	  ABS (current_y - start_y) > DRAG_THRESHOLD);
+}
