@@ -92,6 +92,17 @@ typedef enum
   GTK_DIR_RIGHT
 } GtkDirectionType;
 
+/* Built-in stock icon sizes */
+typedef enum
+{
+  GTK_ICON_SIZE_INVALID,
+  GTK_ICON_SIZE_MENU,
+  GTK_ICON_SIZE_SMALL_TOOLBAR,
+  GTK_ICON_SIZE_LARGE_TOOLBAR,
+  GTK_ICON_SIZE_BUTTON,
+  GTK_ICON_SIZE_DIALOG
+} GtkIconSize;
+
 /* Reading directions for text */
 typedef enum
 {
@@ -144,8 +155,8 @@ typedef enum
 } GtkMetricType;
 
 typedef enum {
-  GTK_MOVEMENT_CHARS,             /* move by forw/back chars */
-  GTK_MOVEMENT_POSITIONS,         /* move by left/right chars */
+  GTK_MOVEMENT_LOGICAL_POSITIONS, /* move by forw/back graphemes */
+  GTK_MOVEMENT_VISUAL_POSITIONS,  /* move by left/right graphemes */
   GTK_MOVEMENT_WORDS,             /* move by forward/back words */
   GTK_MOVEMENT_DISPLAY_LINES,     /* move up/down lines (wrapped lines) */
   GTK_MOVEMENT_DISPLAY_LINE_ENDS, /* move up/down lines (wrapped lines) */
@@ -254,7 +265,15 @@ typedef enum
   GTK_SCROLL_STEP_FORWARD,
   GTK_SCROLL_PAGE_BACKWARD,
   GTK_SCROLL_PAGE_FORWARD,
-  GTK_SCROLL_JUMP
+  GTK_SCROLL_JUMP,
+  GTK_SCROLL_STEP_UP,
+  GTK_SCROLL_STEP_DOWN,
+  GTK_SCROLL_PAGE_UP,
+  GTK_SCROLL_PAGE_DOWN,
+  GTK_SCROLL_STEP_LEFT,
+  GTK_SCROLL_STEP_RIGHT,
+  GTK_SCROLL_PAGE_LEFT,
+  GTK_SCROLL_PAGE_RIGHT
 } GtkScrollType;
 
 /* list selection modes */
@@ -348,7 +367,6 @@ typedef enum
 typedef enum
 {
   GTK_WINDOW_TOPLEVEL,
-  GTK_WINDOW_DIALOG,
   GTK_WINDOW_POPUP
 } GtkWindowType;
 
