@@ -66,6 +66,7 @@
 #include "gtkoptionmenu.h"
 #include "gtkclist.h"
 #include "gtkdialog.h"
+#include "gtkmessagedialog.h"
 #include "gtkintl.h"
 
 #if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
@@ -89,6 +90,18 @@
 #define FILE_LIST_WIDTH  180
 #define FILE_LIST_HEIGHT 180
 
+/* The Hurd doesn't define either PATH_MAX or MAXPATHLEN, so we put this
+ * in here, since the rest of the code in the file does require some
+ * fixed maximum.
+ */
+#ifndef MAXPATHLEN
+#  ifdef PATH_MAX
+#    define MAXPATHLEN PATH_MAX
+#  else
+#    define MAXPATHLEN 2048
+#  endif
+#endif
+
 /* I've put this here so it doesn't get confused with the 
  * file completion interface */
 typedef struct _HistoryCallbackArg HistoryCallbackArg;
@@ -122,6 +135,7 @@ typedef struct _PossibleCompletion PossibleCompletion;
 #define FNMATCH_FLAGS (FNM_PATHNAME | FNM_PERIOD)
 
 #define CMPL_ERRNO_TOO_LONG ((1<<16)-1)
+#define CMPL_ERRNO_DID_NOT_CONVERT ((1<<16)-2)
 
 /* This structure contains all the useful information about a directory
  * for the purposes of filename completion.  These structures are cached
@@ -211,6 +225,11 @@ struct _CompletionState
   struct _CompletionUserDir *user_directories;
 };
 
+enum {
+  PROP_0,
+  PROP_SHOW_FILEOPS,
+  PROP_FILENAME
+};
 
 /* File completion functions which would be external, were they used
  * outside of this file.
@@ -325,11 +344,24 @@ static void update_cmpl(PossibleCompletion* poss,
 			CompletionState* cmpl_state);
 
 static void gtk_file_selection_class_init    (GtkFileSelectionClass *klass);
+static void gtk_file_selection_set_property  (GObject         *object,
+					      guint            prop_id,
+					      const GValue    *value,
+					      GParamSpec      *pspec);
+static void gtk_file_selection_get_property  (GObject         *object,
+					      guint            prop_id,
+					      GValue          *value,
+					      GParamSpec      *pspec);
 static void gtk_file_selection_init          (GtkFileSelection      *filesel);
 static void gtk_file_selection_destroy       (GtkObject             *object);
 static gint gtk_file_selection_key_press     (GtkWidget             *widget,
 					      GdkEventKey           *event,
 					      gpointer               user_data);
+static gint gtk_file_selection_insert_text   (GtkWidget             *widget,
+					      const gchar           *new_text,
+					      gint                   new_text_length,
+					      gint                  *position,
+					      gpointer               user_data);
 
 static void gtk_file_selection_file_button (GtkWidget *widget,
 					    gint row, 
@@ -453,15 +485,94 @@ gtk_file_selection_get_type (void)
 static void
 gtk_file_selection_class_init (GtkFileSelectionClass *class)
 {
+  GObjectClass *gobject_class;
   GtkObjectClass *object_class;
 
+  gobject_class = (GObjectClass*) class;
   object_class = (GtkObjectClass*) class;
 
   parent_class = gtk_type_class (GTK_TYPE_DIALOG);
 
+  gobject_class->set_property = gtk_file_selection_set_property;
+  gobject_class->get_property = gtk_file_selection_get_property;
+   
+  g_object_class_install_property (gobject_class,
+                                   PROP_FILENAME,
+                                   g_param_spec_string ("filename",
+                                                        _("Filename"),
+                                                        _("The currently selected filename."),
+                                                        NULL,
+                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+				   PROP_SHOW_FILEOPS,
+				   g_param_spec_boolean ("show_fileops",
+							 _("Show file operations"),
+							 _("Whether buttons for creating/manipulating files should be displayed."),
+							 FALSE,
+							 G_PARAM_READABLE |
+							 G_PARAM_WRITABLE));
   object_class->destroy = gtk_file_selection_destroy;
 }
 
+static void gtk_file_selection_set_property (GObject         *object,
+					     guint            prop_id,
+					     const GValue    *value,
+					     GParamSpec      *pspec)
+{
+  GtkFileSelection *filesel;
+
+  filesel = GTK_FILE_SELECTION (object);
+
+  switch (prop_id)
+    {
+    case PROP_FILENAME:
+      gtk_file_selection_set_filename (filesel,
+                                       g_value_get_string (value));
+      break;
+      
+    case PROP_SHOW_FILEOPS:
+      if (g_value_get_boolean (value))
+	 gtk_file_selection_show_fileop_buttons (filesel);
+      else
+	 gtk_file_selection_hide_fileop_buttons (filesel);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void gtk_file_selection_get_property (GObject         *object,
+					     guint            prop_id,
+					     GValue          *value,
+					     GParamSpec      *pspec)
+{
+  GtkFileSelection *filesel;
+
+  filesel = GTK_FILE_SELECTION (object);
+
+  switch (prop_id)
+    {
+    case PROP_FILENAME:
+      g_value_set_string (value,
+                          gtk_file_selection_get_filename(filesel));
+      break;
+
+    case PROP_SHOW_FILEOPS:
+      /* This is a little bit hacky, but doing otherwise would require
+       * adding a field to the object.
+       */
+      g_value_set_boolean (value, (filesel->fileop_c_dir && 
+				   filesel->fileop_del_file &&
+				   filesel->fileop_ren_file));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 gtk_file_selection_init (GtkFileSelection *filesel)
 {
@@ -477,7 +588,7 @@ gtk_file_selection_init (GtkFileSelection *filesel)
   char *file_title [2];
 
   dialog = GTK_DIALOG (filesel);
-  
+
   filesel->cmpl_state = cmpl_init_state ();
 
   /* The dialog-sized vertical box  */
@@ -487,7 +598,7 @@ gtk_file_selection_init (GtkFileSelection *filesel)
   /* The horizontal box containing create, rename etc. buttons */
   filesel->button_area = gtk_hbutton_box_new ();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (filesel->button_area), GTK_BUTTONBOX_START);
-  gtk_button_box_set_spacing (GTK_BUTTON_BOX (filesel->button_area), 0);
+  gtk_box_set_spacing (GTK_BOX (filesel->button_area), 0);
   gtk_box_pack_start (GTK_BOX (filesel->main_vbox), filesel->button_area, 
 		      FALSE, FALSE, 0);
   gtk_widget_show (filesel->button_area);
@@ -584,6 +695,8 @@ gtk_file_selection_init (GtkFileSelection *filesel)
   filesel->selection_entry = gtk_entry_new ();
   gtk_signal_connect (GTK_OBJECT (filesel->selection_entry), "key_press_event",
 		      (GtkSignalFunc) gtk_file_selection_key_press, filesel);
+  gtk_signal_connect (GTK_OBJECT (filesel->selection_entry), "insert_text",
+		      (GtkSignalFunc) gtk_file_selection_insert_text, NULL);
   gtk_signal_connect_object (GTK_OBJECT (filesel->selection_entry), "focus_in_event",
 			     (GtkSignalFunc) gtk_widget_grab_default,
 			     GTK_OBJECT (filesel->ok_button));
@@ -616,6 +729,7 @@ gtk_file_selection_new (const gchar *title)
 
   filesel = gtk_type_new (GTK_TYPE_FILE_SELECTION);
   gtk_window_set_title (GTK_WINDOW (filesel), title);
+  gtk_dialog_set_has_separator (GTK_DIALOG (filesel), FALSE);
 
   return GTK_WIDGET (filesel);
 }
@@ -659,7 +773,7 @@ gtk_file_selection_show_fileop_buttons (GtkFileSelection *filesel)
 			  filesel->fileop_ren_file, TRUE, TRUE, 0);
       gtk_widget_show (filesel->fileop_ren_file);
     }
-
+  g_object_notify (G_OBJECT (filesel), "show_fileops");
   gtk_widget_queue_resize (GTK_WIDGET (filesel));
 }
 
@@ -686,6 +800,7 @@ gtk_file_selection_hide_fileop_buttons (GtkFileSelection *filesel)
       gtk_widget_destroy (filesel->fileop_c_dir);
       filesel->fileop_c_dir = NULL;
     }
+  g_object_notify (G_OBJECT (filesel), "show_fileops");
 }
 
 
@@ -720,9 +835,22 @@ gtk_file_selection_set_filename (GtkFileSelection *filesel,
   if (filesel->selection_entry)
     gtk_entry_set_text (GTK_ENTRY (filesel->selection_entry), name);
   g_free (buf);
+  g_object_notify (G_OBJECT (filesel), "filename");
 }
 
-gchar*
+/**
+ * gtk_file_selection_get_filename:
+ * @filesel: a #GtkFileSelection
+ * 
+ * This function returns the selected filename in the C runtime's
+ * multibyte string encoding, which may or may not be the same as that
+ * used by GTK+ (UTF-8). To convert to UTF-8, call g_filename_to_utf8().
+ * The returned string points to a statically allocated buffer and
+ * should be copied if you plan to keep it around.
+ * 
+ * Return value: currently-selected filename in locale's encoding
+ **/
+G_CONST_RETURN gchar*
 gtk_file_selection_get_filename (GtkFileSelection *filesel)
 {
   static gchar nothing[2] = "";
@@ -739,7 +867,9 @@ gtk_file_selection_get_filename (GtkFileSelection *filesel)
   text = gtk_entry_get_text (GTK_ENTRY (filesel->selection_entry));
   if (text)
     {
-      sys_filename = g_filename_from_utf8 (cmpl_completion_fullname (text, filesel->cmpl_state), NULL);
+      sys_filename = g_filename_from_utf8 (cmpl_completion_fullname (text, filesel->cmpl_state), -1, NULL, NULL, NULL);
+      if (!sys_filename)
+	return nothing;
       strncpy (something, sys_filename, sizeof (something));
       g_free (sys_filename);
       return something;
@@ -807,52 +937,25 @@ static void
 gtk_file_selection_fileop_error (GtkFileSelection *fs,
 				 gchar            *error_message)
 {
-  GtkWidget *label;
-  GtkWidget *vbox;
-  GtkWidget *button;
   GtkWidget *dialog;
-  
+    
   g_return_if_fail (error_message != NULL);
-  
+
   /* main dialog */
-  dialog = gtk_dialog_new ();
-  /*
-  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-		      (GtkSignalFunc) gtk_file_selection_fileop_destroy, 
-		      (gpointer) fs);
-  */
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-  
-  /* If file dialog is grabbed, make this dialog modal too */
-  /* When error dialog is closed, file dialog will be grabbed again */
-  if (GTK_WINDOW (fs)->modal)
-      gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-
-  vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox,
-		     FALSE, FALSE, 0);
-  gtk_widget_show (vbox);
-
-  label = gtk_label_new (error_message);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
-  gtk_widget_show (label);
+  dialog = gtk_message_dialog_new (GTK_WINDOW (fs),
+				   GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_MESSAGE_ERROR,
+				   GTK_BUTTONS_CLOSE,
+				   "%s", error_message);
 
   /* yes, we free it */
   g_free (error_message);
-  
-  /* close button */
-  button = gtk_button_new_with_label (_("Close"));
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+
+  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+
+  gtk_signal_connect_object (GTK_OBJECT (dialog), "response",
 			     (GtkSignalFunc) gtk_widget_destroy, 
 			     (gpointer) dialog);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-		     button, TRUE, TRUE, 0);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
 
   gtk_widget_show (dialog);
 }
@@ -880,6 +983,7 @@ gtk_file_selection_create_dir_confirmed (GtkWidget *widget,
   gchar *full_path;
   gchar *sys_full_path;
   gchar *buf;
+  GError *error = NULL;
   CompletionState *cmpl_state;
   
   g_return_if_fail (fs != NULL);
@@ -890,13 +994,27 @@ gtk_file_selection_create_dir_confirmed (GtkWidget *widget,
   path = cmpl_reference_position (cmpl_state);
   
   full_path = g_strconcat (path, G_DIR_SEPARATOR_S, dirname, NULL);
-  sys_full_path = g_filename_from_utf8 (full_path, NULL);
+  sys_full_path = g_filename_from_utf8 (full_path, -1, NULL, NULL, &error);
+  if (error)
+    {
+      if (g_error_matches (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE))
+	buf = g_strdup_printf (_("The directory name \"%s\" contains symbols that are not allowed in filenames"), dirname);
+      else
+	buf = g_strdup_printf (_("Error creating directory \"%s\": %s\n%s"), dirname, error->message,
+			       _("You probably used symbols not allowed in filenames."));
+      gtk_file_selection_fileop_error (fs, buf);
+      g_error_free (error);
+      goto out;
+    }
+
   if (mkdir (sys_full_path, 0755) < 0) 
     {
-      buf = g_strconcat ("Error creating directory \"", dirname, "\":  ", 
-			 g_strerror (errno), NULL);
+      buf = g_strdup_printf (_("Error creating directory \"%s\": %s\n"), dirname,
+			     g_strerror (errno));
       gtk_file_selection_fileop_error (fs, buf);
     }
+
+ out:
   g_free (full_path);
   g_free (sys_full_path);
   
@@ -940,13 +1058,14 @@ gtk_file_selection_create_dir (GtkWidget *widget,
 		     FALSE, FALSE, 0);
   gtk_widget_show( vbox);
   
-  label = gtk_label_new (_("Directory name:"));
+  label = gtk_label_new_with_mnemonic (_("_Directory name:"));
   gtk_misc_set_alignment(GTK_MISC (label), 0.0, 0.0);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
   gtk_widget_show (label);
 
   /*  The directory entry widget  */
   fs->fileop_entry = gtk_entry_new ();
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), fs->fileop_entry);
   gtk_box_pack_start (GTK_BOX (vbox), fs->fileop_entry, 
 		      TRUE, TRUE, 5);
   GTK_WIDGET_SET_FLAGS (fs->fileop_entry, GTK_CAN_DEFAULT);
@@ -984,6 +1103,7 @@ gtk_file_selection_delete_file_confirmed (GtkWidget *widget,
   gchar *path;
   gchar *full_path;
   gchar *sys_full_path;
+  GError *error = NULL;
   gchar *buf;
   
   g_return_if_fail (fs != NULL);
@@ -993,13 +1113,30 @@ gtk_file_selection_delete_file_confirmed (GtkWidget *widget,
   path = cmpl_reference_position (cmpl_state);
   
   full_path = g_strconcat (path, G_DIR_SEPARATOR_S, fs->fileop_file, NULL);
-  sys_full_path = g_filename_from_utf8 (full_path, NULL);
+  sys_full_path = g_filename_from_utf8 (full_path, -1, NULL, NULL, &error);
+  if (error)
+    {
+      if (g_error_matches (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE))
+	buf = g_strdup_printf (_("The filename \"%s\" contains symbols that are not allowed in filenames"),
+			       fs->fileop_file);
+      else
+	buf = g_strdup_printf (_("Error deleting file \"%s\": %s\n%s"),
+			       fs->fileop_file, error->message,
+			       _("It probably contains symbols not allowed in filenames."));
+      
+      gtk_file_selection_fileop_error (fs, buf);
+      g_error_free (error);
+      goto out;
+    }
+
   if (unlink (sys_full_path) < 0) 
     {
-      buf = g_strconcat ("Error deleting file \"", fs->fileop_file, "\":  ", 
-			 g_strerror (errno), NULL);
+      buf = g_strdup_printf (_("Error deleting file \"%s\": %s"),
+			     fs->fileop_file, g_strerror (errno));
       gtk_file_selection_fileop_error (fs, buf);
     }
+  
+ out:
   g_free (full_path);
   g_free (sys_full_path);
   
@@ -1098,6 +1235,7 @@ gtk_file_selection_rename_file_confirmed (GtkWidget *widget,
   gchar *sys_new_filename;
   gchar *sys_old_filename;
   CompletionState *cmpl_state;
+  GError *error = NULL;
   
   g_return_if_fail (fs != NULL);
   g_return_if_fail (GTK_IS_FILE_SELECTION (fs));
@@ -1109,19 +1247,49 @@ gtk_file_selection_rename_file_confirmed (GtkWidget *widget,
   new_filename = g_strconcat (path, G_DIR_SEPARATOR_S, file, NULL);
   old_filename = g_strconcat (path, G_DIR_SEPARATOR_S, fs->fileop_file, NULL);
 
-  sys_new_filename = g_filename_from_utf8 (new_filename, NULL);
-  sys_old_filename = g_filename_from_utf8 (old_filename, NULL);
+  sys_new_filename = g_filename_from_utf8 (new_filename, -1, NULL, NULL, &error);
+  if (error)
+    {
+      if (g_error_matches (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE))
+	buf = g_strdup_printf (_("The file name \"%s\" contains symbols that are not allowed in filenames"), new_filename);
+      else
+	buf = g_strdup_printf (_("Error renaming file to \"%s\": %s\n%s"),
+			       new_filename, error->message,
+			       _("You probably used symbols not allowed in filenames."));
+      gtk_file_selection_fileop_error (fs, buf);
+      g_error_free (error);
+      goto out1;
+    }
 
+  sys_old_filename = g_filename_from_utf8 (old_filename, -1, NULL, NULL, &error);
+  if (error)
+    {
+      if (g_error_matches (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE))
+	buf = g_strdup_printf (_("The file name \"%s\" contains symbols that are not allowed in filenames"), old_filename);
+      else
+	buf = g_strdup_printf (_("Error renaming file \"%s\": %s\n%s"),
+			       old_filename, error->message,
+			       _("It probably contains symbols not allowed in filenames."));
+      gtk_file_selection_fileop_error (fs, buf);
+      g_error_free (error);
+      goto out2;
+    }
+  
   if (rename (sys_old_filename, sys_new_filename) < 0) 
     {
-      buf = g_strconcat ("Error renaming file \"", file, "\":  ", 
-			 g_strerror (errno), NULL);
+      buf = g_strdup_printf (_("Error renaming file \"%s\" to \"%s\": %s"),
+			     sys_old_filename, sys_new_filename,
+			     g_strerror (errno));
       gtk_file_selection_fileop_error (fs, buf);
     }
+  
+ out2:
+  g_free (sys_old_filename);
+
+ out1:
   g_free (new_filename);
   g_free (old_filename);
   g_free (sys_new_filename);
-  g_free (sys_old_filename);
   
   gtk_widget_destroy (fs->fileop_dialog);
   gtk_file_selection_populate (fs, "", FALSE);
@@ -1208,6 +1376,28 @@ gtk_file_selection_rename_file (GtkWidget *widget,
   gtk_widget_show (dialog);
 }
 
+static gint
+gtk_file_selection_insert_text (GtkWidget   *widget,
+				const gchar *new_text,
+				gint         new_text_length,
+				gint        *position,
+				gpointer     user_data)
+{
+  gchar *filename;
+
+  filename = g_filename_from_utf8 (new_text, new_text_length, NULL, NULL, NULL);
+
+  if (!filename)
+    {
+      gdk_beep ();
+      gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "insert_text");
+      return FALSE;
+    }
+  
+  g_free (filename);
+  
+  return TRUE;
+}
 
 static gint
 gtk_file_selection_key_press (GtkWidget   *widget,
@@ -1757,7 +1947,7 @@ cmpl_init_state (void)
 
   /* g_get_current_dir() returns a string in the "system" charset */
   sys_getcwd_buf = g_get_current_dir ();
-  utf8_cwd = g_filename_to_utf8 (sys_getcwd_buf, NULL);
+  utf8_cwd = g_filename_to_utf8 (sys_getcwd_buf, -1, NULL, NULL, NULL);
   g_free (sys_getcwd_buf);
 
 tryagain:
@@ -2049,7 +2239,7 @@ open_ref_dir (gchar           *text_to_complete,
 	{
 	  /* If no possible candidates, use the cwd */
 	  gchar *sys_curdir = g_get_current_dir ();
-	  gchar *utf8_curdir = g_filename_to_utf8 (sys_curdir, NULL);
+	  gchar *utf8_curdir = g_filename_to_utf8 (sys_curdir, -1, NULL, NULL, NULL);
 
 	  g_free (sys_curdir);
 
@@ -2103,7 +2293,7 @@ open_user_dir (gchar           *text_to_complete,
     {
       /* ~/ */
       gchar *homedir = g_get_home_dir ();
-      gchar *utf8_homedir = g_filename_to_utf8 (homedir, NULL);
+      gchar *utf8_homedir = g_filename_to_utf8 (homedir, -1, NULL, NULL, NULL);
 
       g_free (homedir);
 
@@ -2130,7 +2320,7 @@ open_user_dir (gchar           *text_to_complete,
 	  cmpl_errno = errno;
 	  return NULL;
 	}
-      utf8_dir = g_filename_to_utf8 (pwd->pw_dir, NULL);
+      utf8_dir = g_filename_to_utf8 (pwd->pw_dir, -1, NULL, NULL, NULL);
       result = open_dir (utf8_dir, cmpl_state);
       g_free (utf8_dir);
     }
@@ -2173,6 +2363,7 @@ open_new_dir (gchar       *dir_name,
   DIR *directory;
   struct dirent *dirent_ptr;
   gint entry_count = 0;
+  gint n_entries = 0;
   gint i;
   struct stat ent_sbuf;
   GString *path;
@@ -2185,9 +2376,14 @@ open_new_dir (gchar       *dir_name,
 
   path = g_string_sized_new (2*MAXPATHLEN + 10);
 
-  sys_dir_name = g_filename_from_utf8 (dir_name, NULL);
+  sys_dir_name = g_filename_from_utf8 (dir_name, -1, NULL, NULL, NULL);
+  if (!sys_dir_name)
+    {
+      cmpl_errno = CMPL_ERRNO_DID_NOT_CONVERT;
+      return NULL;
+    }
+  
   directory = opendir (sys_dir_name);
-
   if (!directory)
     {
       cmpl_errno = errno;
@@ -2215,7 +2411,12 @@ open_new_dir (gchar       *dir_name,
 	  return NULL;
 	}
 
-      sent->entries[i].entry_name = g_filename_to_utf8 (dirent_ptr->d_name, NULL);
+      sent->entries[n_entries].entry_name = g_filename_to_utf8 (dirent_ptr->d_name, -1, NULL, NULL, NULL);
+      if (!g_utf8_validate (sent->entries[n_entries].entry_name, -1, NULL))
+	{
+	  g_warning (_("The filename %s couldn't be converted to UTF-8. Try setting the environment variable G_BROKEN_FILENAMES."), dirent_ptr->d_name);
+	  continue;
+	}
 
       g_string_assign (path, sys_dir_name);
       if (path->str[path->len-1] != G_DIR_SEPARATOR)
@@ -2228,16 +2429,19 @@ open_new_dir (gchar       *dir_name,
 	{
 	  /* Here we know path->str is a "system charset" string */
 	  if (stat (path->str, &ent_sbuf) >= 0 && S_ISDIR (ent_sbuf.st_mode))
-	    sent->entries[i].is_dir = TRUE;
+	    sent->entries[n_entries].is_dir = TRUE;
 	  else
 	    /* stat may fail, and we don't mind, since it could be a
 	     * dangling symlink. */
-	    sent->entries[i].is_dir = FALSE;
+	    sent->entries[n_entries].is_dir = FALSE;
 	}
       else
-	sent->entries[i].is_dir = 1;
-    }
+	sent->entries[n_entries].is_dir = 1;
 
+      n_entries++;
+    }
+  sent->entry_count = n_entries;
+  
   g_free (sys_dir_name);
   g_string_free (path, TRUE);
   qsort (sent->entries, sent->entry_count, sizeof (CompletionDirEntry), compare_cmpl_dir);
@@ -2283,7 +2487,13 @@ check_dir (gchar       *dir_name,
 	}
     }
 
-  sys_dir_name = g_filename_from_utf8 (dir_name, NULL);
+  sys_dir_name = g_filename_from_utf8 (dir_name, -1, NULL, NULL, NULL);
+  if (!sys_dir_name)
+    {
+      cmpl_errno = CMPL_ERRNO_DID_NOT_CONVERT;
+      return FALSE;
+    }
+  
   if (stat (sys_dir_name, result) < 0)
     {
       g_free (sys_dir_name);
@@ -2416,7 +2626,13 @@ correct_dir_fullname (CompletionDir* cmpl_dir)
 	  return TRUE;
 	}
 
-      sys_filename = g_filename_from_utf8 (cmpl_dir->fullname, NULL);
+      sys_filename = g_filename_from_utf8 (cmpl_dir->fullname, -1, NULL, NULL, NULL);
+      if (!sys_filename)
+	{
+	  cmpl_errno = CMPL_ERRNO_DID_NOT_CONVERT;
+	  return FALSE;
+	}
+      
       if (stat (sys_filename, &sbuf) < 0)
 	{
 	  g_free (sys_filename);
@@ -2444,7 +2660,13 @@ correct_dir_fullname (CompletionDir* cmpl_dir)
 	  return TRUE;
 	}
 
-      sys_filename = g_filename_from_utf8 (cmpl_dir->fullname, NULL);
+      sys_filename = g_filename_from_utf8 (cmpl_dir->fullname, -1, NULL, NULL, NULL);
+      if (!sys_filename)
+	{
+	  cmpl_errno = CMPL_ERRNO_DID_NOT_CONVERT;
+	  return FALSE;
+	}
+      
       if (stat (sys_filename, &sbuf) < 0)
 	{
 	  g_free (sys_filename);
@@ -2492,7 +2714,15 @@ correct_parent (CompletionDir *cmpl_dir,
       last_slash[1] = 0;
     }
 
-  sys_filename = g_filename_from_utf8 (cmpl_dir->fullname, NULL);
+  sys_filename = g_filename_from_utf8 (cmpl_dir->fullname, -1, NULL, NULL, NULL);
+  if (!sys_filename)
+    {
+      cmpl_errno = CMPL_ERRNO_DID_NOT_CONVERT;
+      if (!c)
+	last_slash[0] = G_DIR_SEPARATOR;
+      return FALSE;
+    }
+  
   if (stat (sys_filename, &parbuf) < 0)
     {
       g_free (sys_filename);
@@ -2539,7 +2769,14 @@ find_parent_dir_fullname (gchar* dirname)
   gchar *sys_dirname;
 
   sys_orig_dir = g_get_current_dir ();
-  sys_dirname = g_filename_from_utf8 (dirname, NULL);
+  sys_dirname = g_filename_from_utf8 (dirname, -1, NULL, NULL, NULL);
+  if (!sys_dirname)
+    {
+      g_free (sys_orig_dir);
+      cmpl_errno = CMPL_ERRNO_DID_NOT_CONVERT;
+      return NULL;
+    }
+  
   if (chdir (sys_dirname) != 0 || chdir ("..") != 0)
     {
       g_free (sys_dirname);
@@ -2550,7 +2787,7 @@ find_parent_dir_fullname (gchar* dirname)
   g_free (sys_dirname);
 
   sys_cwd = g_get_current_dir ();
-  result = g_filename_to_utf8 (sys_cwd, NULL);
+  result = g_filename_to_utf8 (sys_cwd, -1, NULL, NULL, NULL);
   g_free (sys_cwd);
 
   if (chdir (sys_orig_dir) != 0)
@@ -2944,10 +3181,10 @@ get_pwdb (CompletionState* cmpl_state)
 
   while ((pwd_ptr = getpwent ()) != NULL)
     {
-      utf8 = g_filename_to_utf8 (pwd_ptr->pw_name, NULL);
+      utf8 = g_filename_to_utf8 (pwd_ptr->pw_name, -1, NULL, NULL, NULL);
       len += strlen (utf8);
       g_free (utf8);
-      utf8 = g_filename_to_utf8 (pwd_ptr->pw_dir, NULL);
+      utf8 = g_filename_to_utf8 (pwd_ptr->pw_dir, -1, NULL, NULL, NULL);
       len += strlen (utf8);
       g_free (utf8);
       len += 2;
@@ -2971,7 +3208,7 @@ get_pwdb (CompletionState* cmpl_state)
 	  goto error;
 	}
 
-      utf8 = g_filename_to_utf8 (pwd_ptr->pw_name, NULL);
+      utf8 = g_filename_to_utf8 (pwd_ptr->pw_name, -1, NULL, NULL, NULL);
       strcpy (buf_ptr, utf8);
       g_free (utf8);
 
@@ -2980,7 +3217,7 @@ get_pwdb (CompletionState* cmpl_state)
       buf_ptr += strlen (buf_ptr);
       buf_ptr += 1;
 
-      utf8 = g_filename_to_utf8 (pwd_ptr->pw_dir, NULL);
+      utf8 = g_filename_to_utf8 (pwd_ptr->pw_dir, -1, NULL, NULL, NULL);
       strcpy (buf_ptr, utf8);
       g_free (utf8);
 
@@ -3045,7 +3282,9 @@ static gchar*
 cmpl_strerror (gint err)
 {
   if (err == CMPL_ERRNO_TOO_LONG)
-    return "Name too long";
+    return _("Name too long");
+  else if (err == CMPL_ERRNO_DID_NOT_CONVERT)
+    return _("Couldn't convert filename");
   else
     return g_strerror (err);
 }
