@@ -70,10 +70,11 @@ struct _GtkFrameClass
 
 GtkType    gtk_frame_get_type         (void) G_GNUC_CONST;
 GtkWidget* gtk_frame_new              (const gchar   *label);
-void       gtk_frame_set_label        (GtkFrame      *frame,
-				       const gchar   *label);
-/* Temporarily commented out until memory management behavior is figured out */
-/* gchar *    gtk_frame_get_label        (GtkFrame      *frame); */
+
+void                  gtk_frame_set_label (GtkFrame    *frame,
+					   const gchar *label);
+G_CONST_RETURN gchar *gtk_frame_get_label (GtkFrame    *frame);
+
 void       gtk_frame_set_label_widget (GtkFrame      *frame,
 				       GtkWidget     *label_widget);
 void       gtk_frame_set_label_align  (GtkFrame      *frame,
