@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <string.h> /* memset */
+
 #include "gtklist.h"
 #include "gtklistitem.h"
 #include "gtkmain.h"
@@ -66,10 +68,6 @@ static void gtk_list_map	     (GtkWidget	     *widget);
 static void gtk_list_unmap	     (GtkWidget	     *widget);
 static void gtk_list_style_set	     (GtkWidget      *widget,
 				      GtkStyle       *previous_style);
-static void gtk_list_draw	     (GtkWidget	     *widget,
-				      GdkRectangle   *area);
-static gint gtk_list_expose	     (GtkWidget	     *widget,
-				      GdkEventExpose *event);
 static gint gtk_list_motion_notify   (GtkWidget      *widget,
 				      GdkEventMotion *event);
 static gint gtk_list_button_press    (GtkWidget      *widget,
@@ -216,31 +214,6 @@ gtk_list_class_init (GtkListClass *class)
 
   gobject_class->shutdown = gtk_list_shutdown;
 
-  list_signals[SELECTION_CHANGED] =
-    gtk_signal_new ("selection_changed",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkListClass, selection_changed),
-		    gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
-  list_signals[SELECT_CHILD] =
-    gtk_signal_new ("select_child",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkListClass, select_child),
-		    gtk_marshal_VOID__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
-  list_signals[UNSELECT_CHILD] =
-    gtk_signal_new ("unselect_child",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkListClass, unselect_child),
-		    gtk_marshal_VOID__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
-
-  gtk_object_class_add_signals (object_class, list_signals, LAST_SIGNAL);
 
   object_class->set_arg = gtk_list_set_arg;
   object_class->get_arg = gtk_list_get_arg;
@@ -249,8 +222,6 @@ gtk_list_class_init (GtkListClass *class)
   widget_class->unmap = gtk_list_unmap;
   widget_class->style_set = gtk_list_style_set;
   widget_class->realize = gtk_list_realize;
-  widget_class->draw = gtk_list_draw;
-  widget_class->expose_event = gtk_list_expose;
   widget_class->button_press_event = gtk_list_button_press;
   widget_class->button_release_event = gtk_list_button_release;
   widget_class->motion_notify_event = gtk_list_motion_notify;
@@ -268,6 +239,30 @@ gtk_list_class_init (GtkListClass *class)
   class->selection_changed = NULL;
   class->select_child = gtk_real_list_select_child;
   class->unselect_child = gtk_real_list_unselect_child;
+
+  list_signals[SELECTION_CHANGED] =
+    gtk_signal_new ("selection_changed",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkListClass, selection_changed),
+		    gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
+  list_signals[SELECT_CHILD] =
+    gtk_signal_new ("select_child",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkListClass, select_child),
+		    gtk_marshal_VOID__OBJECT,
+		    GTK_TYPE_NONE, 1,
+		    GTK_TYPE_WIDGET);
+  list_signals[UNSELECT_CHILD] =
+    gtk_signal_new ("unselect_child",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkListClass, unselect_child),
+		    gtk_marshal_VOID__OBJECT,
+		    GTK_TYPE_NONE, 1,
+		    GTK_TYPE_WIDGET);
   
   gtk_object_add_arg_type ("GtkList::selection_mode",
 			   GTK_TYPE_SELECTION_MODE, GTK_ARG_READWRITE,
@@ -587,7 +582,7 @@ gtk_list_motion_notify (GtkWidget      *widget,
 		(x > adj->value + adj->page_size &&
 		 adj->value >= adj->upper - adj->page_size)))
 	    {
-	      gfloat value;
+	      gdouble value;
 
 	      if (x < adj->value)
 		value = adj->value + (x - adj->value) / 2 - 1;
@@ -781,6 +776,8 @@ gtk_list_button_press (GtkWidget      *widget,
 	default:
 	  break;
 	}
+
+      return TRUE;
     }
 
   return FALSE;
@@ -833,74 +830,13 @@ gtk_list_button_release (GtkWidget	*widget,
 	default:
 	  break;
 	}
+
+      return TRUE;
     }
   
   return FALSE;
 }
 
-static void
-gtk_list_draw (GtkWidget    *widget,
-	       GdkRectangle *area)
-{
-  GtkList *list;
-  GtkWidget *child;
-  GdkRectangle child_area;
-  GList *children;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_LIST (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      list = GTK_LIST (widget);
-
-      children = list->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-
-	  if (gtk_widget_intersect (child, area, &child_area))
-	    gtk_widget_draw (child, &child_area);
-	}
-    }
-}
-
-static gint
-gtk_list_expose (GtkWidget	*widget,
-		 GdkEventExpose *event)
-{
-  GtkList *list;
-  GtkWidget *child;
-  GdkEventExpose child_event;
-  GList *children;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_LIST (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      list = GTK_LIST (widget);
-
-      child_event = *event;
-
-      children = list->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-
-	  if (GTK_WIDGET_NO_WINDOW (child) &&
-	      gtk_widget_intersect (child, &event->area, &child_event.area))
-	    gtk_widget_event (child, (GdkEvent*) &child_event);
-	}
-    }
-
-  return FALSE;
-}
-
 static void 
 gtk_list_style_set	(GtkWidget      *widget,
 			 GtkStyle       *previous_style)
@@ -2250,18 +2186,22 @@ gtk_list_scroll_horizontal (GtkList       *list,
 
   switch (scroll_type)
     {
+    case GTK_SCROLL_STEP_UP:
     case GTK_SCROLL_STEP_BACKWARD:
       adj->value = CLAMP (adj->value - adj->step_increment, adj->lower,
 			  adj->upper - adj->page_size);
       break;
+    case GTK_SCROLL_STEP_DOWN:
     case GTK_SCROLL_STEP_FORWARD:
       adj->value = CLAMP (adj->value + adj->step_increment, adj->lower,
 			  adj->upper - adj->page_size);
       break;
+    case GTK_SCROLL_PAGE_UP:
     case GTK_SCROLL_PAGE_BACKWARD:
       adj->value = CLAMP (adj->value - adj->page_increment, adj->lower,
 			  adj->upper - adj->page_size);
       break;
+    case GTK_SCROLL_PAGE_DOWN:
     case GTK_SCROLL_PAGE_FORWARD:
       adj->value = CLAMP (adj->value + adj->page_increment, adj->lower,
 			  adj->upper - adj->page_size);
@@ -2464,7 +2404,9 @@ gtk_list_move_focus_child (GtkList       *list,
 static gint
 gtk_list_horizontal_timeout (GtkWidget *list)
 {
-  GdkEventMotion event = { 0 };
+  GdkEventMotion event;
+
+  memset (&event, 0, sizeof (event));
 
   GDK_THREADS_ENTER ();
 
@@ -2483,7 +2425,9 @@ gtk_list_horizontal_timeout (GtkWidget *list)
 static gint
 gtk_list_vertical_timeout (GtkWidget *list)
 {
-  GdkEventMotion event = { 0 };
+  GdkEventMotion event;
+
+  memset (&event, 0, sizeof (event));
 
   GDK_THREADS_ENTER ();
 
@@ -2733,6 +2677,14 @@ gtk_list_signal_item_toggle (GtkListItem *list_item,
   g_return_if_fail (list != NULL);
   g_return_if_fail (GTK_IS_LIST (list));
 
+  if ((list->selection_mode == GTK_SELECTION_BROWSE ||
+       list->selection_mode == GTK_SELECTION_EXTENDED) &&
+      GTK_WIDGET (list_item)->state == GTK_STATE_NORMAL)
+    {
+      gtk_widget_set_state (GTK_WIDGET (list_item), GTK_STATE_SELECTED);
+      return;
+    }
+  
   switch (GTK_WIDGET (list_item)->state)
     {
     case GTK_STATE_SELECTED:
