@@ -56,17 +56,10 @@ static void gtk_list_item_size_allocate     (GtkWidget        *widget,
 					     GtkAllocation    *allocation);
 static void gtk_list_item_style_set         (GtkWidget        *widget,
 					     GtkStyle         *previous_style);
-static void gtk_list_item_draw              (GtkWidget        *widget,
-					     GdkRectangle     *area);
-static void gtk_list_item_draw_focus        (GtkWidget        *widget);
 static gint gtk_list_item_button_press      (GtkWidget        *widget,
 					     GdkEventButton   *event);
 static gint gtk_list_item_expose            (GtkWidget        *widget,
 					     GdkEventExpose   *event);
-static gint gtk_list_item_focus_in          (GtkWidget        *widget,
-					     GdkEventFocus    *event);
-static gint gtk_list_item_focus_out         (GtkWidget        *widget,
-					     GdkEventFocus    *event);
 static void gtk_real_list_item_select       (GtkItem          *item);
 static void gtk_real_list_item_deselect     (GtkItem          *item);
 static void gtk_real_list_item_toggle       (GtkItem          *item);
@@ -115,6 +108,28 @@ gtk_list_item_class_init (GtkListItemClass *class)
 
   parent_class = gtk_type_class (gtk_item_get_type ());
 
+  widget_class->realize = gtk_list_item_realize;
+  widget_class->size_request = gtk_list_item_size_request;
+  widget_class->size_allocate = gtk_list_item_size_allocate;
+  widget_class->style_set = gtk_list_item_style_set;
+  widget_class->button_press_event = gtk_list_item_button_press;
+  widget_class->expose_event = gtk_list_item_expose;
+
+  item_class->select = gtk_real_list_item_select;
+  item_class->deselect = gtk_real_list_item_deselect;
+  item_class->toggle = gtk_real_list_item_toggle;
+
+  class->toggle_focus_row = NULL;
+  class->select_all = NULL;
+  class->unselect_all = NULL;
+  class->undo_selection = NULL;
+  class->start_selection = NULL;
+  class->end_selection = NULL;
+  class->extend_selection = NULL;
+  class->scroll_horizontal = NULL;
+  class->scroll_vertical = NULL;
+  class->toggle_add_mode = NULL;
+
   list_item_signals[TOGGLE_FOCUS_ROW] =
     gtk_signal_new ("toggle_focus_row",
                     GTK_RUN_LAST | GTK_RUN_ACTION,
@@ -187,35 +202,6 @@ gtk_list_item_class_init (GtkListItemClass *class)
                     gtk_marshal_VOID__ENUM_FLOAT,
                     GTK_TYPE_NONE, 2, GTK_TYPE_SCROLL_TYPE, GTK_TYPE_FLOAT);
 
-  gtk_object_class_add_signals (object_class, list_item_signals, LAST_SIGNAL);
-
-  widget_class->realize = gtk_list_item_realize;
-  widget_class->size_request = gtk_list_item_size_request;
-  widget_class->size_allocate = gtk_list_item_size_allocate;
-  widget_class->style_set = gtk_list_item_style_set;
-  widget_class->draw = gtk_list_item_draw;
-  widget_class->draw_focus = gtk_list_item_draw_focus;
-  widget_class->button_press_event = gtk_list_item_button_press;
-  widget_class->expose_event = gtk_list_item_expose;
-  widget_class->focus_in_event = gtk_list_item_focus_in;
-  widget_class->focus_out_event = gtk_list_item_focus_out;
-
-  item_class->select = gtk_real_list_item_select;
-  item_class->deselect = gtk_real_list_item_deselect;
-  item_class->toggle = gtk_real_list_item_toggle;
-
-  class->toggle_focus_row = NULL;
-  class->select_all = NULL;
-  class->unselect_all = NULL;
-  class->undo_selection = NULL;
-  class->start_selection = NULL;
-  class->end_selection = NULL;
-  class->extend_selection = NULL;
-  class->scroll_horizontal = NULL;
-  class->scroll_vertical = NULL;
-  class->toggle_add_mode = NULL;
-
-
   binding_set = gtk_binding_set_by_class (class);
   gtk_binding_entry_add_signal (binding_set, GDK_Up, 0,
 				"scroll_vertical", 2,
@@ -456,64 +442,6 @@ gtk_list_item_style_set	(GtkWidget      *widget,
     gdk_window_set_background (widget->window, &widget->style->base[GTK_WIDGET_STATE (widget)]);
 }
 
-static void
-gtk_list_item_draw (GtkWidget    *widget,
-		    GdkRectangle *area)
-{
-  GtkBin *bin;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_LIST_ITEM (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      bin = GTK_BIN (widget);
-
-      if (widget->state == GTK_STATE_NORMAL)
-	{
-	  gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
-	  gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);
-	}
-      else
-	{
-	  gtk_paint_flat_box(widget->style, widget->window, 
-			     widget->state, GTK_SHADOW_ETCHED_OUT,
-			     area, widget, "listitem",
-			     0, 0, -1, -1);	      
-	}
-
-      if (bin->child && gtk_widget_intersect (bin->child, area, &child_area))
-	gtk_widget_draw (bin->child, &child_area);
-
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  if (GTK_IS_LIST (widget->parent) && GTK_LIST (widget->parent)->add_mode)
-	    gtk_paint_focus (widget->style, widget->window,
-			     NULL, widget, "add-mode",
-			     0, 0,
-			     widget->allocation.width - 1,
-			     widget->allocation.height - 1);
-	  else
-	    gtk_paint_focus (widget->style, widget->window,
-			     NULL, widget, NULL,
-			     0, 0,
-			     widget->allocation.width - 1,
-			     widget->allocation.height - 1);
-	}
-    }
-}
-
-static void
-gtk_list_item_draw_focus (GtkWidget *widget)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_LIST_ITEM (widget));
-  
-  gtk_widget_draw(widget, NULL);
-}
-
 static gint
 gtk_list_item_button_press (GtkWidget      *widget,
 			    GdkEventButton *event)
@@ -523,8 +451,12 @@ gtk_list_item_button_press (GtkWidget      *widget,
   g_return_val_if_fail (event != NULL, FALSE);
 
   if (event->type == GDK_BUTTON_PRESS)
-    if (!GTK_WIDGET_HAS_FOCUS (widget))
-      gtk_widget_grab_focus (widget);
+    {
+      if (!GTK_WIDGET_HAS_FOCUS (widget))
+	gtk_widget_grab_focus (widget);
+
+      return TRUE;
+    }
 
   return FALSE;
 }
@@ -534,69 +466,49 @@ gtk_list_item_expose (GtkWidget      *widget,
 		      GdkEventExpose *event)
 {
   GtkBin *bin;
-  GdkEventExpose child_event;
 
   g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_LIST_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       bin = GTK_BIN (widget);
 
-      if (!GTK_WIDGET_IS_SENSITIVE (widget))
-	gdk_window_set_background (widget->window, &widget->style->bg[GTK_STATE_INSENSITIVE]);
-      else if (widget->state == GTK_STATE_NORMAL)
-	gdk_window_set_background (widget->window, &widget->style->base[GTK_STATE_NORMAL]);
+      if (widget->state == GTK_STATE_NORMAL)
+        {
+          gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
+          gdk_window_clear_area (widget->window, event->area.x, event->area.y,
+                                 event->area.width, event->area.height);
+        }
       else
-	gdk_window_set_background (widget->window, &widget->style->bg[widget->state]);
+        {
+          gtk_paint_flat_box (widget->style, widget->window, 
+                              widget->state, GTK_SHADOW_ETCHED_OUT,
+                              &event->area, widget, "listitem",
+                              0, 0, -1, -1);           
+        }
 
-      gdk_window_clear_area (widget->window, event->area.x, event->area.y,
-			     event->area.width, event->area.height);
-
-      if (bin->child)
-	{
-	  child_event = *event;
-
-	  if (GTK_WIDGET_NO_WINDOW (bin->child) &&
-	      gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	    gtk_widget_event (bin->child, (GdkEvent*) &child_event);
-	}
-
-      gtk_widget_draw_focus (widget);
+      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+      
+      if (GTK_WIDGET_HAS_FOCUS (widget))
+        {
+          if (GTK_IS_LIST (widget->parent) && GTK_LIST (widget->parent)->add_mode)
+            gtk_paint_focus (widget->style, widget->window,
+                             NULL, widget, "add-mode",
+                             0, 0,
+                             widget->allocation.width - 1,
+                             widget->allocation.height - 1);
+          else
+            gtk_paint_focus (widget->style, widget->window,
+                             NULL, widget, NULL,
+                             0, 0,
+                             widget->allocation.width - 1,
+                             widget->allocation.height - 1);
+        }
     }
 
   return FALSE;
 }
 
-static gint
-gtk_list_item_focus_in (GtkWidget     *widget,
-			GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_LIST_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-  return FALSE;
-}
-
-static gint
-gtk_list_item_focus_out (GtkWidget     *widget,
-			 GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_LIST_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-  return FALSE;
-}
-
 static void
 gtk_real_list_item_select (GtkItem *item)
 {
