@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include "gdkconfig.h"
+
 #ifdef GDK_WINDOWING_X11
 #include <X11/Xlocale.h>	/* so we get the right setlocale */
 #else
@@ -43,6 +45,7 @@
 #include "gtkmain.h"
 #include "gtkrc.h"
 #include "gtkselection.h"
+#include "gtksettings.h"
 #include "gtksignal.h"
 #include "gtkwidget.h"
 #include "gtkwindow.h"
@@ -142,12 +145,12 @@ guint gtk_debug_flags = 0;		   /* Global GTK debug flag */
 
 #ifdef G_ENABLE_DEBUG
 static const GDebugKey gtk_debug_keys[] = {
-  {"objects", GTK_DEBUG_OBJECTS},
   {"misc", GTK_DEBUG_MISC},
-  {"signals", GTK_DEBUG_SIGNALS},
   {"dnd", GTK_DEBUG_DND},
   {"plugsocket", GTK_DEBUG_PLUGSOCKET},
-  {"text", GTK_DEBUG_TEXT}
+  {"text", GTK_DEBUG_TEXT},
+  {"tree", GTK_DEBUG_TREE},
+  {"updates", GTK_DEBUG_UPDATES}
 };
 
 static const guint gtk_ndebug_keys = sizeof (gtk_debug_keys) / sizeof (GDebugKey);
@@ -192,6 +195,51 @@ static gchar *add_dll_suffix(gchar *module_name)
 
 #undef gtk_init_check
 
+/* This checks to see if the process is running suid or sgid
+ * at the current time. If so, we don't allow GTK+ to be initialized.
+ * This is meant to be a mild check - we only error out if we
+ * can prove the programmer is doing something wrong, not if
+ * they could be doing something wrong. For this reason, we
+ * don't use issetugid() on BSD or prctl (PR_GET_DUMPABLE).
+ */
+static gboolean
+check_setugid (void)
+{
+/* this isn't at all relevant on Windoze and doesn't compile ... --hb */
+#ifndef G_OS_WIN32
+  uid_t ruid, euid, suid; /* Real, effective and saved user ID's */
+  gid_t rgid, egid, sgid; /* Real, effective and saved group ID's */
+  
+#ifdef HAVE_GETRESUID
+  /* These aren't in the header files, so we prototype them here.
+   */
+  int getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);
+  int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);
+
+  if (getresuid (&ruid, &euid, &suid) != 0 ||
+      getresgid (&rgid, &egid, &sgid) != 0)
+#endif /* HAVE_GETRESUID */
+    {
+      suid = ruid = getuid ();
+      sgid = rgid = getgid ();
+      euid = geteuid ();
+      egid = getegid ();
+    }
+
+  if (ruid != euid || ruid != suid ||
+      rgid != egid || rgid != sgid)
+    {
+      g_warning ("This process is currently running setuid or setgid.\n"
+		 "This is not a supported use of GTK+. You must create a helper\n"
+		 "program instead. For further details, see:\n\n"
+		 "    http://www.gtk.org/setuid.html\n\n"
+		 "Refusing to initialize GTK+.");
+      exit (1);
+    }
+#endif
+  return TRUE;
+}
+
 gboolean
 gtk_init_check (int	 *argc,
 		char   ***argv)
@@ -203,6 +251,9 @@ gtk_init_check (int	 *argc,
   if (gtk_initialized)
     return TRUE;
 
+  if (!check_setugid ())
+    return FALSE;
+  
 #if	0
   g_set_error_handler (gtk_error);
   g_set_warning_handler (gtk_warning);
@@ -345,6 +396,9 @@ gtk_init_check (int	 *argc,
 	    }
 	}
     }
+
+  if (gtk_debug_flags & GTK_DEBUG_UPDATES)
+    gdk_window_set_debug_updates (TRUE);
   
   /* load gtk modules */
   gtk_modules = g_slist_reverse (gtk_modules);
@@ -405,12 +459,10 @@ gtk_init_check (int	 *argc,
 #    endif
 #  else /* !G_OS_WIN32 */
   {
-    /* GTk+ locale dir is %WinDir%\gtk+\locale */
     bindtextdomain (GETTEXT_PACKAGE,
-		    g_strconcat (gtk_win32_get_installation_directory (),
-				 G_DIR_SEPARATOR_S,
-				 "locale",
-				 NULL));
+		    g_win32_get_package_installation_subdirectory (GETTEXT_PACKAGE,
+								   g_strdup_printf ("gtk-%d.%d.dll", GTK_MAJOR_VERSION, GTK_MINOR_VERSION),
+								   "locale"));
   }
 #endif
 #endif  
@@ -436,8 +488,7 @@ gtk_init_check (int	 *argc,
   gtk_visual = gdk_visual_get_system ();
   gtk_colormap = gdk_colormap_get_system ();
 
-  gtk_type_init ();
-  gtk_signal_init ();
+  gtk_type_init (0);
   gtk_rc_init ();
   
   
@@ -537,6 +588,17 @@ gtk_set_locale (void)
   return gdk_set_locale ();
 }
 
+/**
+ * gtk_get_default_language:
+ *
+ * Returns the ISO language code for the default language currently in
+ * effect. (Note that this can change over the life of an
+ * application.)  The default language is derived from the current
+ * locale. It determines, for example, whether GTK+ uses the
+ * right-to-left or left-to-right text direction.
+ * 
+ * Return value: the default language as an allocated string, must be freed
+ **/
 gchar*
 gtk_get_default_language (void)
 {
@@ -658,7 +720,7 @@ gtk_events_pending (void)
   return result;
 }
 
-gint 
+gboolean
 gtk_main_iteration (void)
 {
   GDK_THREADS_LEAVE ();
@@ -671,7 +733,7 @@ gtk_main_iteration (void)
     return TRUE;
 }
 
-gint 
+gboolean
 gtk_main_iteration_do (gboolean blocking)
 {
   GDK_THREADS_LEAVE ();
@@ -743,7 +805,9 @@ gtk_main_do_event (GdkEvent *event)
       if (event->type == GDK_PROPERTY_NOTIFY)
 	gtk_selection_incr_event (event->any.window,
 				  &event->property);
-      
+      else if (event->type == GDK_SETTING)
+	_gtk_settings_handle_event (&event->setting);
+
       return;
     }
   
@@ -810,9 +874,9 @@ gtk_main_do_event (GdkEvent *event)
       
     case GDK_EXPOSE:
       if (event->any.window && GTK_WIDGET_DOUBLE_BUFFERED (event_widget))
-	gdk_window_begin_paint_rect (event->any.window, &event->expose.area);
+	gdk_window_begin_paint_region (event->any.window, event->expose.region);
 
-      gtk_widget_event (event_widget, event);
+      gtk_widget_send_expose (event_widget, event);
 
       if (event->any.window && GTK_WIDGET_DOUBLE_BUFFERED (event_widget))
 	gdk_window_end_paint (event->any.window);
@@ -829,6 +893,7 @@ gtk_main_do_event (GdkEvent *event)
     case GDK_SELECTION_NOTIFY:
     case GDK_CLIENT_EVENT:
     case GDK_VISIBILITY_NOTIFY:
+    case GDK_WINDOW_STATE:
       gtk_widget_event (event_widget, event);
       break;
 
@@ -890,13 +955,13 @@ gtk_main_do_event (GdkEvent *event)
   g_list_free_1 (tmp_list);
 }
 
-gint
+gboolean
 gtk_true (void)
 {
   return TRUE;
 }
 
-gint
+gboolean
 gtk_false (void)
 {
   return FALSE;
@@ -1292,7 +1357,7 @@ gtk_invoke_input (gpointer	    data,
   args[0].type = GTK_TYPE_INT;
   args[0].name = NULL;
   GTK_VALUE_INT(args[0]) = source;
-  args[1].type = GTK_TYPE_GDK_INPUT_CONDITION;
+  args[1].type = GDK_TYPE_INPUT_CONDITION;
   args[1].name = NULL;
   GTK_VALUE_FLAGS(args[1]) = condition;
   args[2].type = GTK_TYPE_NONE;
@@ -1301,6 +1366,17 @@ gtk_invoke_input (gpointer	    data,
   closure->marshal (NULL, closure->data, 2, args);
 }
 
+/**
+ * gtk_get_current_event:
+ * 
+ * Obtains a copy of the event currently being processed by GTK+.  For
+ * example, if you get a "clicked" signal from #GtkButton, the current
+ * event will be the #GdkEventButton that triggered the "clicked"
+ * signal. The returned event must be freed with gdk_event_free().
+ * If there is no current event, the function returns %NULL.
+ * 
+ * Return value: a copy of the current event, or %NULL if no current event.
+ **/
 GdkEvent*
 gtk_get_current_event (void)
 {
@@ -1327,6 +1403,40 @@ gtk_get_current_event_time (void)
     return GDK_CURRENT_TIME;
 }
 
+/**
+ * gtk_get_current_event_state:
+ * @state: a location to store the state of the current event
+ * 
+ * If there is a current event and it has a state field, place
+ * that state field in @state and return %TRUE, otherwise return
+ * %FALSE.
+ * 
+ * Return value: %TRUE if there was a current event and it had a state field
+ **/
+gboolean
+gtk_get_current_event_state (GdkModifierType *state)
+{
+  g_return_val_if_fail (state != NULL, FALSE);
+  
+  if (current_events)
+    return gdk_event_get_state (current_events->data, state);
+  else
+    {
+      *state = 0;
+      return FALSE;
+    }
+}
+
+/**
+ * gtk_get_event_widget:
+ * @event: a #GdkEvent
+ *
+ * If @event is %NULL or the event was not associated with any widget,
+ * returns %NULL, otherwise returns the widget that received the event
+ * originally.
+ * 
+ * Return value: the widget that originally received @event, or %NULL
+ **/
 GtkWidget*
 gtk_get_event_widget (GdkEvent *event)
 {
@@ -1369,6 +1479,29 @@ gtk_quit_invoke_function (GtkQuitFunction *quitf)
     }
 }
 
+/**
+ * gtk_propagate_event:
+ * @widget: a #GtkWidget
+ * @event: an event
+ *
+ * Sends an event to a widget, propagating the event to parent widgets
+ * if the event remains unhandled. Events received by GTK+ from GDK
+ * normally begin in gtk_main_do_event(). Depending on the type of
+ * event, existence of modal dialogs, grabs, etc., the event may be
+ * propagated; if so, this function is used. gtk_propagate_event()
+ * calls gtk_widget_event() on each widget it decides to send the
+ * event to.  So gtk_widget_event() is the lowest-level function; it
+ * simply emits the "event" and possibly an event-specific signal on a
+ * widget.  gtk_propagate_event() is a bit higher-level, and
+ * gtk_main_do_event() is the highest level.
+ *
+ * All that said, you most likely don't want to use any of these
+ * functions; synthesizing events is rarely needed. Consider asking on
+ * the mailing list for better ways to achieve your goals. For
+ * example, use gdk_window_invalidate_rect() or
+ * gtk_widget_queue_draw() instead of making up expose events.
+ * 
+ **/
 void
 gtk_propagate_event (GtkWidget *widget,
 		     GdkEvent  *event)
@@ -1381,6 +1514,8 @@ gtk_propagate_event (GtkWidget *widget,
   
   handled_event = FALSE;
 
+  gtk_widget_ref (widget);
+      
   if ((event->type == GDK_KEY_PRESS) ||
       (event->type == GDK_KEY_RELEASE))
     {
@@ -1392,29 +1527,53 @@ gtk_propagate_event (GtkWidget *widget,
       GtkWidget *window;
 
       window = gtk_widget_get_ancestor (widget, GTK_TYPE_WINDOW);
-      if (window)
-        {
-	  if (GTK_WIDGET_IS_SENSITIVE (window))
-	    gtk_widget_event (window, event);
 
-          handled_event = TRUE; /* don't send to widget */
-        }
+      if (window)
+	{
+	  /* If there is a grab within the window, give the grab widget
+	   * a first crack at the key event
+	   */
+	  if (widget != window && GTK_WIDGET_HAS_GRAB (widget))
+	    handled_event = gtk_widget_event (widget, event);
+	  
+	  if (!handled_event)
+	    {
+	      window = gtk_widget_get_ancestor (widget, GTK_TYPE_WINDOW);
+	      if (window)
+		{
+		  if (GTK_WIDGET_IS_SENSITIVE (window))
+		    gtk_widget_event (window, event);
+		}
+	    }
+		  
+	  handled_event = TRUE; /* don't send to widget */
+	}
     }
   
   /* Other events get propagated up the widget tree
    *  so that parents can see the button and motion
    *  events of the children.
    */
-  while (!handled_event && widget)
+  if (!handled_event)
     {
-      GtkWidget *tmp;
+      while (TRUE)
+	{
+	  GtkWidget *tmp;
+	  
+	  handled_event = !GTK_WIDGET_IS_SENSITIVE (widget) || gtk_widget_event (widget, event);
+	  tmp = widget->parent;
+	  gtk_widget_unref (widget);
 
-      gtk_widget_ref (widget);
-      handled_event = !GTK_WIDGET_IS_SENSITIVE (widget) || gtk_widget_event (widget, event);
-      tmp = widget->parent;
-      gtk_widget_unref (widget);
-      widget  = tmp;
+	  widget = tmp;
+	  
+	  if (!handled_event && widget)
+	    gtk_widget_ref (widget);
+	  else
+	    break;
+	}
     }
+  else
+    gtk_widget_unref (widget);
 }
 
 #if 0
@@ -1529,3 +1688,19 @@ gtk_print (gchar *str)
     gtk_widget_show (window);
 }
 #endif
+
+gboolean
+_gtk_boolean_handled_accumulator (GSignalInvocationHint *ihint,
+				  GValue                *return_accu,
+				  const GValue          *handler_return,
+				  gpointer               dummy)
+{
+  gboolean continue_emission;
+  gboolean signal_handled;
+  
+  signal_handled = g_value_get_boolean (handler_return);
+  g_value_set_boolean (return_accu, signal_handled);
+  continue_emission = !signal_handled;
+  
+  return continue_emission;
+}
