@@ -57,8 +57,6 @@ static void gtk_menu_item_size_allocate  (GtkWidget        *widget,
 					  GtkAllocation    *allocation);
 static void gtk_menu_item_paint          (GtkWidget        *widget,
 					  GdkRectangle     *area);
-static void gtk_menu_item_draw           (GtkWidget        *widget,
-					  GdkRectangle     *area);
 static gint gtk_menu_item_expose         (GtkWidget        *widget,
 					  GdkEventExpose   *event);
 
@@ -66,9 +64,11 @@ static void gtk_real_menu_item_select               (GtkItem     *item);
 static void gtk_real_menu_item_deselect             (GtkItem     *item);
 static void gtk_real_menu_item_activate_item        (GtkMenuItem *item);
 static void gtk_real_menu_item_toggle_size_request  (GtkMenuItem *menu_item,
-						     guint16     *requisition);
+						     gint        *requisition);
 static void gtk_real_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
-						     guint16      allocation);
+						     gint         allocation);
+static gboolean gtk_menu_item_mnemonic_activate     (GtkWidget   *widget,
+						     gboolean     group_cycling);
 
 static gint gtk_menu_item_select_timeout (gpointer          data);
 static void gtk_menu_item_popup_submenu  (gpointer     data);
@@ -84,6 +84,7 @@ static void gtk_menu_item_forall         (GtkContainer    *container,
 					  GtkCallback      callback,
 					  gpointer         callback_data);
 
+
 static GtkItemClass *parent_class;
 static guint menu_item_signals[LAST_SIGNAL] = { 0 };
 static guint32	last_submenu_deselect_time = 0;
@@ -131,6 +132,28 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 
   parent_class = gtk_type_class (gtk_item_get_type ());
 
+
+  object_class->destroy = gtk_menu_item_destroy;
+
+  widget_class->size_request = gtk_menu_item_size_request;
+  widget_class->size_allocate = gtk_menu_item_size_allocate;
+  widget_class->expose_event = gtk_menu_item_expose;
+  widget_class->show_all = gtk_menu_item_show_all;
+  widget_class->hide_all = gtk_menu_item_hide_all;
+  widget_class->mnemonic_activate = gtk_menu_item_mnemonic_activate;
+
+  container_class->forall = gtk_menu_item_forall;
+
+  item_class->select = gtk_real_menu_item_select;
+  item_class->deselect = gtk_real_menu_item_deselect;
+
+  klass->activate = NULL;
+  klass->activate_item = gtk_real_menu_item_activate_item;
+  klass->toggle_size_request = gtk_real_menu_item_toggle_size_request;
+  klass->toggle_size_allocate = gtk_real_menu_item_toggle_size_allocate;
+
+  klass->hide_on_activate = TRUE;
+
   menu_item_signals[ACTIVATE] =
     gtk_signal_new ("activate",
                     GTK_RUN_FIRST | GTK_RUN_ACTION,
@@ -138,6 +161,7 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
                     GTK_SIGNAL_OFFSET (GtkMenuItemClass, activate),
                     gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
+  widget_class->activate_signal = menu_item_signals[ACTIVATE];
 
   menu_item_signals[ACTIVATE_ITEM] =
     gtk_signal_new ("activate_item",
@@ -161,33 +185,9 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
                     GTK_RUN_FIRST,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkMenuItemClass, toggle_size_allocate),
-                    gtk_marshal_NONE__UINT,
+                    gtk_marshal_NONE__INT,
 		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_UINT);
-
-  gtk_object_class_add_signals (object_class, menu_item_signals, LAST_SIGNAL);
-
-  object_class->destroy = gtk_menu_item_destroy;
-
-  widget_class->activate_signal = menu_item_signals[ACTIVATE];
-  widget_class->size_request = gtk_menu_item_size_request;
-  widget_class->size_allocate = gtk_menu_item_size_allocate;
-  widget_class->draw = gtk_menu_item_draw;
-  widget_class->expose_event = gtk_menu_item_expose;
-  widget_class->show_all = gtk_menu_item_show_all;
-  widget_class->hide_all = gtk_menu_item_hide_all;
-
-  container_class->forall = gtk_menu_item_forall;
-
-  item_class->select = gtk_real_menu_item_select;
-  item_class->deselect = gtk_real_menu_item_deselect;
-
-  klass->activate = NULL;
-  klass->activate_item = gtk_real_menu_item_activate_item;
-  klass->toggle_size_request = gtk_real_menu_item_toggle_size_request;
-  klass->toggle_size_allocate = gtk_real_menu_item_toggle_size_allocate;
-
-  klass->hide_on_activate = TRUE;
+		    GTK_TYPE_INT);
 }
 
 static void
@@ -304,14 +304,14 @@ gtk_menu_item_set_placement (GtkMenuItem         *menu_item,
 
 void
 gtk_menu_item_configure (GtkMenuItem *menu_item,
-			 gint         show_toggle_indicator,
-			 gint         show_submenu_indicator)
+			 gboolean     show_toggle_indicator,
+			 gboolean     show_submenu_indicator)
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
-  menu_item->show_toggle_indicator = (show_toggle_indicator == TRUE);
-  menu_item->show_submenu_indicator = (show_submenu_indicator == TRUE);
+  menu_item->show_toggle_indicator = show_toggle_indicator;
+  menu_item->show_submenu_indicator = show_submenu_indicator;
 }
 
 void
@@ -343,7 +343,7 @@ gtk_menu_item_activate (GtkMenuItem *menu_item)
 
 void
 gtk_menu_item_toggle_size_request (GtkMenuItem *menu_item,
-				   guint16     *requisition)
+				   gint        *requisition)
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
@@ -353,7 +353,7 @@ gtk_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 
 void
 gtk_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
-				    guint16      allocation)
+				    gint         allocation)
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
@@ -523,38 +523,10 @@ gtk_menu_item_paint (GtkWidget    *widget,
     }
 }
 
-static void
-gtk_menu_item_draw (GtkWidget    *widget,
-		    GdkRectangle *area)
-{
-  GtkBin *bin;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      gtk_menu_item_paint (widget, area);
-
-      bin = GTK_BIN (widget);
-
-      if (bin->child)
-        {
-          if (gtk_widget_intersect (bin->child, area, &child_area))
-            gtk_widget_draw (bin->child, &child_area);
-        }
-    }
-}
-
 static gint
 gtk_menu_item_expose (GtkWidget      *widget,
 		      GdkEventExpose *event)
 {
-  GtkBin *bin;
-  GdkEventExpose child_event;
-
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_MENU_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
@@ -563,16 +535,7 @@ gtk_menu_item_expose (GtkWidget      *widget,
     {
       gtk_menu_item_paint (widget, &event->area);
 
-      bin = GTK_BIN (widget);
-
-      if (bin->child)
-        {
-          child_event = *event;
-
-          if (GTK_WIDGET_NO_WINDOW (bin->child) &&
-              gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-            gtk_widget_event (bin->child, (GdkEvent*) &child_event);
-        }
+      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
     }
 
   return FALSE;
@@ -642,6 +605,25 @@ gtk_real_menu_item_deselect (GtkItem *item)
   gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
 }
 
+static gboolean
+gtk_menu_item_mnemonic_activate (GtkWidget *widget,
+				 gboolean   group_cycling)
+{
+  if (group_cycling)
+    {
+      if (widget->parent &&
+	  GTK_IS_MENU_SHELL (widget->parent))
+	gtk_menu_shell_select_item (GTK_MENU_SHELL (widget->parent),
+				    widget);
+
+    }
+  else
+    gtk_signal_emit (GTK_OBJECT (widget), menu_item_signals[ACTIVATE_ITEM]);
+  
+  return TRUE;
+}
+
+
 static void
 gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
 {
@@ -681,7 +663,7 @@ gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
 }
 static void
 gtk_real_menu_item_toggle_size_request (GtkMenuItem *menu_item,
-					guint16     *requisition)
+					gint        *requisition)
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
@@ -691,7 +673,7 @@ gtk_real_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 
 static void
 gtk_real_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
-					 guint16      allocation)
+					 gint         allocation)
 {
   g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
