@@ -127,10 +127,6 @@ static void gtk_packer_class_init    (GtkPackerClass   *klass);
 static void gtk_packer_init          (GtkPacker        *packer);
 static void gtk_packer_map           (GtkWidget        *widget);
 static void gtk_packer_unmap         (GtkWidget        *widget);
-static void gtk_packer_draw          (GtkWidget        *widget,
-                                      GdkRectangle     *area);
-static gint gtk_packer_expose        (GtkWidget        *widget,
-                                      GdkEventExpose   *event);
 static void gtk_packer_size_request  (GtkWidget      *widget,
                                       GtkRequisition *requisition);
 static void gtk_packer_size_allocate (GtkWidget      *widget,
@@ -217,15 +213,13 @@ gtk_packer_class_init (GtkPackerClass *klass)
   gtk_container_add_child_arg_type ("GtkPacker::pad_y", GTK_TYPE_UINT, GTK_ARG_READWRITE, CHILD_ARG_PAD_Y);
   gtk_container_add_child_arg_type ("GtkPacker::ipad_x", GTK_TYPE_UINT, GTK_ARG_READWRITE, CHILD_ARG_I_PAD_X);
   gtk_container_add_child_arg_type ("GtkPacker::ipad_y", GTK_TYPE_UINT, GTK_ARG_READWRITE, CHILD_ARG_I_PAD_Y);
-  gtk_container_add_child_arg_type ("GtkPacker::position", GTK_TYPE_LONG, GTK_ARG_READWRITE, CHILD_ARG_POSITION);
+  gtk_container_add_child_arg_type ("GtkPacker::position", GTK_TYPE_INT, GTK_ARG_READWRITE, CHILD_ARG_POSITION);
 
   object_class->set_arg = gtk_packer_set_arg;
   object_class->get_arg = gtk_packer_get_arg;
 
   widget_class->map = gtk_packer_map;
   widget_class->unmap = gtk_packer_unmap;
-  widget_class->draw = gtk_packer_draw;
-  widget_class->expose_event = gtk_packer_expose;
   
   widget_class->size_request = gtk_packer_size_request;
   widget_class->size_allocate = gtk_packer_size_allocate;
@@ -401,7 +395,7 @@ gtk_packer_set_child_arg (GtkContainer   *container,
     case CHILD_ARG_POSITION:
       gtk_packer_reorder_child (packer,
 				child,
-				GTK_VALUE_LONG (*arg));
+				GTK_VALUE_INT (*arg));
       break;
     default:
       break;
@@ -479,16 +473,16 @@ gtk_packer_get_child_arg (GtkContainer   *container,
       GTK_VALUE_UINT (*arg) = child_info->i_pad_y;
       break;
     case CHILD_ARG_POSITION:
-      GTK_VALUE_LONG (*arg) = 0;
+      GTK_VALUE_INT (*arg) = 0;
       for (list = packer->children; list; list = list->next)
 	{
 	  child_info = list->data;
 	  if (child_info->widget == child)
 	    break;
-	  GTK_VALUE_LONG (*arg)++;
+	  GTK_VALUE_INT (*arg)++;
 	}
       if (!list)
-	GTK_VALUE_LONG (*arg) = -1;
+	GTK_VALUE_INT (*arg) = -1;
       break;
     default:
       arg->type = GTK_TYPE_INVALID;
@@ -896,69 +890,6 @@ gtk_packer_unmap (GtkWidget *widget)
     }
 }
 
-static void 
-gtk_packer_draw (GtkWidget    *widget,
-		 GdkRectangle *area)
-{
-  GtkPacker *packer;
-  GtkPackerChild *child;
-  GdkRectangle child_area;
-  GList *children;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_PACKER (widget));
- 
-  if (GTK_WIDGET_DRAWABLE (widget)) 
-    {
-      packer = GTK_PACKER (widget);
-      
-      children = g_list_first(packer->children);
-      while (children != NULL) 
-	{
-	  child = children->data;
-	  children = g_list_next(children);
-	  
-	  if (gtk_widget_intersect (child->widget, area, &child_area))
-	    gtk_widget_draw (child->widget, &child_area);
-        }
-    }
-  
-}
-
-static gint 
-gtk_packer_expose (GtkWidget      *widget,
-		   GdkEventExpose *event)
-{
-  GtkPacker *packer;
-  GtkPackerChild *child;
-  GdkEventExpose child_event;
-  GList *children;
-  
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_PACKER (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
-  if (GTK_WIDGET_DRAWABLE (widget)) 
-    {
-      packer = GTK_PACKER (widget);
-      
-      child_event = *event;
-      
-      children = g_list_first(packer->children);
-      while (children) 
-	{
-	  child = children->data;
-	  children = g_list_next(children);
-	  
-	  if (GTK_WIDGET_NO_WINDOW (child->widget) &&
-	      gtk_widget_intersect (child->widget, &event->area, &child_event.area))
-	    gtk_widget_event (child->widget, (GdkEvent*) &child_event);
-	}
-    }   
-  
-  return FALSE;
-}
-
 static void 
 gtk_packer_size_request (GtkWidget      *widget,
 			 GtkRequisition *requisition)
