@@ -25,33 +25,35 @@
  */
 
 #include <stdio.h>
+#include "gtkintl.h"
 #include "gtkmain.h"
 #include "gtkrange.h"
 #include "gtksignal.h"
+#include "gtkintl.h"
 
 #define SCROLL_TIMER_LENGTH  20
-#define SCROLL_INITIAL_DELAY 100
+#define SCROLL_INITIAL_DELAY 250  /* must hold button this long before ... */
+#define SCROLL_LATER_DELAY   100  /* ... it starts repeating at this rate  */
 #define SCROLL_DELAY_LENGTH  300
 
 #define RANGE_CLASS(w)  GTK_RANGE_GET_CLASS (w)
 
 enum {
-  ARG_0,
-  ARG_UPDATE_POLICY
+  PROP_0,
+  PROP_UPDATE_POLICY
 };
 
 static void gtk_range_class_init               (GtkRangeClass    *klass);
 static void gtk_range_init                     (GtkRange         *range);
-static void gtk_range_set_arg		       (GtkObject        *object,
-						GtkArg           *arg,
-						guint             arg_id);
-static void gtk_range_get_arg		       (GtkObject        *object,
-						GtkArg           *arg,
-						guint             arg_id);
+static void gtk_range_set_property             (GObject          *object,
+						guint             prop_id,
+						const GValue     *value,
+						GParamSpec       *pspec);
+static void gtk_range_get_property             (GObject          *object,
+						guint             prop_id,
+						GValue           *value,
+						GParamSpec       *pspec);
 static void gtk_range_destroy                  (GtkObject        *object);
-static void gtk_range_draw                     (GtkWidget        *widget,
-						GdkRectangle     *area);
-static void gtk_range_draw_focus               (GtkWidget        *widget);
 static void gtk_range_unrealize                (GtkWidget        *widget);
 static gint gtk_range_expose                   (GtkWidget        *widget,
 						GdkEventExpose   *event);
@@ -67,10 +69,6 @@ static gint gtk_range_enter_notify             (GtkWidget        *widget,
 						GdkEventCrossing *event);
 static gint gtk_range_leave_notify             (GtkWidget        *widget,
 						GdkEventCrossing *event);
-static gint gtk_range_focus_in                 (GtkWidget        *widget,
-						GdkEventFocus    *event);
-static gint gtk_range_focus_out                (GtkWidget        *widget,
-						GdkEventFocus    *event);
 static gint gtk_range_scroll_event             (GtkWidget        *widget,
 						GdkEventScroll   *event);
 static void gtk_range_style_set                 (GtkWidget       *widget,
@@ -80,7 +78,7 @@ static void gtk_real_range_draw_trough         (GtkRange         *range);
 static void gtk_real_range_draw_slider         (GtkRange         *range);
 static gint gtk_real_range_timer               (GtkRange         *range);
 static gint gtk_range_scroll                   (GtkRange         *range,
-						gfloat            jump_perc);
+						gdouble           jump_perc);
 
 static void gtk_range_add_timer                (GtkRange         *range);
 static void gtk_range_remove_timer             (GtkRange         *range);
@@ -128,20 +126,20 @@ gtk_range_get_type (void)
 static void
 gtk_range_class_init (GtkRangeClass *class)
 {
+  GObjectClass   *gobject_class;
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
+  gobject_class = G_OBJECT_CLASS (class);
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
 
   parent_class = gtk_type_class (GTK_TYPE_WIDGET);
 
-  object_class->set_arg = gtk_range_set_arg;
-  object_class->get_arg = gtk_range_get_arg;
+  gobject_class->set_property = gtk_range_set_property;
+  gobject_class->get_property = gtk_range_get_property;
   object_class->destroy = gtk_range_destroy;
 
-  widget_class->draw = gtk_range_draw;
-  widget_class->draw_focus = gtk_range_draw_focus;
   widget_class->unrealize = gtk_range_unrealize;
   widget_class->expose_event = gtk_range_expose;
   widget_class->button_press_event = gtk_range_button_press;
@@ -151,13 +149,8 @@ gtk_range_class_init (GtkRangeClass *class)
   widget_class->key_press_event = gtk_range_key_press;
   widget_class->enter_notify_event = gtk_range_enter_notify;
   widget_class->leave_notify_event = gtk_range_leave_notify;
-  widget_class->focus_in_event = gtk_range_focus_in;
-  widget_class->focus_out_event = gtk_range_focus_out;
   widget_class->style_set = gtk_range_style_set;
 
-  class->slider_width = 11;
-  class->stepper_size = 11;
-  class->stepper_slider_spacing = 1;
   class->min_slider_size = 7;
   class->trough = 1;
   class->slider = 2;
@@ -174,47 +167,87 @@ gtk_range_class_init (GtkRangeClass *class)
   class->motion = NULL;
   class->timer = gtk_real_range_timer;
 
-  gtk_object_add_arg_type ("GtkRange::update_policy",
-			   GTK_TYPE_UPDATE_TYPE,
-			   GTK_ARG_READWRITE,
-			   ARG_UPDATE_POLICY);
+  g_object_class_install_property (gobject_class,
+                                   PROP_UPDATE_POLICY,
+                                   g_param_spec_enum ("update_policy",
+						      _("Update policy"),
+						      _("How the range should be updated on the screen"),
+						      GTK_TYPE_UPDATE_TYPE,
+						      GTK_UPDATE_CONTINUOUS,
+						      G_PARAM_READWRITE));
+  
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("slider_width",
+							     _("Slider Width"),
+							     _("Width of scrollbar or scale thumb"),
+							     0,
+							     G_MAXINT,
+							     11,
+							     G_PARAM_READABLE));
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("trough_border",
+							     _("Trough Border"),
+							     _("Width of border around range"),
+							     0,
+							     G_MAXINT,
+							     2,
+							     G_PARAM_READABLE));
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("stepper_size",
+							     _("Stepper Size"),
+							     _("Size of step buttons at ends"),
+							     0,
+							     G_MAXINT,
+							     11,
+							     G_PARAM_READABLE));
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_int ("stepper_spacing",
+							     _("Stepper Spacing"),
+							     _("Spacing between step buttons and thumb"),
+							     G_MININT,
+							     G_MAXINT,
+							     1,
+							     G_PARAM_READABLE));
 }
 
 static void
-gtk_range_set_arg (GtkObject      *object,
-		   GtkArg         *arg,
-		   guint           arg_id)
+gtk_range_set_property (GObject      *object,
+			guint         prop_id,
+			const GValue *value,
+			GParamSpec   *pspec)
 {
   GtkRange *range;
 
   range = GTK_RANGE (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_UPDATE_POLICY:
-      gtk_range_set_update_policy (range, GTK_VALUE_ENUM (*arg));
+    case PROP_UPDATE_POLICY:
+      gtk_range_set_update_policy (range, g_value_get_enum (value));
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
 static void
-gtk_range_get_arg (GtkObject      *object,
-		   GtkArg         *arg,
-		   guint           arg_id)
+gtk_range_get_property (GObject      *object,
+			guint         prop_id,
+			GValue       *value,
+			GParamSpec   *pspec)
 {
   GtkRange *range;
 
   range = GTK_RANGE (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_UPDATE_POLICY:
-      GTK_VALUE_ENUM (*arg) = range->policy;
+    case PROP_UPDATE_POLICY:
+      g_value_set_enum (value, range->policy);
       break;
     default:
-      arg->type = GTK_TYPE_INVALID;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -264,7 +297,11 @@ gtk_range_set_update_policy (GtkRange      *range,
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
 
-  range->policy = policy;
+  if (range->policy != policy)
+    {
+      range->policy = policy;
+      g_object_notify (G_OBJECT (range), "update_policy");
+    }
 }
 
 void
@@ -309,7 +346,30 @@ gtk_range_set_adjustment (GtkRange      *range,
 }
 
 void
-gtk_range_draw_background (GtkRange *range)
+gtk_range_set_inverted (GtkRange *range,
+                        gboolean  setting)
+{
+  g_return_if_fail (GTK_IS_RANGE (range));
+  
+  setting = setting != FALSE;
+
+  if (setting != range->inverted)
+    {
+      range->inverted = setting;
+      gtk_widget_queue_resize (GTK_WIDGET (range));
+    }
+}
+
+gboolean
+gtk_range_get_inverted (GtkRange *range)
+{
+  g_return_val_if_fail (GTK_IS_RANGE (range), FALSE);
+
+  return range->inverted;
+}
+
+void
+_gtk_range_draw_background (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -319,7 +379,7 @@ gtk_range_draw_background (GtkRange *range)
 }
 
 void
-gtk_range_clear_background (GtkRange *range)
+_gtk_range_clear_background (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -329,7 +389,7 @@ gtk_range_clear_background (GtkRange *range)
 }
 
 void
-gtk_range_draw_trough (GtkRange *range)
+_gtk_range_draw_trough (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -339,7 +399,7 @@ gtk_range_draw_trough (GtkRange *range)
 }
 
 void
-gtk_range_draw_slider (GtkRange *range)
+_gtk_range_draw_slider (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -349,7 +409,7 @@ gtk_range_draw_slider (GtkRange *range)
 }
 
 void
-gtk_range_draw_step_forw (GtkRange *range)
+_gtk_range_draw_step_forw (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -359,7 +419,7 @@ gtk_range_draw_step_forw (GtkRange *range)
 }
 
 void
-gtk_range_draw_step_back (GtkRange *range)
+_gtk_range_draw_step_back (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -369,7 +429,7 @@ gtk_range_draw_step_back (GtkRange *range)
 }
 
 void
-gtk_range_slider_update (GtkRange *range)
+_gtk_range_slider_update (GtkRange *range)
 {
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
@@ -378,11 +438,11 @@ gtk_range_slider_update (GtkRange *range)
     (* RANGE_CLASS (range)->slider_update) (range);
 }
 
-gint
-gtk_range_trough_click (GtkRange *range,
-			gint      x,
-			gint      y,
-			gfloat   *jump_perc)
+gboolean
+_gtk_range_trough_click (GtkRange  *range,
+                         gint       x,
+                         gint       y,
+                         gdouble   *jump_perc)
 {
   g_return_val_if_fail (range != NULL, GTK_TROUGH_NONE);
   g_return_val_if_fail (GTK_IS_RANGE (range), GTK_TROUGH_NONE);
@@ -423,16 +483,32 @@ move_and_update_window (GdkWindow *window, gint x, gint y)
   gdk_window_process_updates (parent, TRUE);
 }
 
+static gboolean
+should_invert (GtkRange *range,
+               gboolean  horizontal)
+{  
+  if (horizontal)
+    return
+      (range->inverted && !range->flippable) ||
+      (range->inverted && range->flippable && gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_LTR) ||
+      (!range->inverted && range->flippable && gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_RTL);
+  else
+    return range->inverted;
+}
+
 void
-gtk_range_default_hslider_update (GtkRange *range)
+_gtk_range_default_hslider_update (GtkRange *range)
 {
   gint left;
   gint right;
   gint x;
+  gint trough_border;
 
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
 
+  _gtk_range_get_props (range, NULL, &trough_border, NULL, NULL);
+
   if (GTK_WIDGET_REALIZED (range))
     {
       gtk_range_trough_hdims (range, &left, &right);
@@ -458,23 +534,26 @@ gtk_range_default_hslider_update (GtkRange *range)
       else if (x > right)
 	x = right;
 
-      if (range->flippable && gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_RTL)
+      if (should_invert (range, TRUE))
 	x = right - (x - left);
       
-      move_and_update_window (range->slider, x, GTK_WIDGET (range)->style->ythickness);
+      move_and_update_window (range->slider, x, trough_border);
     }
 }
 
 void
-gtk_range_default_vslider_update (GtkRange *range)
+_gtk_range_default_vslider_update (GtkRange *range)
 {
   gint top;
   gint bottom;
   gint y;
+  gint trough_border;
 
   g_return_if_fail (range != NULL);
   g_return_if_fail (GTK_IS_RANGE (range));
 
+  _gtk_range_get_props (range, NULL, &trough_border, NULL, NULL);
+
   if (GTK_WIDGET_REALIZED (range))
     {
       gtk_range_trough_vdims (range, &top, &bottom);
@@ -500,17 +579,20 @@ gtk_range_default_vslider_update (GtkRange *range)
       else if (y > bottom)
 	y = bottom;
 
-      move_and_update_window (range->slider, GTK_WIDGET (range)->style->xthickness, y);
+      if (should_invert (range, FALSE))
+	y = bottom - (y - top);
+      
+      move_and_update_window (range->slider, trough_border, y);
     }
 }
 
-gint
-gtk_range_default_htrough_click (GtkRange *range,
-				 gint      x,
-				 gint      y,
-				 gfloat	  *jump_perc)
+gboolean
+_gtk_range_default_htrough_click (GtkRange  *range,
+                                  gint       x,
+                                  gint       y,
+                                  gdouble   *jump_perc)
 {
-  gint ythickness;
+  gint trough_border;
   gint trough_width;
   gint trough_height;
   gint slider_x;
@@ -520,20 +602,20 @@ gtk_range_default_htrough_click (GtkRange *range,
   g_return_val_if_fail (range != NULL, GTK_TROUGH_NONE);
   g_return_val_if_fail (GTK_IS_RANGE (range), GTK_TROUGH_NONE);
 
-  ythickness = GTK_WIDGET (range)->style->ythickness;
+  _gtk_range_get_props (range, NULL, &trough_border, NULL, NULL);
 
   gtk_range_trough_hdims (range, &left, &right);
   gdk_window_get_size (range->slider, &slider_length, NULL);
   right += slider_length;
 
-  if (range->flippable && gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_RTL)
+  if (should_invert (range, TRUE))
     x = (right - x) + left;
 
-  if ((x > left) && (y > ythickness))
+  if ((x > left) && (y > trough_border))
     {
       gdk_window_get_size (range->trough, &trough_width, &trough_height);
-
-      if ((x < right) && (y < (trough_height - ythickness)))
+      
+      if ((x < right) && (y < (trough_height - trough_border)))
 	{
 	  if (jump_perc)
 	    {
@@ -553,13 +635,13 @@ gtk_range_default_htrough_click (GtkRange *range,
   return GTK_TROUGH_NONE;
 }
 
-gint
-gtk_range_default_vtrough_click (GtkRange *range,
-				 gint      x,
-				 gint      y,
-				 gfloat   *jump_perc)
+gboolean
+_gtk_range_default_vtrough_click (GtkRange  *range,
+                                  gint       x,
+                                  gint       y,
+                                  gdouble   *jump_perc)
 {
-  gint xthickness;
+  gint trough_border;
   gint trough_width;
   gint trough_height;
   gint slider_y;
@@ -569,17 +651,20 @@ gtk_range_default_vtrough_click (GtkRange *range,
   g_return_val_if_fail (range != NULL, GTK_TROUGH_NONE);
   g_return_val_if_fail (GTK_IS_RANGE (range), GTK_TROUGH_NONE);
 
-  xthickness = GTK_WIDGET (range)->style->xthickness;
-
+  _gtk_range_get_props (range, NULL, &trough_border, NULL, NULL);
+  
   gtk_range_trough_vdims (range, &top, &bottom);
   gdk_window_get_size (range->slider, NULL, &slider_length);
   bottom += slider_length;
-	      
-  if ((x > xthickness) && (y > top))
+
+  if (should_invert (range, FALSE))
+    y = (bottom - y) + top;
+  
+  if ((x > trough_border) && (y > top))
     {
       gdk_window_get_size (range->trough, &trough_width, &trough_height);
 
-      if ((x < (trough_width - xthickness) && (y < bottom)))
+      if ((x < (trough_width - trough_border) && (y < bottom)))
 	{
 	  if (jump_perc)
 	    {
@@ -601,9 +686,9 @@ gtk_range_default_vtrough_click (GtkRange *range,
 }
 
 void
-gtk_range_default_hmotion (GtkRange *range,
-			   gint      xdelta,
-			   gint      ydelta)
+_gtk_range_default_hmotion (GtkRange *range,
+                            gint      xdelta,
+                            gint      ydelta)
 {
   gdouble old_value;
   gint left, right;
@@ -621,7 +706,7 @@ gtk_range_default_hmotion (GtkRange *range,
 
   new_pos = slider_x + xdelta;
 
-  if (range->flippable && gtk_widget_get_direction (GTK_WIDGET (range)) == GTK_TEXT_DIR_RTL)
+  if (should_invert (range, TRUE))
     new_pos = (right - new_pos) + left;
 
   if (new_pos < left)
@@ -641,7 +726,7 @@ gtk_range_default_hmotion (GtkRange *range,
       char buffer[64];
 
       sprintf (buffer, "%0.*f", range->digits, range->adjustment->value);
-      sscanf (buffer, "%f", &range->adjustment->value);
+      sscanf (buffer, "%lf", &range->adjustment->value);
     }
 
   if (old_value != range->adjustment->value)
@@ -652,8 +737,8 @@ gtk_range_default_hmotion (GtkRange *range,
 	}
       else
 	{
-	  gtk_range_slider_update (range);
-	  gtk_range_clear_background (range);
+	  _gtk_range_slider_update (range);
+	  _gtk_range_clear_background (range);
 
 	  if (range->policy == GTK_UPDATE_DELAYED)
 	    {
@@ -667,9 +752,9 @@ gtk_range_default_hmotion (GtkRange *range,
 }
 
 void
-gtk_range_default_vmotion (GtkRange *range,
-			   gint      xdelta,
-			   gint      ydelta)
+_gtk_range_default_vmotion (GtkRange *range,
+                            gint      xdelta,
+                            gint      ydelta)
 {
   gdouble old_value;
   gint top, bottom;
@@ -689,6 +774,9 @@ gtk_range_default_vmotion (GtkRange *range,
 
   new_pos = slider_y + ydelta;
 
+  if (should_invert (range, FALSE))
+    new_pos = (bottom - new_pos) + top;
+  
   if (new_pos < top)
     new_pos = top;
   else if (new_pos > bottom)
@@ -706,7 +794,7 @@ gtk_range_default_vmotion (GtkRange *range,
       char buffer[64];
 
       sprintf (buffer, "%0.*f", range->digits, range->adjustment->value);
-      sscanf (buffer, "%f", &range->adjustment->value);
+      sscanf (buffer, "%lf", &range->adjustment->value);
     }
 
   if (old_value != range->adjustment->value)
@@ -717,8 +805,8 @@ gtk_range_default_vmotion (GtkRange *range,
 	}
       else
 	{
-	  gtk_range_slider_update (range);
-	  gtk_range_clear_background (range);
+	  _gtk_range_slider_update (range);
+	  _gtk_range_clear_background (range);
 
 	  if (range->policy == GTK_UPDATE_DELAYED)
 	    {
@@ -755,38 +843,6 @@ gtk_range_destroy (GtkObject *object)
   (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-static void
-gtk_range_draw (GtkWidget    *widget,
-		GdkRectangle *area)
-{
-  GtkRange *range;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_RANGE (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      range = GTK_RANGE (widget);
-
-      gtk_range_draw_background (range);
-      gtk_range_draw_trough (range);
-      gtk_range_draw_slider (range);
-      gtk_range_draw_step_forw (range);
-      gtk_range_draw_step_back (range);
-    }
-}
-
-static void
-gtk_range_draw_focus (GtkWidget *widget)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_RANGE (widget));
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    gtk_range_draw_trough (GTK_RANGE (widget));
-}
-
 static void
 gtk_range_unrealize (GtkWidget *widget)
 {
@@ -843,23 +899,23 @@ gtk_range_expose (GtkWidget      *widget,
    */
   if (event->window == range->trough)
     {
-      gtk_range_draw_trough (range);
+      _gtk_range_draw_trough (range);
     }
   else if (event->window == widget->window)
     {
-      gtk_range_draw_background (range); 
+      _gtk_range_draw_background (range); 
     }
   else if (event->window == range->slider)
     {
-      gtk_range_draw_slider (range);
+      _gtk_range_draw_slider (range);
     }
   else if (event->window == range->step_forw)
     {
-      gtk_range_draw_step_forw (range);
+      _gtk_range_draw_step_forw (range);
     }
   else if (event->window == range->step_back)
     {
-      gtk_range_draw_step_back (range);
+      _gtk_range_draw_step_back (range);
     }
   return FALSE;
 }
@@ -870,7 +926,7 @@ gtk_range_button_press (GtkWidget      *widget,
 {
   GtkRange *range;
   gint trough_part;
-  gfloat jump_perc;
+  gdouble jump_perc;
 
   g_return_val_if_fail (GTK_IS_RANGE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
@@ -893,9 +949,9 @@ gtk_range_button_press (GtkWidget      *widget,
 	  range->click_child = RANGE_CLASS (range)->trough;
 	  
 	  if (range->button == 2)
-	    trough_part = gtk_range_trough_click (range, event->x, event->y, &jump_perc);
+	    trough_part = _gtk_range_trough_click (range, event->x, event->y, &jump_perc);
 	  else
-	    trough_part = gtk_range_trough_click (range, event->x, event->y, NULL);
+	    trough_part = _gtk_range_trough_click (range, event->x, event->y, NULL);
 	  
 	  range->scroll_type = GTK_SCROLL_NONE;
 	  if (trough_part == GTK_TROUGH_START)
@@ -944,14 +1000,14 @@ gtk_range_button_press (GtkWidget      *widget,
 	      gtk_range_add_timer (range);
 	      
 	      if (back)
-		gtk_range_draw_step_back (range);
+		_gtk_range_draw_step_back (range);
 	      else
-		gtk_range_draw_step_forw (range);
+		_gtk_range_draw_step_forw (range);
 	    }
 	}
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 static gint
@@ -995,19 +1051,19 @@ gtk_range_button_release (GtkWidget      *widget,
 	  if (range->click_child == RANGE_CLASS (range)->step_forw)
 	    {
 	      range->click_child = 0;
-	      gtk_range_draw_step_forw (range);
+	      _gtk_range_draw_step_forw (range);
 	    }
 	  else if (range->click_child == RANGE_CLASS (range)->step_back)
 	    {
 	      range->click_child = 0;
-	      gtk_range_draw_step_back (range);
+	      _gtk_range_draw_step_back (range);
 	    }
 	}
 
       range->click_child = 0;
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 static gint
@@ -1024,9 +1080,9 @@ gtk_range_scroll_event (GtkWidget      *widget,
   if (GTK_WIDGET_REALIZED (range))
     {
       GtkAdjustment *adj = GTK_RANGE (range)->adjustment;
-      gfloat new_value = adj->value + ((event->direction == GDK_SCROLL_UP) ? 
-				       -adj->page_increment / 2: 
-				       adj->page_increment / 2);
+      gdouble new_value = adj->value + ((event->direction == GDK_SCROLL_UP) ? 
+					-adj->page_increment / 2: 
+					adj->page_increment / 2);
       new_value = CLAMP (new_value, adj->lower, adj->upper - adj->page_size);
       gtk_adjustment_set_value (adj, new_value);
     }
@@ -1079,7 +1135,7 @@ gtk_range_motion_notify (GtkWidget      *widget,
 	}
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 static gint
@@ -1096,7 +1152,7 @@ gtk_range_key_press (GtkWidget   *widget,
 
   range = GTK_RANGE (widget);
   return_val = FALSE;
-
+  
   if (RANGE_CLASS (range)->trough_keys)
     return_val = (* RANGE_CLASS (range)->trough_keys) (range, event, &scroll, &pos);
 
@@ -1105,17 +1161,47 @@ gtk_range_key_press (GtkWidget   *widget,
       if (scroll != GTK_SCROLL_NONE)
 	{
 	  range->scroll_type = scroll;
+
 	  gtk_range_scroll (range, -1);
 	  if (range->old_value != range->adjustment->value)
 	    {
 	      gtk_signal_emit_by_name (GTK_OBJECT (range->adjustment), "value_changed");
 	      switch (range->scroll_type)
 		{
+                case GTK_SCROLL_STEP_LEFT:
+                  if (should_invert (range, TRUE))
+                    _gtk_range_draw_step_forw (range);
+                  else
+                    _gtk_range_draw_step_back (range);
+                  break;
+                    
+                case GTK_SCROLL_STEP_UP:
+                  if (should_invert (range, FALSE))
+                    _gtk_range_draw_step_forw (range);
+                  else
+                    _gtk_range_draw_step_back (range);
+                  break;
+
+                case GTK_SCROLL_STEP_RIGHT:
+                  if (should_invert (range, TRUE))
+                    _gtk_range_draw_step_back (range);
+                  else
+                    _gtk_range_draw_step_forw (range);
+                  break;
+                    
+                case GTK_SCROLL_STEP_DOWN:
+                  if (should_invert (range, FALSE))
+                    _gtk_range_draw_step_back (range);
+                  else
+                    _gtk_range_draw_step_forw (range);
+                  break;
+                  
 		case GTK_SCROLL_STEP_BACKWARD:
-		  gtk_range_draw_step_back (range);
+		  _gtk_range_draw_step_back (range);
 		  break;
+                  
 		case GTK_SCROLL_STEP_FORWARD:
-		  gtk_range_draw_step_forw (range);
+		  _gtk_range_draw_step_forw (range);
 		  break;
 		}
 	    }
@@ -1133,8 +1219,8 @@ gtk_range_key_press (GtkWidget   *widget,
 	      gtk_signal_emit_by_name (GTK_OBJECT (range->adjustment),
 				       "value_changed");
 
-	      gtk_range_slider_update (range);
-	      gtk_range_clear_background (range);
+	      _gtk_range_slider_update (range);
+	      _gtk_range_clear_background (range);
 	    }
 	}
     }
@@ -1162,7 +1248,7 @@ gtk_range_enter_notify (GtkWidget        *widget,
 
       if ((range->click_child == 0) ||
 	  (range->click_child == RANGE_CLASS (range)->trough))
-	gtk_range_draw_slider (range);
+	_gtk_range_draw_slider (range);
     }
   else if (event->window == range->step_forw)
     {
@@ -1170,7 +1256,7 @@ gtk_range_enter_notify (GtkWidget        *widget,
 
       if ((range->click_child == 0) ||
 	  (range->click_child == RANGE_CLASS (range)->trough))
-	gtk_range_draw_step_forw (range);
+	_gtk_range_draw_step_forw (range);
     }
   else if (event->window == range->step_back)
     {
@@ -1178,10 +1264,10 @@ gtk_range_enter_notify (GtkWidget        *widget,
 
       if ((range->click_child == 0) ||
 	  (range->click_child == RANGE_CLASS (range)->trough))
-	gtk_range_draw_step_back (range);
+	_gtk_range_draw_step_back (range);
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 static gint
@@ -1204,48 +1290,22 @@ gtk_range_leave_notify (GtkWidget        *widget,
     {
       if ((range->click_child == 0) ||
 	  (range->click_child == RANGE_CLASS (range)->trough))
-	gtk_range_draw_slider (range);
+	_gtk_range_draw_slider (range);
     }
   else if (event->window == range->step_forw)
     {
       if ((range->click_child == 0) ||
 	  (range->click_child == RANGE_CLASS (range)->trough))
-	gtk_range_draw_step_forw (range);
+	_gtk_range_draw_step_forw (range);
     }
   else if (event->window == range->step_back)
     {
       if ((range->click_child == 0) ||
 	  (range->click_child == RANGE_CLASS (range)->trough))
-	gtk_range_draw_step_back (range);
+	_gtk_range_draw_step_back (range);
     }
 
-  return FALSE;
-}
-
-static gint
-gtk_range_focus_in (GtkWidget     *widget,
-		    GdkEventFocus *event)
-{
-  g_return_val_if_fail (GTK_IS_RANGE (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-  return FALSE;
-}
-
-static gint
-gtk_range_focus_out (GtkWidget     *widget,
-		     GdkEventFocus *event)
-{
-  g_return_val_if_fail (GTK_IS_RANGE (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-  return FALSE;
+  return TRUE;
 }
 
 static void
@@ -1349,18 +1409,84 @@ gtk_real_range_timer (GtkRange *range)
 }
 
 static gint
-gtk_range_scroll (GtkRange *range,
-		  gfloat    jump_perc)
+gtk_range_scroll (GtkRange  *range,
+		  gdouble    jump_perc)
 {
-  gfloat new_value;
+  gdouble new_value;
   gint return_val;
-
+  GtkScrollType scroll_type;
+  
   g_return_val_if_fail (GTK_IS_RANGE (range), FALSE);
 
   new_value = range->adjustment->value;
   return_val = TRUE;
 
-  switch (range->scroll_type)
+  /* Translate visual to logical */
+
+  scroll_type = range->scroll_type;
+  switch (scroll_type)
+    {
+    case GTK_SCROLL_STEP_UP:
+      if (should_invert (range, FALSE))
+        scroll_type = GTK_SCROLL_STEP_FORWARD;
+      else
+        scroll_type = GTK_SCROLL_STEP_BACKWARD;
+      break;
+
+    case GTK_SCROLL_STEP_DOWN:
+      if (should_invert (range, FALSE))
+        scroll_type = GTK_SCROLL_STEP_BACKWARD;
+      else
+        scroll_type = GTK_SCROLL_STEP_FORWARD;
+      break;
+
+    case GTK_SCROLL_PAGE_UP:
+      if (should_invert (range, FALSE))
+        scroll_type = GTK_SCROLL_PAGE_FORWARD;
+      else
+        scroll_type = GTK_SCROLL_PAGE_BACKWARD;
+      break;
+
+    case GTK_SCROLL_PAGE_DOWN:
+      if (should_invert (range, FALSE))
+        scroll_type = GTK_SCROLL_PAGE_BACKWARD;
+      else
+        scroll_type = GTK_SCROLL_PAGE_FORWARD;
+      break;
+
+    case GTK_SCROLL_STEP_LEFT:
+      if (should_invert (range, TRUE))
+        scroll_type = GTK_SCROLL_STEP_FORWARD;
+      else
+        scroll_type = GTK_SCROLL_STEP_BACKWARD;
+      break;
+
+    case GTK_SCROLL_STEP_RIGHT:
+      if (should_invert (range, TRUE))
+        scroll_type = GTK_SCROLL_STEP_BACKWARD;
+      else
+        scroll_type = GTK_SCROLL_STEP_FORWARD;
+      break;
+
+    case GTK_SCROLL_PAGE_LEFT:
+      if (should_invert (range, TRUE))
+        scroll_type = GTK_SCROLL_PAGE_FORWARD;
+      else
+        scroll_type = GTK_SCROLL_PAGE_BACKWARD;
+      break;
+
+    case GTK_SCROLL_PAGE_RIGHT:
+      if (should_invert (range, TRUE))
+        scroll_type = GTK_SCROLL_PAGE_BACKWARD;
+      else
+        scroll_type = GTK_SCROLL_PAGE_FORWARD;
+      break;
+
+    default:
+      break;
+    }
+  
+  switch (scroll_type)
     {
     case GTK_SCROLL_NONE:
       break;
@@ -1413,8 +1539,20 @@ gtk_range_scroll (GtkRange *range,
 	  range->timer = 0;
 	}
       break;
-    }
 
+    case GTK_SCROLL_STEP_UP:
+    case GTK_SCROLL_STEP_DOWN:
+    case GTK_SCROLL_PAGE_UP:
+    case GTK_SCROLL_PAGE_DOWN:
+    case GTK_SCROLL_STEP_LEFT:
+    case GTK_SCROLL_STEP_RIGHT:
+    case GTK_SCROLL_PAGE_LEFT:
+    case GTK_SCROLL_PAGE_RIGHT:
+      g_assert_not_reached ();
+      break;
+
+    }
+  
   if (new_value != range->adjustment->value)
     {
       range->adjustment->value = new_value;
@@ -1426,8 +1564,8 @@ gtk_range_scroll (GtkRange *range,
 	}
       else
 	{
-	  gtk_range_slider_update (range);
-	  gtk_range_clear_background (range);
+	  _gtk_range_slider_update (range);
+	  _gtk_range_clear_background (range);
 	}
     }
 
@@ -1435,6 +1573,35 @@ gtk_range_scroll (GtkRange *range,
 }
 
 
+static gboolean
+gtk_range_timer_1st_time (GtkRange *range)
+{
+  /*
+   * If the real timeout function succeeds and the timeout is still set,
+   * replace it with a quicker one so successive scrolling goes faster.
+   */
+  gtk_object_ref (GTK_OBJECT (range));
+
+  if (RANGE_CLASS (range)->timer (range))
+    {
+      if (range->timer)
+	{
+	  /* We explicitely remove ourselves here in the paranoia
+	   * that due to things happening above in the callback
+	   * above, we might have been removed, and another added.
+	   */
+	  g_source_remove (range->timer);
+	  range->timer = gtk_timeout_add (SCROLL_LATER_DELAY,
+					  (GtkFunction) RANGE_CLASS (range)->timer,
+					  range);
+	}
+    }
+  
+  gtk_object_unref (GTK_OBJECT (range));
+  
+  return FALSE;  /* don't keep calling this function */
+}
+
 static void
 gtk_range_add_timer (GtkRange *range)
 {
@@ -1444,8 +1611,8 @@ gtk_range_add_timer (GtkRange *range)
     {
       range->need_timer = TRUE;
       range->timer = gtk_timeout_add (SCROLL_INITIAL_DELAY,
-				      (GtkFunction) RANGE_CLASS (range)->timer,
-				      (gpointer) range);
+				      (GtkFunction) gtk_range_timer_1st_time,
+				      range);
     }
 }
 
@@ -1491,8 +1658,8 @@ gtk_range_adjustment_changed (GtkAdjustment *adjustment,
       (range->old_upper != adjustment->upper) ||
       (range->old_page_size != adjustment->page_size))
     {
-      gtk_range_slider_update (range);
-      gtk_range_clear_background (range);
+      _gtk_range_slider_update (range);
+      _gtk_range_clear_background (range);
 
       range->old_value = adjustment->value;
       range->old_lower = adjustment->lower;
@@ -1514,8 +1681,8 @@ gtk_range_adjustment_value_changed (GtkAdjustment *adjustment,
 
   if (range->old_value != adjustment->value)
     {
-      gtk_range_slider_update (range);
-      gtk_range_clear_background (range);
+      _gtk_range_slider_update (range);
+      _gtk_range_clear_background (range);
 
       range->old_value = adjustment->value;
     }
@@ -1532,25 +1699,29 @@ gtk_range_trough_hdims (GtkRange *range,
   gint tmp_width;
   gint tleft;
   gint tright;
+  gint stepper_spacing;
+  gint trough_border;
 
   g_return_if_fail (range != NULL);
 
   gdk_window_get_size (range->trough, &trough_width, NULL);
   gdk_window_get_size (range->slider, &slider_length, NULL);
 
-  tleft = GTK_WIDGET (range)->style->xthickness;
-  tright = trough_width - slider_length - GTK_WIDGET (range)->style->xthickness;
+  _gtk_range_get_props (range, NULL, &trough_border, NULL, &stepper_spacing);
+   
+  tleft = trough_border;
+  tright = trough_width - slider_length - trough_border;
 
   if (range->step_back)
     {
       gdk_window_get_size (range->step_back, &tmp_width, NULL);
-      tleft += (tmp_width + RANGE_CLASS (range)->stepper_slider_spacing);
+      tleft += (tmp_width + stepper_spacing);
     }
 
   if (range->step_forw)
     {
       gdk_window_get_size (range->step_forw, &tmp_width, NULL);
-      tright -= (tmp_width + RANGE_CLASS (range)->stepper_slider_spacing);
+      tright -= (tmp_width + stepper_spacing);
     }
 
   if (left)
@@ -1569,25 +1740,29 @@ gtk_range_trough_vdims (GtkRange *range,
   gint tmp_height;
   gint ttop;
   gint tbottom;
+  gint stepper_spacing;
+  gint trough_border;
 
   g_return_if_fail (range != NULL);
 
+  _gtk_range_get_props (range, NULL, &trough_border, NULL, &stepper_spacing);
+   
   gdk_window_get_size (range->trough, NULL, &trough_height);
   gdk_window_get_size (range->slider, NULL, &slider_length);
 
-  ttop = GTK_WIDGET (range)->style->ythickness;
-  tbottom = trough_height - slider_length - GTK_WIDGET (range)->style->ythickness;
+  ttop = trough_border;
+  tbottom = trough_height - slider_length - trough_border;
 
   if (range->step_back)
     {
       gdk_window_get_size (range->step_back, NULL, &tmp_height);
-      ttop += (tmp_height + RANGE_CLASS (range)->stepper_slider_spacing);
+      ttop += (tmp_height + stepper_spacing);
     }
 
   if (range->step_forw)
     {
       gdk_window_get_size (range->step_forw, NULL, &tmp_height);
-      tbottom -= (tmp_height + RANGE_CLASS (range)->stepper_slider_spacing);
+      tbottom -= (tmp_height + stepper_spacing);
     }
 
   if (top)
@@ -1633,3 +1808,27 @@ gtk_range_style_set (GtkWidget *widget,
 	}
     }
 }
+
+void
+_gtk_range_get_props (GtkRange *range,
+		      gint     *slider_width,
+		      gint     *trough_border,
+		      gint     *stepper_size,
+		      gint     *stepper_spacing)
+{
+  GtkWidget *widget =  GTK_WIDGET (range);
+  
+
+  if (slider_width)
+    gtk_widget_style_get (widget, "slider_width", slider_width, NULL);
+  
+  if (trough_border)
+    gtk_widget_style_get (widget, "trough_border", trough_border, NULL);
+
+  if (stepper_size)
+    gtk_widget_style_get (widget, "stepper_size", stepper_size, NULL);
+
+  if (stepper_spacing)
+    gtk_widget_style_get (widget, "stepper_spacing", stepper_spacing, NULL);
+}
+
