@@ -117,6 +117,18 @@ gtk_tips_query_class_init (GtkTipsQueryClass *class)
 
   parent_class = gtk_type_class (gtk_label_get_type ());
 
+
+  object_class->set_arg = gtk_tips_query_set_arg;
+  object_class->get_arg = gtk_tips_query_get_arg;
+  object_class->destroy = gtk_tips_query_destroy;
+
+  widget_class->event = gtk_tips_query_event;
+
+  class->start_query = gtk_tips_query_real_start_query;
+  class->stop_query = gtk_tips_query_real_stop_query;
+  class->widget_entered = gtk_tips_query_widget_entered;
+  class->widget_selected = NULL;
+
   gtk_object_add_arg_type ("GtkTipsQuery::emit_always", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_EMIT_ALWAYS);
   gtk_object_add_arg_type ("GtkTipsQuery::caller", GTK_TYPE_WIDGET, GTK_ARG_READWRITE, ARG_CALLER);
   gtk_object_add_arg_type ("GtkTipsQuery::label_inactive", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_LABEL_INACTIVE);
@@ -141,34 +153,23 @@ gtk_tips_query_class_init (GtkTipsQueryClass *class)
 		    GTK_RUN_LAST,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkTipsQueryClass, widget_entered),
-		    gtk_marshal_VOID__POINTER_STRING_STRING,
+		    gtk_marshal_VOID__OBJECT_STRING_STRING,
 		    GTK_TYPE_NONE, 3,
 		    GTK_TYPE_WIDGET,
 		    GTK_TYPE_STRING,
 		    GTK_TYPE_STRING);
   tips_query_signals[SIGNAL_WIDGET_SELECTED] =
-    gtk_signal_new ("widget_selected",
-		    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkTipsQueryClass, widget_selected),
-		    gtk_marshal_BOOLEAN__POINTER_STRING_STRING_POINTER,
-		    GTK_TYPE_BOOL, 4,
-		    GTK_TYPE_WIDGET,
-		    GTK_TYPE_STRING,
-		    GTK_TYPE_STRING,
-		    GTK_TYPE_GDK_EVENT);
-  gtk_object_class_add_signals (object_class, tips_query_signals, SIGNAL_LAST);
-
-  object_class->set_arg = gtk_tips_query_set_arg;
-  object_class->get_arg = gtk_tips_query_get_arg;
-  object_class->destroy = gtk_tips_query_destroy;
-
-  widget_class->event = gtk_tips_query_event;
-
-  class->start_query = gtk_tips_query_real_start_query;
-  class->stop_query = gtk_tips_query_real_stop_query;
-  class->widget_entered = gtk_tips_query_widget_entered;
-  class->widget_selected = NULL;
+    g_signal_newc ("widget_selected",
+		   G_TYPE_FROM_CLASS(object_class),
+		   G_SIGNAL_RUN_LAST,
+		   G_STRUCT_OFFSET(GtkTipsQueryClass, widget_selected),
+		   _gtk_boolean_handled_accumulator, NULL,
+		   gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED,
+		   G_TYPE_BOOLEAN, 4,
+		   GTK_TYPE_WIDGET,
+		   G_TYPE_STRING,
+		   G_TYPE_STRING,
+		   GDK_TYPE_EVENT);
 }
 
 static void
