@@ -52,8 +52,6 @@ static void gtk_toggle_button_paint      (GtkWidget            *widget,
 					  GdkRectangle         *area);
 static void gtk_toggle_button_size_allocate (GtkWidget         *widget,
 					     GtkAllocation     *allocation);
-static void gtk_toggle_button_draw       (GtkWidget            *widget,
-					  GdkRectangle         *area);
 static gint gtk_toggle_button_expose     (GtkWidget            *widget,
 					  GdkEventExpose       *event);
 static void gtk_toggle_button_pressed    (GtkButton            *button);
@@ -116,24 +114,11 @@ gtk_toggle_button_class_init (GtkToggleButtonClass *class)
 
   parent_class = gtk_type_class (GTK_TYPE_BUTTON);
 
-  gtk_object_add_arg_type ("GtkToggleButton::active", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_ACTIVE);
-  gtk_object_add_arg_type ("GtkToggleButton::draw_indicator", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_DRAW_INDICATOR);
-
-  toggle_button_signals[TOGGLED] =
-    gtk_signal_new ("toggled",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkToggleButtonClass, toggled),
-                    gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
-
-  gtk_object_class_add_signals (object_class, toggle_button_signals, LAST_SIGNAL);
 
   object_class->set_arg = gtk_toggle_button_set_arg;
   object_class->get_arg = gtk_toggle_button_get_arg;
 
   widget_class->size_allocate = gtk_toggle_button_size_allocate;
-  widget_class->draw = gtk_toggle_button_draw;
   widget_class->expose_event = gtk_toggle_button_expose;
   widget_class->realize = gtk_toggle_button_realize;
   widget_class->unrealize = gtk_toggle_button_unrealize;
@@ -147,6 +132,17 @@ gtk_toggle_button_class_init (GtkToggleButtonClass *class)
   button_class->leave = gtk_toggle_button_leave;
 
   class->toggled = NULL;
+
+  gtk_object_add_arg_type ("GtkToggleButton::active", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_ACTIVE);
+  gtk_object_add_arg_type ("GtkToggleButton::draw_indicator", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_DRAW_INDICATOR);
+
+  toggle_button_signals[TOGGLED] =
+    gtk_signal_new ("toggled",
+                    GTK_RUN_FIRST,
+                    GTK_CLASS_TYPE (object_class),
+                    GTK_SIGNAL_OFFSET (GtkToggleButtonClass, toggled),
+                    gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
 }
 
 static void
@@ -309,6 +305,51 @@ gtk_toggle_button_toggled (GtkToggleButton *toggle_button)
   gtk_signal_emit (GTK_OBJECT (toggle_button), toggle_button_signals[TOGGLED]);
 }
 
+/**
+ * gtk_toggle_button_set_inconsistent:
+ * @toggle_button: a #GtkToggleButton
+ * @setting: %TRUE if state is inconsistent
+ *
+ * If the user has selected a range of elements (such as some text or
+ * spreadsheet cells) that are affected by a toggle button, and the
+ * current values in that range are inconsistent, you may want to
+ * display the toggle in an "in between" state. This function turns on
+ * "in between" display.  Normally you would turn off the inconsistent
+ * state again if the user toggles the toggle button. This has to be
+ * done manually, gtk_toggle_button_set_inconsistent() only affects
+ * visual appearance, it doesn't affect the semantics of the button.
+ * 
+ **/
+void
+gtk_toggle_button_set_inconsistent (GtkToggleButton *toggle_button,
+                                    gboolean         setting)
+{
+  g_return_if_fail (GTK_IS_TOGGLE_BUTTON (toggle_button));
+  
+  setting = setting != FALSE;
+
+  if (setting != toggle_button->inconsistent)
+    {
+      toggle_button->inconsistent = setting;
+      gtk_widget_queue_draw (GTK_WIDGET (toggle_button));
+    }
+}
+
+/**
+ * gtk_toggle_button_get_inconsistent:
+ * @toggle_button: a #GtkToggleButton
+ * 
+ * Gets the value set by gtk_toggle_button_set_inconsistent().
+ * 
+ * Return value: %TRUE if the button is displayed as inconsistent, %FALSE otherwise
+ **/
+gboolean
+gtk_toggle_button_get_inconsistent (GtkToggleButton *toggle_button)
+{
+  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (toggle_button), FALSE);
+
+  return toggle_button->inconsistent;
+}
 
 static void
 gtk_toggle_button_paint (GtkWidget    *widget,
@@ -317,7 +358,9 @@ gtk_toggle_button_paint (GtkWidget    *widget,
   GtkButton *button;
   GtkToggleButton *toggle_button;
   GtkShadowType shadow_type;
+  GtkStateType state_type;
   gint width, height;
+  gboolean interior_focus;
   gint x, y;
 
   button = GTK_BUTTON (widget);
@@ -325,6 +368,8 @@ gtk_toggle_button_paint (GtkWidget    *widget,
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
+      gtk_widget_style_get (widget, "interior_focus", &interior_focus, NULL);
+      
       x = 0;
       y = 0;
       width = widget->allocation.width - GTK_CONTAINER (widget)->border_width * 2;
@@ -352,7 +397,7 @@ gtk_toggle_button_paint (GtkWidget    *widget,
           y += DEFAULT_TOP_POS;
         }
 
-      if (GTK_WIDGET_HAS_FOCUS (widget))
+      if (GTK_WIDGET_HAS_FOCUS (widget) && !interior_focus)
 	{
 	  x += 1;
 	  y += 1;
@@ -360,26 +405,44 @@ gtk_toggle_button_paint (GtkWidget    *widget,
 	  height -= 2;
 	}
 
-      if ((GTK_WIDGET_STATE (widget) == GTK_STATE_ACTIVE) ||
+      state_type = GTK_WIDGET_STATE (widget);
+      
+      if (toggle_button->inconsistent)
+        {
+          if (state_type == GTK_STATE_ACTIVE)
+            state_type = GTK_STATE_NORMAL;
+          shadow_type = GTK_SHADOW_ETCHED_IN;
+        }
+      else if ((GTK_WIDGET_STATE (widget) == GTK_STATE_ACTIVE) ||
 	  toggle_button->active)
-	shadow_type = GTK_SHADOW_IN;
+        shadow_type = GTK_SHADOW_IN;
       else
 	shadow_type = GTK_SHADOW_OUT;
-      
+
       if (button->relief != GTK_RELIEF_NONE ||
 	  (GTK_WIDGET_STATE(widget) != GTK_STATE_NORMAL &&
 	   GTK_WIDGET_STATE(widget) != GTK_STATE_INSENSITIVE))
 	gtk_paint_box (widget->style, widget->window,
-		       GTK_WIDGET_STATE (widget),
+                       state_type,
 		       shadow_type, area, widget, "togglebutton",
 		       x, y, width, height);
       
       if (GTK_WIDGET_HAS_FOCUS (widget))
 	{
-	  x -= 1;
-	  y -= 1;
-	  width += 2;
-	  height += 2;
+	  if (interior_focus)
+	    {
+	      x += widget->style->xthickness + 1;
+	      y += widget->style->xthickness + 1;
+	      width -= 2 * (widget->style->xthickness + 1);
+	      height -= 2 * (widget->style->ythickness + 1);
+	    }
+	  else
+	    {
+	      x -= 1;
+	      y -= 1;
+	      width += 2;
+	      height += 2;
+	    }
 
 	  gtk_paint_focus (widget->style, widget->window,
 			   area, widget, "togglebutton",
@@ -401,38 +464,16 @@ static gint
 gtk_toggle_button_expose (GtkWidget      *widget,
 			  GdkEventExpose *event)
 {
-  if (!GTK_WIDGET_NO_WINDOW (widget) &&
-      GTK_WIDGET_CLASS (parent_class)->expose_event)
-    return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
-  else
-    return FALSE;
-}
-
-static void
-gtk_toggle_button_draw (GtkWidget    *widget,
-			GdkRectangle *area)
-{
-  GdkRectangle child_area;
-  GdkRectangle tmp_area;
-  GtkBin *bin;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TOGGLE_BUTTON (widget));
-  g_return_if_fail (area != NULL);
-
-  bin = GTK_BIN (widget);
-
-  if (GTK_WIDGET_DRAWABLE (widget) && !GTK_WIDGET_NO_WINDOW (widget))
+  if (GTK_WIDGET_DRAWABLE (widget))
     {
-      tmp_area = *area;
-      tmp_area.x -= GTK_CONTAINER (widget)->border_width;
-      tmp_area.y -= GTK_CONTAINER (widget)->border_width;
-
-      gtk_toggle_button_paint (widget, &tmp_area);
-
-      if (bin->child && gtk_widget_intersect (bin->child, &tmp_area, &child_area))
-	gtk_widget_draw (bin->child, &child_area);
+      gtk_toggle_button_paint (widget, &event->area);
+      
+      gtk_container_propagate_expose (GTK_CONTAINER (widget),
+				      GTK_BIN (widget)->child,
+				      event);
     }
+  
+  return TRUE;
 }
 
 static void
