@@ -32,11 +32,16 @@
 #include "gtkvbox.h"
 #include "gtkhbox.h"
 #include "gtktoolbar.h"
+#include "gtkstock.h"
+#include "gtkiconfactory.h"
+#include "gtkimage.h"
 
 
 #define DEFAULT_SPACE_SIZE  5
 #define DEFAULT_SPACE_STYLE GTK_TOOLBAR_SPACE_EMPTY
 
+#define DEFAULT_ICON_SIZE GTK_ICON_SIZE_LARGE_TOOLBAR
+
 #define SPACE_LINE_DIVISION 10
 #define SPACE_LINE_START    3
 #define SPACE_LINE_END      7
@@ -75,8 +80,6 @@ static void gtk_toolbar_get_arg                  (GtkObject       *object,
 static void gtk_toolbar_destroy                  (GtkObject       *object);
 static void gtk_toolbar_map                      (GtkWidget       *widget);
 static void gtk_toolbar_unmap                    (GtkWidget       *widget);
-static void gtk_toolbar_draw                     (GtkWidget       *widget,
-				                  GdkRectangle    *area);
 static gint gtk_toolbar_expose                   (GtkWidget       *widget,
 						  GdkEventExpose  *event);
 static void gtk_toolbar_size_request             (GtkWidget       *widget,
@@ -96,6 +99,27 @@ static void gtk_real_toolbar_orientation_changed (GtkToolbar      *toolbar,
 static void gtk_real_toolbar_style_changed       (GtkToolbar      *toolbar,
 						  GtkToolbarStyle  style);
 
+static GtkWidget * gtk_toolbar_internal_insert_element (GtkToolbar          *toolbar,
+                                                        GtkToolbarChildType  type,
+                                                        GtkWidget           *widget,
+                                                        const char          *text,
+                                                        const char          *tooltip_text,
+                                                        const char          *tooltip_private_text,
+                                                        GtkWidget           *icon,
+                                                        GtkSignalFunc        callback,
+                                                        gpointer             user_data,
+                                                        gint                 position,
+                                                        gboolean             has_mnemonic);
+
+static GtkWidget * gtk_toolbar_internal_insert_item (GtkToolbar    *toolbar,
+                                                     const char    *text,
+                                                     const char    *tooltip_text,
+                                                     const char    *tooltip_private_text,
+                                                     GtkWidget     *icon,
+                                                     GtkSignalFunc  callback,
+                                                     gpointer       user_data,
+                                                     gint           position,
+                                                     gboolean       has_mnemonic);
 
 static GtkContainerClass *parent_class;
 
@@ -140,6 +164,24 @@ gtk_toolbar_class_init (GtkToolbarClass *class)
 
   parent_class = gtk_type_class (gtk_container_get_type ());
   
+  object_class->destroy = gtk_toolbar_destroy;
+  object_class->set_arg = gtk_toolbar_set_arg;
+  object_class->get_arg = gtk_toolbar_get_arg;
+
+  widget_class->map = gtk_toolbar_map;
+  widget_class->unmap = gtk_toolbar_unmap;
+  widget_class->expose_event = gtk_toolbar_expose;
+  widget_class->size_request = gtk_toolbar_size_request;
+  widget_class->size_allocate = gtk_toolbar_size_allocate;
+
+  container_class->add = gtk_toolbar_add;
+  container_class->remove = gtk_toolbar_remove;
+  container_class->forall = gtk_toolbar_forall;
+  container_class->focus = NULL;
+  
+  class->orientation_changed = gtk_real_toolbar_orientation_changed;
+  class->style_changed = gtk_real_toolbar_style_changed;
+
   toolbar_signals[ORIENTATION_CHANGED] =
     gtk_signal_new ("orientation_changed",
 		    GTK_RUN_FIRST,
@@ -156,27 +198,6 @@ gtk_toolbar_class_init (GtkToolbarClass *class)
 		    gtk_marshal_VOID__INT,
 		    GTK_TYPE_NONE, 1,
 		    GTK_TYPE_INT);
-
-  gtk_object_class_add_signals (object_class, toolbar_signals, LAST_SIGNAL);
-
-  object_class->destroy = gtk_toolbar_destroy;
-  object_class->set_arg = gtk_toolbar_set_arg;
-  object_class->get_arg = gtk_toolbar_get_arg;
-
-  widget_class->map = gtk_toolbar_map;
-  widget_class->unmap = gtk_toolbar_unmap;
-  widget_class->draw = gtk_toolbar_draw;
-  widget_class->expose_event = gtk_toolbar_expose;
-  widget_class->size_request = gtk_toolbar_size_request;
-  widget_class->size_allocate = gtk_toolbar_size_allocate;
-
-  container_class->add = gtk_toolbar_add;
-  container_class->remove = gtk_toolbar_remove;
-  container_class->forall = gtk_toolbar_forall;
-  container_class->focus = NULL;
-  
-  class->orientation_changed = gtk_real_toolbar_orientation_changed;
-  class->style_changed = gtk_real_toolbar_style_changed;
   
   gtk_object_add_arg_type ("GtkToolbar::orientation", GTK_TYPE_ORIENTATION,
 			   GTK_ARG_READWRITE, ARG_ORIENTATION);
@@ -203,6 +224,7 @@ gtk_toolbar_init (GtkToolbar *toolbar)
   toolbar->relief       = GTK_RELIEF_NORMAL;
   toolbar->space_size   = DEFAULT_SPACE_SIZE;
   toolbar->space_style  = DEFAULT_SPACE_STYLE;
+  toolbar->icon_size    = DEFAULT_ICON_SIZE;
   toolbar->tooltips     = gtk_tooltips_new ();
   toolbar->button_maxw  = 0;
   toolbar->button_maxh  = 0;
@@ -405,37 +427,6 @@ gtk_toolbar_paint_space_line (GtkWidget       *widget,
 		      widget->style->ythickness) / 2);
 }
 
-static void
-gtk_toolbar_draw (GtkWidget    *widget,
-		  GdkRectangle *area)
-{
-  GtkToolbar *toolbar;
-  GList *children;
-  GtkToolbarChild *child;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TOOLBAR (widget));
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      toolbar = GTK_TOOLBAR (widget);
-
-      for (children = toolbar->children; children; children = children->next)
-	{
-	  child = children->data;
-
-	  if (child->type == GTK_TOOLBAR_CHILD_SPACE)
-	    {
-	      if (toolbar->space_style == GTK_TOOLBAR_SPACE_LINE)
-		gtk_toolbar_paint_space_line (widget, area, child);
-	    }
-	  else if (gtk_widget_intersect (child->widget, area, &child_area))
-	    gtk_widget_draw (child->widget, &child_area);
-	}
-    }
-}
-
 static gint
 gtk_toolbar_expose (GtkWidget      *widget,
 		    GdkEventExpose *event)
@@ -443,8 +434,7 @@ gtk_toolbar_expose (GtkWidget      *widget,
   GtkToolbar *toolbar;
   GList *children;
   GtkToolbarChild *child;
-  GdkEventExpose child_event;
-
+  
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TOOLBAR (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
@@ -453,8 +443,6 @@ gtk_toolbar_expose (GtkWidget      *widget,
     {
       toolbar = GTK_TOOLBAR (widget);
 
-      child_event = *event;
-
       for (children = toolbar->children; children; children = children->next)
 	{
 	  child = children->data;
@@ -464,9 +452,10 @@ gtk_toolbar_expose (GtkWidget      *widget,
 	      if (toolbar->space_style == GTK_TOOLBAR_SPACE_LINE)
 		gtk_toolbar_paint_space_line (widget, &event->area, child);
 	    }
-	  else if (GTK_WIDGET_NO_WINDOW (child->widget)
-		   && gtk_widget_intersect (child->widget, &event->area, &child_event.area))
-	    gtk_widget_event (child->widget, (GdkEvent *) &child_event);
+	  else
+	    gtk_container_propagate_expose (GTK_CONTAINER (widget),
+					    child->widget,
+					    event);
 	}
     }
 
@@ -772,6 +761,24 @@ gtk_toolbar_prepend_item (GtkToolbar    *toolbar,
 				     0);
 }
 
+static GtkWidget *
+gtk_toolbar_internal_insert_item (GtkToolbar    *toolbar,
+                                  const char    *text,
+                                  const char    *tooltip_text,
+                                  const char    *tooltip_private_text,
+                                  GtkWidget     *icon,
+                                  GtkSignalFunc  callback,
+                                  gpointer       user_data,
+                                  gint           position,
+                                  gboolean       has_mnemonic)
+{
+  return gtk_toolbar_internal_insert_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON,
+                                              NULL, text,
+                                              tooltip_text, tooltip_private_text,
+                                              icon, callback, user_data,
+                                              position, has_mnemonic);
+}
+     
 GtkWidget *
 gtk_toolbar_insert_item (GtkToolbar    *toolbar,
 			 const char    *text,
@@ -782,13 +789,118 @@ gtk_toolbar_insert_item (GtkToolbar    *toolbar,
 			 gpointer       user_data,
 			 gint           position)
 {
-  return gtk_toolbar_insert_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON,
-				     NULL, text,
-				     tooltip_text, tooltip_private_text,
-				     icon, callback, user_data,
-				     position);
+  return gtk_toolbar_internal_insert_item (toolbar, 
+                                           text, tooltip_text, tooltip_private_text,
+                                           icon, callback, user_data,
+                                           position, FALSE);
 }
 
+/**
+ * gtk_toolbar_set_icon_size:
+ * @toolbar: A #GtkToolbar
+ * @icon_size: The #GtkIconSize that stock icons in the toolbar shall have.
+ *
+ * This function sets the size of stock icons in the toolbar. You
+ * can call it both before you add the icons and after they've been
+ * added.
+ **/
+void
+gtk_toolbar_set_icon_size (GtkToolbar  *toolbar,
+			   GtkIconSize  icon_size)
+{
+  GList *children;
+  GtkToolbarChild *child;
+  GtkImage *image;
+  gchar *stock_id;
+
+  g_return_if_fail (toolbar != NULL);
+  g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
+
+  if (toolbar->icon_size == icon_size)
+    return;
+  
+  toolbar->icon_size = icon_size;
+
+  for (children = toolbar->children; children; children = children->next)
+    {
+      child = children->data;
+      if (child->type == GTK_TOOLBAR_CHILD_BUTTON &&
+	  GTK_IS_IMAGE (child->icon))
+	{
+	  image = GTK_IMAGE (child->icon);
+	  if (gtk_image_get_storage_type (image) == GTK_IMAGE_STOCK)
+	    {
+	      gtk_image_get_stock (image, &stock_id, NULL);
+	      stock_id = g_strdup (stock_id);
+	      gtk_image_set_from_stock (image,
+					stock_id,
+					icon_size);
+	      g_free (stock_id);
+	    }
+	}
+    }
+  
+  gtk_widget_queue_resize (GTK_WIDGET (toolbar));
+}
+
+/**
+ * gtk_toolbar_insert_stock:
+ * @toolbar: A #GtkToolbar
+ * @stock_id: The id of the stock item you want to insert
+ * @tooltip_text: The text in the tooltip of the toolbar button
+ * @tooltip_private_text: The private text of the tooltip
+ * @callback: The callback called when the toolbar button is clicked.
+ * @user_data: user data passed to callback
+ * @position: The position the button shall be inserted at.
+ *            -1 means at the end.
+ *
+ * Inserts a stock item at the specified position of the toolbar.  If
+ * @stock_id is not a known stock item ID, it's inserted verbatim,
+ * except that underscores are used to mark mnemonics (see
+ * gtk_label_new_with_mnemonic()).
+ *
+ * Returns: the inserted widget
+ */
+GtkWidget*
+gtk_toolbar_insert_stock (GtkToolbar      *toolbar,
+			  const gchar     *stock_id,
+			  const char      *tooltip_text,
+			  const char      *tooltip_private_text,
+			  GtkSignalFunc    callback,
+			  gpointer         user_data,
+			  gint             position)
+{
+  GtkStockItem item;
+  GtkWidget *image;
+
+  if (gtk_stock_lookup (stock_id, &item))
+    {
+      image = gtk_image_new_from_stock (stock_id, toolbar->icon_size);
+
+      return gtk_toolbar_internal_insert_item (toolbar,
+                                               item.label,
+                                               tooltip_text,
+                                               tooltip_private_text,
+                                               image,
+                                               callback,
+                                               user_data,
+                                               position,
+                                               TRUE);
+    }
+  else
+    return gtk_toolbar_internal_insert_item (toolbar,
+                                             stock_id,
+                                             tooltip_text,
+                                             tooltip_private_text,
+                                             NULL,
+                                             callback,
+                                             user_data,
+                                             position,
+                                             TRUE);
+}
+
+
+
 void
 gtk_toolbar_append_space (GtkToolbar *toolbar)
 {
@@ -904,6 +1016,35 @@ gtk_toolbar_insert_element (GtkToolbar          *toolbar,
 			    GtkSignalFunc        callback,
 			    gpointer             user_data,
 			    gint                 position)
+{
+  g_return_val_if_fail (toolbar != NULL, NULL);
+  g_return_val_if_fail (GTK_IS_TOOLBAR (toolbar), NULL);
+  if (type == GTK_TOOLBAR_CHILD_WIDGET)
+    {
+      g_return_val_if_fail (widget != NULL, NULL);
+      g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+    }
+  else if (type != GTK_TOOLBAR_CHILD_RADIOBUTTON)
+    g_return_val_if_fail (widget == NULL, NULL);
+  
+  return gtk_toolbar_internal_insert_element (toolbar, type, widget, text,
+                                              tooltip_text, tooltip_private_text,
+                                              icon, callback, user_data,
+                                              position, FALSE);
+}
+
+static GtkWidget *
+gtk_toolbar_internal_insert_element (GtkToolbar          *toolbar,
+                                     GtkToolbarChildType  type,
+                                     GtkWidget           *widget,
+                                     const char          *text,
+                                     const char          *tooltip_text,
+                                     const char          *tooltip_private_text,
+                                     GtkWidget           *icon,
+                                     GtkSignalFunc        callback,
+                                     gpointer             user_data,
+                                     gint                 position,
+                                     gboolean             has_mnemonic)
 {
   GtkToolbarChild *child;
   GtkWidget *box;
@@ -978,7 +1119,10 @@ gtk_toolbar_insert_element (GtkToolbar          *toolbar,
 
       if (text)
 	{
-	  child->label = gtk_label_new (text);
+          if (has_mnemonic)
+            child->label = gtk_label_new_with_mnemonic (text);
+          else
+            child->label = gtk_label_new (text);
 	  gtk_box_pack_end (GTK_BOX (box), child->label, FALSE, FALSE, 0);
 	  if (toolbar->style != GTK_TOOLBAR_ICONS)
 	    gtk_widget_show (child->label);
@@ -1077,7 +1221,7 @@ gtk_toolbar_set_space_style (GtkToolbar           *toolbar,
 
 void
 gtk_toolbar_set_tooltips (GtkToolbar *toolbar,
-			  gint        enable)
+			  gboolean    enable)
 {
   g_return_if_fail (toolbar != NULL);
   g_return_if_fail (GTK_IS_TOOLBAR (toolbar));
