@@ -87,6 +87,7 @@ struct _GtkToolbar
   GtkToolbarStyle  style;
   gint             space_size; /* big optional space between buttons */
   GtkToolbarSpaceStyle space_style;
+  GtkIconSize      icon_size;
 
   GtkTooltips     *tooltips;
 
@@ -134,6 +135,19 @@ GtkWidget* gtk_toolbar_insert_item     (GtkToolbar      *toolbar,
 					gpointer         user_data,
 					gint             position);
 
+/* Stock Items */
+void       gtk_toolbar_set_icon_size   (GtkToolbar      *toolbar,
+					GtkIconSize      icon_size);
+GtkWidget* gtk_toolbar_insert_stock    (GtkToolbar      *toolbar,
+					const gchar     *stock_id,
+					const char      *tooltip_text,
+					const char      *tooltip_private_text,
+					GtkSignalFunc    callback,
+					gpointer         user_data,
+					gint             position);
+
+
+
 /* Space Items */
 void       gtk_toolbar_append_space    (GtkToolbar      *toolbar);
 void       gtk_toolbar_prepend_space   (GtkToolbar      *toolbar);
@@ -197,7 +211,7 @@ void       gtk_toolbar_set_space_size        (GtkToolbar           *toolbar,
 void       gtk_toolbar_set_space_style       (GtkToolbar           *toolbar,
 					      GtkToolbarSpaceStyle  space_style);
 void       gtk_toolbar_set_tooltips          (GtkToolbar           *toolbar,
-					      gint                  enable);
+					      gboolean		    enable);
 void       gtk_toolbar_set_button_relief     (GtkToolbar           *toolbar,
 					      GtkReliefStyle        relief);
 GtkReliefStyle gtk_toolbar_get_button_relief (GtkToolbar           *toolbar);
