@@ -29,7 +29,27 @@ extern "C" {
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkrbtree.h>
 
-typedef enum {
+
+/* the width of the column resize windows */
+#define TREE_VIEW_EXPANDER_WIDTH 14
+#define TREE_VIEW_EXPANDER_HEIGHT 14
+
+/* The TREE_VIEW_VERTICAL_SEPARATOR is the space between rows. The
+ * GTK_RBNODE_GET_HEIGHT() value includes the separators; the offset
+ * of each row points to the center of the inter-row space. For an odd
+ * separator, the extra pixel by convention goes _below_ the row.  So
+ * a node in the rbtree owns TREE_VIEW_VERTICAL_SEPARATOR/2 pixels
+ * above the row, and TREE_VIEW_VERTICAL_SEPARATOR/2 +
+ * TREE_VIEW_VERTICAL_SEPARATOR%2 pixels below the row.
+ */
+
+#define TREE_VIEW_VERTICAL_SEPARATOR 2
+#define TREE_VIEW_HORIZONTAL_SEPARATOR 0
+
+#define TREE_VIEW_DRAG_WIDTH 6
+
+typedef enum
+{
   GTK_TREE_VIEW_IS_LIST = 1 << 0,
   GTK_TREE_VIEW_SHOW_EXPANDERS = 1 << 1,
   GTK_TREE_VIEW_IN_COLUMN_RESIZE = 1 << 2,
@@ -43,7 +63,7 @@ typedef enum {
 #define GTK_TREE_VIEW_UNSET_FLAG(tree_view, flag) G_STMT_START{ (tree_view->priv->flags&=~(flag)); }G_STMT_END
 #define GTK_TREE_VIEW_FLAG_SET(tree_view, flag)   ((tree_view->priv->flags&flag)==flag)
 #define TREE_VIEW_HEADER_HEIGHT(tree_view)        (GTK_TREE_VIEW_FLAG_SET (tree_view, GTK_TREE_VIEW_HEADERS_VISIBLE)?tree_view->priv->header_height:0)
-#define TREE_VIEW_COLUMN_SIZE(column)             (CLAMP (column->size, (column->min_width!=-1)?column->min_width:column->size, (column->max_width!=-1)?column->max_width:column->size))
+#define TREE_VIEW_COLUMN_WIDTH(column)             (CLAMP (column->width, (column->min_width!=-1)?column->min_width:column->width, (column->max_width!=-1)?column->max_width:column->width))
 #define TREE_VIEW_DRAW_EXPANDERS(tree_view)       (!GTK_TREE_VIEW_FLAG_SET (tree_view, GTK_TREE_VIEW_IS_LIST)&&GTK_TREE_VIEW_FLAG_SET (tree_view, GTK_TREE_VIEW_SHOW_EXPANDERS))
 
 struct _GtkTreeViewPrivate
@@ -68,9 +88,16 @@ struct _GtkTreeViewPrivate
   GdkWindow *bin_window;
   GdkWindow *header_window;
 
+  gint expander_column;
+
+  /* Focus code */
+  gboolean header_has_focus;
+  GtkTreeViewColumn *focus_column;
+  GtkTreeViewColumn *scroll_column;
+
   /* Selection stuff */
-  GtkTreePath *anchor;
-  GtkTreePath *cursor;
+  GtkTreeRowReference *anchor;
+  GtkTreeRowReference *cursor;
 
   /* Column Resizing */
   GdkCursor *cursor_drag;
@@ -81,15 +108,35 @@ struct _GtkTreeViewPrivate
   /* Prelight information */
   GtkRBNode *prelight_node;
   GtkRBTree *prelight_tree;
-  gint prelight_offset;
 
   /* Selection information */
   GtkTreeSelection *selection;
 
   /* Header information */
-  gint columns;
-  GList *column;
+  gint n_columns;
+  GList *columns;
   gint header_height;
+
+  /* Scroll timeout (e.g. during dnd) */
+  guint scroll_timeout;
+
+  /* Row drag-and-drop */
+  GtkTreeRowReference *drag_dest_row;
+  GtkTreeViewDropPosition drag_dest_pos;
+  guint open_dest_timeout;
+
+  gint pressed_button;
+  gint press_start_x;
+  gint press_start_y;
+
+  /* Scroll-to functionality when unrealized */
+  GtkTreePath *scroll_to_path;
+  GtkTreeViewColumn *scroll_to_column;
+  gfloat scroll_to_row_align;
+  gfloat scroll_to_col_align;
+
+  /* hint to display rows in alternating colors */
+  guint has_rules : 1;
 };
 
 #ifdef __GNUC__
@@ -176,9 +223,22 @@ gboolean     _gtk_tree_view_find_node                 (GtkTreeView       *tree_v
 GtkTreePath *_gtk_tree_view_find_path                 (GtkTreeView       *tree_view,
 						       GtkRBTree         *tree,
 						       GtkRBNode         *node);
-void         _gtk_tree_view_set_size                  (GtkTreeView       *tree_view,
-						       gint               width,
-						       gint               height);
+void         _gtk_tree_view_update_size               (GtkTreeView       *tree_view);
+
+
+void _gtk_tree_view_column_create_button    (GtkTreeViewColumn *column);
+void _gtk_tree_view_column_realize_button   (GtkTreeViewColumn *column);
+void _gtk_tree_view_column_unrealize_button (GtkTreeViewColumn *column);
+void _gtk_tree_view_column_set_tree_view    (GtkTreeViewColumn *column,
+					     GtkTreeView       *tree_view);
+void _gtk_tree_view_column_unset_tree_view  (GtkTreeViewColumn *column);
+
+
+GtkTreeSelection* _gtk_tree_selection_new                (void);
+GtkTreeSelection* _gtk_tree_selection_new_with_tree_view (GtkTreeView      *tree_view);
+void              _gtk_tree_selection_set_tree_view      (GtkTreeSelection *selection,
+                                                          GtkTreeView      *tree_view);
+
 
 
 
