@@ -20,7 +20,6 @@
 #ifndef __GTK_TREE_SELECTION_H__
 #define __GTK_TREE_SELECTION_H__
 
-#include <gobject/gobject.h>
 #include <gtk/gtkobject.h>
 #include <gtk/gtktreeview.h>
 
@@ -38,7 +37,7 @@ typedef enum
 {
   GTK_TREE_SELECTION_SINGLE,
   GTK_TREE_SELECTION_MULTI
-} GtkTreeSelectionType;
+} GtkTreeSelectionMode;
 
 typedef gboolean (* GtkTreeSelectionFunc)    (GtkTreeSelection  *selection,
 					      GtkTreeModel      *model,
@@ -52,10 +51,13 @@ struct _GtkTreeSelection
 {
   GtkObject parent;
 
+  /*< private >*/
+  
   GtkTreeView *tree_view;
-  GtkTreeSelectionType type;
+  GtkTreeSelectionMode type;
   GtkTreeSelectionFunc user_func;
   gpointer user_data;
+  GtkDestroyNotify destroy;
 };
 
 struct _GtkTreeSelectionClass
@@ -67,14 +69,15 @@ struct _GtkTreeSelectionClass
 
 
 GtkType          gtk_tree_selection_get_type            (void);
-GtkObject       *gtk_tree_selection_new                 (void);
-void             gtk_tree_selection_set_type            (GtkTreeSelection            *selection,
-							 GtkTreeSelectionType         type);
+
+void             gtk_tree_selection_set_mode            (GtkTreeSelection            *selection,
+							 GtkTreeSelectionMode         type);
 void             gtk_tree_selection_set_select_function (GtkTreeSelection            *selection,
 							 GtkTreeSelectionFunc         func,
-							 gpointer                     data);
+							 gpointer                     data,
+							 GtkDestroyNotify             destroy);
 gpointer         gtk_tree_selection_get_user_data       (GtkTreeSelection            *selection);
-
+GtkTreeView*     gtk_tree_selection_get_tree_view       (GtkTreeSelection            *selection);
 
 /* Only meaningful if GTK_TREE_SELECTION_SINGLE is set */
 /* Use selected_foreach for GTK_TREE_SELECTION_MULTI */
@@ -101,12 +104,6 @@ void             gtk_tree_selection_select_range        (GtkTreeSelection
 							 GtkTreePath                 *end_path);
 
 
-/*< private >*/
-GtkObject       *gtk_tree_selection_new_with_tree_view  (GtkTreeView                 *tree_view);
-void             gtk_tree_selection_set_tree_view       (GtkTreeSelection            *selection,
-							 GtkTreeView                 *tree_view);
-
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
