@@ -59,8 +59,7 @@ typedef enum	/*< skip >*/
   GTK_TYPE_DOUBLE	= G_TYPE_DOUBLE,
   GTK_TYPE_STRING	= G_TYPE_STRING,
   GTK_TYPE_BOXED	= G_TYPE_BOXED,
-  GTK_TYPE_POINTER	= G_TYPE_POINTER,
-  GTK_TYPE_SIGNAL	= G_TYPE_GTK_SIGNAL
+  GTK_TYPE_POINTER	= G_TYPE_POINTER
 } GtkFundamentalType;
 
 
@@ -121,7 +120,7 @@ typedef void (*GtkCallbackMarshal)  (GtkObject    *object,
 				     gpointer      data,
 				     guint         n_args,
 				     GtkArg       *args);
-typedef void (*GtkSignalFunc)       ();
+typedef void (*GtkSignalFunc)       (void);
 typedef GSignalCMarshaller          GtkSignalMarshaller;
 #define GTK_SIGNAL_FUNC(f)	    ((GtkSignalFunc) (f))
 
@@ -224,8 +223,8 @@ gpointer	gtk_type_class	(GtkType	 type) G_GNUC_CONST;
 gpointer	gtk_type_new	(GtkType	 type);
 
 
-/* --- initialize the type system --- */
-void		gtk_type_init	(void);
+/* deprecated, use g_type_init() instead */
+void		gtk_type_init	(GTypeDebugFlags debug_flags);
 
 
 /* --- compatibility defines --- */
