@@ -32,8 +32,8 @@ static void     gtk_vpaned_size_request   (GtkWidget      *widget,
 					   GtkRequisition *requisition);
 static void     gtk_vpaned_size_allocate  (GtkWidget      *widget,
 					   GtkAllocation  *allocation);
-static void     gtk_vpaned_draw           (GtkWidget      *widget,
-					   GdkRectangle   *area);
+static gint     gtk_vpaned_expose         (GtkWidget      *widget,
+                                           GdkEventExpose *expose);
 static void     gtk_vpaned_xor_line       (GtkPaned       *paned);
 static gboolean gtk_vpaned_button_press   (GtkWidget      *widget,
 					   GdkEventButton *event);
@@ -42,6 +42,8 @@ static gboolean gtk_vpaned_button_release (GtkWidget      *widget,
 static gboolean gtk_vpaned_motion         (GtkWidget      *widget,
 					   GdkEventMotion *event);
 
+static gpointer parent_class;
+
 GtkType
 gtk_vpaned_get_type (void)
 {
@@ -72,11 +74,13 @@ gtk_vpaned_class_init (GtkVPanedClass *class)
 {
   GtkWidgetClass *widget_class;
 
+  parent_class = gtk_type_class (GTK_TYPE_PANED);
+  
   widget_class = (GtkWidgetClass *) class;
 
   widget_class->size_request = gtk_vpaned_size_request;
   widget_class->size_allocate = gtk_vpaned_size_allocate;
-  widget_class->draw = gtk_vpaned_draw;
+  widget_class->expose_event = gtk_vpaned_expose;
   widget_class->button_press_event = gtk_vpaned_button_press;
   widget_class->button_release_event = gtk_vpaned_button_release;
   widget_class->motion_notify_event = gtk_vpaned_motion;
@@ -109,14 +113,12 @@ static void
 gtk_vpaned_size_request (GtkWidget      *widget,
 			 GtkRequisition *requisition)
 {
-  GtkPaned *paned;
+  GtkPaned *paned = GTK_PANED (widget);
   GtkRequisition child_requisition;
+  gint handle_size;
 
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_VPANED (widget));
-  g_return_if_fail (requisition != NULL);
+  gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
 
-  paned = GTK_PANED (widget);
   requisition->width = 0;
   requisition->height = 0;
 
@@ -136,7 +138,7 @@ gtk_vpaned_size_request (GtkWidget      *widget,
       requisition->height += child_requisition.height;
     }
 
-  requisition->height += GTK_CONTAINER (paned)->border_width * 2 + paned->handle_size;
+  requisition->height += GTK_CONTAINER (paned)->border_width * 2 + handle_size;
   requisition->width += GTK_CONTAINER (paned)->border_width * 2;
 }
 
@@ -144,20 +146,18 @@ static void
 gtk_vpaned_size_allocate (GtkWidget     *widget,
 			  GtkAllocation *allocation)
 {
-  GtkPaned *paned;
+  GtkPaned *paned = GTK_PANED (widget);
   GtkRequisition child1_requisition;
   GtkRequisition child2_requisition;
   GtkAllocation child1_allocation;
   GtkAllocation child2_allocation;
   gint border_width;
+  gint handle_size;
 
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_VPANED (widget));
-  g_return_if_fail (allocation != NULL);
-  
   widget->allocation = *allocation;
-  paned = GTK_PANED (widget);
+
   border_width = GTK_CONTAINER (widget)->border_width;
+  gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
 
   if (paned->child1)
     gtk_widget_get_child_requisition (paned->child1, &child1_requisition);
@@ -170,8 +170,8 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
     child2_requisition.height = 0;
     
   gtk_paned_compute_position (paned,
-			      MAX (1, (gint) widget->allocation.height
-				   - (gint) paned->handle_size
+			      MAX (1, widget->allocation.height
+				   - handle_size
 				   - 2 * border_width),
 			      child1_requisition.height,
 			      child2_requisition.height);
@@ -181,7 +181,7 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
   paned->handle_xpos = border_width;
   paned->handle_ypos = paned->child1_size + border_width;
   paned->handle_width = MAX (1, (gint) widget->allocation.width - 2 * border_width);
-  paned->handle_height = paned->handle_size;
+  paned->handle_height = handle_size;
 
   if (GTK_WIDGET_REALIZED(widget))
     {
@@ -194,7 +194,7 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
 			      paned->handle_xpos,
 			      paned->handle_ypos,
 			      paned->handle_width,
-			      paned->handle_size);
+			      handle_size);
     }
 
   child1_allocation.width = child2_allocation.width = MAX (1, (gint) allocation->width - border_width * 2);
@@ -224,54 +224,48 @@ gtk_vpaned_size_allocate (GtkWidget     *widget,
     }
 }
 
-static void
-gtk_vpaned_draw (GtkWidget    *widget,
-		 GdkRectangle *area)
+static gint
+gtk_vpaned_expose (GtkWidget      *widget,
+                   GdkEventExpose *event)
 {
   GtkPaned *paned;
-  GdkRectangle handle_area, child_area;
   guint16 border_width;
 
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_PANED (widget));
+  g_return_val_if_fail (widget != NULL, FALSE);
+  g_return_val_if_fail (GTK_IS_PANED (widget), FALSE);
 
   if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget))
     {
       paned = GTK_PANED (widget);
       border_width = GTK_CONTAINER (paned)->border_width;
 
-      gdk_window_clear_area (widget->window,
-			     area->x, area->y, area->width,
-			     area->height);
+      if (event->window == widget->window)
+        {
+          gdk_window_clear_area (widget->window,
+                                 event->area.x, event->area.y,
+                                 event->area.width,
+                                 event->area.height);
 
-      handle_area.x = paned->handle_xpos;
-      handle_area.y = paned->handle_ypos;
-      handle_area.width = paned->handle_width;
-      handle_area.height = paned->handle_size;
-
-      if (gdk_rectangle_intersect (&handle_area, area, &child_area))
-	{
-	  child_area.x -= paned->handle_xpos;
-	  child_area.y -= paned->handle_ypos;
 
+          /* Chain up to draw children */
+          GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+        }
+      else if (event->window == paned->handle)
+        {
 	  gtk_paint_handle (widget->style,
 			    paned->handle,
 			    GTK_STATE_NORMAL,
 			    GTK_SHADOW_NONE,
-			    &child_area,
+                            &event->area,
 			    widget,
 			    "paned",
 			    0, 0, -1, -1,
 			    GTK_ORIENTATION_HORIZONTAL);
 
 	}
-      /* Redraw the children
-       */
-      if (paned->child1 && gtk_widget_intersect (paned->child1, area, &child_area))
-	gtk_widget_draw (paned->child1, &child_area);
-      if (paned->child2 && gtk_widget_intersect (paned->child2, area, &child_area))
-	gtk_widget_draw (paned->child2, &child_area);
     }
+
+  return FALSE;
 }
 
 static void
@@ -280,9 +274,12 @@ gtk_vpaned_xor_line (GtkPaned *paned)
   GtkWidget *widget;
   GdkGCValues values;
   guint16 ypos;
+  gint handle_size;
 
   widget = GTK_WIDGET (paned);
 
+  gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
+
   if (!paned->xor_gc)
     {
       values.function = GDK_INVERT;
@@ -296,7 +293,7 @@ gtk_vpaned_xor_line (GtkPaned *paned)
 			      GDK_CAP_NOT_LAST, GDK_JOIN_BEVEL);
 
   ypos = paned->child1_size
-    + GTK_CONTAINER (paned)->border_width + paned->handle_size / 2;
+    + GTK_CONTAINER (paned)->border_width + handle_size / 2;
 
   gdk_draw_line (widget->window, paned->xor_gc,
 		 0,
@@ -309,12 +306,10 @@ static gboolean
 gtk_vpaned_button_press (GtkWidget      *widget,
 			 GdkEventButton *event)
 {
-  GtkPaned *paned;
+  GtkPaned *paned = GTK_PANED (widget);
+  gint handle_size;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_PANED (widget), FALSE);
-
-  paned = GTK_PANED (widget);
+  gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
 
   if (!paned->in_drag &&
       (event->window == paned->handle) && (event->button == 1))
@@ -327,10 +322,10 @@ gtk_vpaned_button_press (GtkWidget      *widget,
 			| GDK_BUTTON1_MOTION_MASK
 			| GDK_BUTTON_RELEASE_MASK, NULL, NULL,
 			event->time);
-      paned->child1_size += event->y - paned->handle_size / 2;
+      paned->child1_size += event->y - handle_size / 2;
       paned->child1_size = CLAMP (paned->child1_size, 0,
 				  widget->allocation.height -
-				  paned->handle_size -
+				  handle_size -
 				  2 * GTK_CONTAINER (paned)->border_width);
       gtk_vpaned_xor_line(paned);
 
@@ -369,22 +364,20 @@ static gboolean
 gtk_vpaned_motion (GtkWidget      *widget,
 		   GdkEventMotion *event)
 {
-  GtkPaned *paned;
+  GtkPaned *paned = GTK_PANED (widget);
   gint y;
+  gint handle_size;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_PANED (widget), FALSE);
+  gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
 
   if (event->is_hint || event->window != widget->window)
     gtk_widget_get_pointer (widget, NULL, &y);
   else
     y = event->y;
 
-  paned = GTK_PANED (widget);
-
   if (paned->in_drag)
     {
-      gint size = y - GTK_CONTAINER(paned)->border_width - paned->handle_size / 2;
+      gint size = y - GTK_CONTAINER(paned)->border_width - handle_size / 2;
 
       gtk_vpaned_xor_line (paned);
       paned->child1_size = CLAMP (size, paned->min_position, paned->max_position);
