@@ -55,11 +55,13 @@ typedef enum
   GTK_HAS_FOCUS        = 1 << 12,
 
   /* widget is allowed to receive the default via gtk_widget_grab_default
-   * and will reserve space to draw the default if possible */
+   * and will reserve space to draw the default if possible
+   */
   GTK_CAN_DEFAULT      = 1 << 13,
 
   /* the widget currently is receiving the default action and should be drawn
-   * appropriately if possible */
+   * appropriately if possible
+   */
   GTK_HAS_DEFAULT      = 1 << 14,
 
   GTK_HAS_GRAB	       = 1 << 15,
@@ -69,7 +71,8 @@ typedef enum
   GTK_APP_PAINTABLE    = 1 << 19,
 
   /* the widget when focused will receive the default action and have
-   * HAS_DEFAULT set even if there is a different widget set as default */
+   * HAS_DEFAULT set even if there is a different widget set as default
+   */
   GTK_RECEIVES_DEFAULT = 1 << 20,
 
   GTK_DOUBLE_BUFFERED  = 1 << 21
@@ -119,18 +122,18 @@ typedef enum
  */
 #define GTK_WIDGET_SET_FLAGS(wid,flag)	  G_STMT_START{ (GTK_WIDGET_FLAGS (wid) |= (flag)); }G_STMT_END
 #define GTK_WIDGET_UNSET_FLAGS(wid,flag)  G_STMT_START{ (GTK_WIDGET_FLAGS (wid) &= ~(flag)); }G_STMT_END
-  
-  
-  
-typedef struct _GtkRequisition	  GtkRequisition;
-typedef struct _GdkRectangle	  GtkAllocation;
-typedef struct _GtkSelectionData GtkSelectionData;
-typedef struct _GtkWidgetClass	  GtkWidgetClass;
-typedef struct _GtkWidgetAuxInfo  GtkWidgetAuxInfo;
-typedef struct _GtkWidgetShapeInfo GtkWidgetShapeInfo;
 
-typedef void (*GtkCallback) (GtkWidget *widget,
-			     gpointer	data);
+
+/* forward declaration to avoid excessive includes (and concurrent includes)
+ */
+typedef struct _GtkRequisition	   GtkRequisition;
+typedef struct _GdkRectangle	   GtkAllocation;
+typedef struct _GtkSelectionData   GtkSelectionData;
+typedef struct _GtkWidgetClass	   GtkWidgetClass;
+typedef struct _GtkWidgetAuxInfo   GtkWidgetAuxInfo;
+typedef struct _GtkWidgetShapeInfo GtkWidgetShapeInfo;
+typedef void     (*GtkCallback)        (GtkWidget        *widget,
+					gpointer	  data);
 
 /* A requisition is a desired amount of space which a
  *  widget may request.
@@ -242,10 +245,6 @@ struct _GtkWidgetClass
   void (* unmap)	       (GtkWidget        *widget);
   void (* realize)	       (GtkWidget        *widget);
   void (* unrealize)	       (GtkWidget        *widget);
-  void (* draw)		       (GtkWidget        *widget,
-				GdkRectangle     *area);
-  void (* draw_focus)	       (GtkWidget        *widget);
-  void (* draw_default)	       (GtkWidget        *widget);
   void (* size_request)	       (GtkWidget        *widget,
 				GtkRequisition   *requisition);
   void (* size_allocate)       (GtkWidget        *widget,
@@ -254,13 +253,14 @@ struct _GtkWidgetClass
 				GtkStateType   	  previous_state);
   void (* parent_set)	       (GtkWidget        *widget,
 				GtkWidget        *previous_parent);
+  void (* hierarchy_changed)   (GtkWidget        *widget);
   void (* style_set)	       (GtkWidget        *widget,
 				GtkStyle         *previous_style);
   void (* direction_changed)   (GtkWidget        *widget,
 				GtkTextDirection  previous_direction);
   
   /* accelerators */
-  gint (* add_accelerator)     (GtkWidget      *widget,
+  void (* add_accelerator)     (GtkWidget      *widget,
 				guint           accel_signal_id,
 				GtkAccelGroup  *accel_group,
 				guint           accel_key,
@@ -271,63 +271,69 @@ struct _GtkWidgetClass
 				guint           accel_key,
 				GdkModifierType accel_mods);
 
+  /* Mnemonics */
+  gboolean (* mnemonic_activate) (GtkWidget    *widget,
+				  gboolean      group_cycling);
+  
   /* explicit focus */
   void (* grab_focus)          (GtkWidget      *widget);
   
   /* events */
-  gint (* event)		   (GtkWidget	       *widget,
-				    GdkEvent	       *event);
-  gint (* button_press_event)	   (GtkWidget	       *widget,
-				    GdkEventButton     *event);
-  gint (* button_release_event)	   (GtkWidget	       *widget,
-				    GdkEventButton     *event);
-  gint (* scroll_event)            (GtkWidget          *widget,
-				    GdkEventScroll     *event);
-  gint (* motion_notify_event)	   (GtkWidget	       *widget,
-				    GdkEventMotion     *event);
-  gint (* delete_event)		   (GtkWidget	       *widget,
-				    GdkEventAny	       *event);
-  gint (* destroy_event)	   (GtkWidget	       *widget,
-				    GdkEventAny	       *event);
-  gint (* expose_event)		   (GtkWidget	       *widget,
-				    GdkEventExpose     *event);
-  gint (* key_press_event)	   (GtkWidget	       *widget,
-				    GdkEventKey	       *event);
-  gint (* key_release_event)	   (GtkWidget	       *widget,
-				    GdkEventKey	       *event);
-  gint (* enter_notify_event)	   (GtkWidget	       *widget,
-				    GdkEventCrossing   *event);
-  gint (* leave_notify_event)	   (GtkWidget	       *widget,
-				    GdkEventCrossing   *event);
-  gint (* configure_event)	   (GtkWidget	       *widget,
-				    GdkEventConfigure  *event);
-  gint (* focus_in_event)	   (GtkWidget	       *widget,
-				    GdkEventFocus      *event);
-  gint (* focus_out_event)	   (GtkWidget	       *widget,
-				    GdkEventFocus      *event);
-  gint (* map_event)		   (GtkWidget	       *widget,
-				    GdkEventAny	       *event);
-  gint (* unmap_event)		   (GtkWidget	       *widget,
-				    GdkEventAny	       *event);
-  gint (* property_notify_event)   (GtkWidget	       *widget,
-				    GdkEventProperty   *event);
-  gint (* selection_clear_event)   (GtkWidget	       *widget,
-				    GdkEventSelection  *event);
-  gint (* selection_request_event) (GtkWidget	       *widget,
-				    GdkEventSelection  *event);
-  gint (* selection_notify_event)  (GtkWidget	       *widget,
-				    GdkEventSelection  *event);
-  gint (* proximity_in_event)	   (GtkWidget	       *widget,
-				    GdkEventProximity  *event);
-  gint (* proximity_out_event)	   (GtkWidget	       *widget,
-				    GdkEventProximity  *event);
-  gint (* visibility_notify_event)  (GtkWidget	       *widget,
-				     GdkEventVisibility *event);
-  gint (* client_event)		   (GtkWidget	       *widget,
-				    GdkEventClient     *event);
-  gint (* no_expose_event)	   (GtkWidget	       *widget,
-				    GdkEventAny	       *event);
-
+  gboolean (* event)			(GtkWidget	     *widget,
+					 GdkEvent	     *event);
+  gboolean (* button_press_event)	(GtkWidget	     *widget,
+					 GdkEventButton      *event);
+  gboolean (* button_release_event)	(GtkWidget	     *widget,
+					 GdkEventButton      *event);
+  gboolean (* scroll_event)		(GtkWidget           *widget,
+					 GdkEventScroll      *event);
+  gboolean (* motion_notify_event)	(GtkWidget	     *widget,
+					 GdkEventMotion      *event);
+  gboolean (* delete_event)		(GtkWidget	     *widget,
+					 GdkEventAny	     *event);
+  gboolean (* destroy_event)		(GtkWidget	     *widget,
+					 GdkEventAny	     *event);
+  gboolean (* expose_event)		(GtkWidget	     *widget,
+					 GdkEventExpose      *event);
+  gboolean (* key_press_event)		(GtkWidget	     *widget,
+					 GdkEventKey	     *event);
+  gboolean (* key_release_event)	(GtkWidget	     *widget,
+					 GdkEventKey	     *event);
+  gboolean (* enter_notify_event)	(GtkWidget	     *widget,
+					 GdkEventCrossing    *event);
+  gboolean (* leave_notify_event)	(GtkWidget	     *widget,
+					 GdkEventCrossing    *event);
+  gboolean (* configure_event)		(GtkWidget	     *widget,
+					 GdkEventConfigure   *event);
+  gboolean (* focus_in_event)		(GtkWidget	     *widget,
+					 GdkEventFocus       *event);
+  gboolean (* focus_out_event)		(GtkWidget	     *widget,
+					 GdkEventFocus       *event);
+  gboolean (* map_event)		(GtkWidget	     *widget,
+					 GdkEventAny	     *event);
+  gboolean (* unmap_event)		(GtkWidget	     *widget,
+					 GdkEventAny	     *event);
+  gboolean (* property_notify_event)	(GtkWidget	     *widget,
+					 GdkEventProperty    *event);
+  gboolean (* selection_clear_event)	(GtkWidget	     *widget,
+					 GdkEventSelection   *event);
+  gboolean (* selection_request_event)	(GtkWidget	     *widget,
+					 GdkEventSelection   *event);
+  gboolean (* selection_notify_event)	(GtkWidget	     *widget,
+					 GdkEventSelection   *event);
+  gboolean (* proximity_in_event)	(GtkWidget	     *widget,
+					 GdkEventProximity   *event);
+  gboolean (* proximity_out_event)	(GtkWidget	     *widget,
+					 GdkEventProximity   *event);
+  gboolean (* visibility_notify_event)	(GtkWidget	     *widget,
+					 GdkEventVisibility  *event);
+  gboolean (* client_event)		(GtkWidget	     *widget,
+					 GdkEventClient	     *event);
+  gboolean (* no_expose_event)		(GtkWidget	     *widget,
+					 GdkEventAny	     *event);
+  gboolean (* window_state_event)	(GtkWidget	     *widget,
+					 GdkEventWindowState *event);
+  
   /* selection */
   void (* selection_get)           (GtkWidget          *widget,
 				    GtkSelectionData   *selection_data,
@@ -371,12 +377,11 @@ struct _GtkWidgetClass
 				    GtkSelectionData   *selection_data,
 				    guint               info,
 				    guint               time);
-  
-  /* action signals */
-  void (* debug_msg)		   (GtkWidget	       *widget,
-				    const gchar	       *string);
 
-  /* Padding for future expandsion */
+  /* Signals used only for keybindings */
+  void (* popup_menu)              (GtkWidget          *widget);
+  
+  /* Padding for future expansion */
   GtkFunction pad1;
   GtkFunction pad2;
   GtkFunction pad3;
@@ -385,10 +390,12 @@ struct _GtkWidgetClass
 
 struct _GtkWidgetAuxInfo
 {
-  gint16  x;
-  gint16  y;
-  gint16 width;
-  gint16 height;
+  gint x;
+  gint y;
+  gint width;
+  gint height;
+  guint x_set : 1;
+  guint y_set : 1;
 };
 
 struct _GtkWidgetShapeInfo
@@ -400,27 +407,16 @@ struct _GtkWidgetShapeInfo
 
 GtkType	   gtk_widget_get_type		  (void) G_GNUC_CONST;
 GtkWidget* gtk_widget_new		  (GtkType		type,
-					   const gchar	       *first_arg_name,
+					   const gchar	       *first_property_name,
 					   ...);
-GtkWidget* gtk_widget_newv		  (GtkType		type,
-					   guint		nargs,
-					   GtkArg	       *args);
 GtkWidget* gtk_widget_ref		  (GtkWidget	       *widget);
 void	   gtk_widget_unref		  (GtkWidget	       *widget);
 void	   gtk_widget_destroy		  (GtkWidget	       *widget);
 void	   gtk_widget_destroyed		  (GtkWidget	       *widget,
 					   GtkWidget	      **widget_pointer);
-void	   gtk_widget_get		  (GtkWidget	       *widget,
-					   GtkArg	       *arg);
-void	   gtk_widget_getv		  (GtkWidget	       *widget,
-					   guint		nargs,
-					   GtkArg	       *args);
 void	   gtk_widget_set		  (GtkWidget	       *widget,
-					   const gchar         *first_arg_name,
+					   const gchar         *first_property_name,
 					   ...);
-void	   gtk_widget_setv		  (GtkWidget	       *widget,
-					   guint		nargs,
-					   GtkArg	       *args);
 void	   gtk_widget_unparent		  (GtkWidget	       *widget);
 void	   gtk_widget_show		  (GtkWidget	       *widget);
 void       gtk_widget_show_now            (GtkWidget           *widget);
@@ -439,19 +435,21 @@ void	   gtk_widget_queue_draw_area	  (GtkWidget	       *widget,
 					   gint                 y,
 					   gint                 width,
 					   gint                 height);
+#ifndef GTK_DISABLE_DEPRECATED
 void	   gtk_widget_queue_clear	  (GtkWidget	       *widget);
 void	   gtk_widget_queue_clear_area	  (GtkWidget	       *widget,
 					   gint                 x,
 					   gint                 y,
 					   gint                 width,
 					   gint                 height);
+#endif /* GTK_DISABLE_DEPRECATED */
 
 
 void	   gtk_widget_queue_resize	  (GtkWidget	       *widget);
+#ifndef GTK_DISABLE_DEPRECATED
 void	   gtk_widget_draw		  (GtkWidget	       *widget,
 					   GdkRectangle	       *area);
-void	   gtk_widget_draw_focus	  (GtkWidget	       *widget);
-void	   gtk_widget_draw_default	  (GtkWidget	       *widget);
+#endif /* GTK_DISABLE_DEPRECATED */
 void	   gtk_widget_size_request	  (GtkWidget	       *widget,
 					   GtkRequisition      *requisition);
 void	   gtk_widget_size_allocate	  (GtkWidget	       *widget,
@@ -478,8 +476,12 @@ guint	   gtk_widget_accelerator_signal  (GtkWidget           *widget,
 void	   gtk_widget_lock_accelerators   (GtkWidget	       *widget);
 void	   gtk_widget_unlock_accelerators (GtkWidget	       *widget);
 gboolean   gtk_widget_accelerators_locked (GtkWidget	       *widget);
-gint	   gtk_widget_event		  (GtkWidget	       *widget,
+gboolean   gtk_widget_mnemonic_activate   (GtkWidget           *widget,
+					   gboolean             group_cycling);
+gboolean   gtk_widget_event		  (GtkWidget	       *widget,
 					   GdkEvent	       *event);
+gint       gtk_widget_send_expose         (GtkWidget           *widget,
+					   GdkEvent            *event);
 
 gboolean   gtk_widget_activate		     (GtkWidget	       *widget);
 gboolean   gtk_widget_set_scroll_adjustments (GtkWidget        *widget,
@@ -488,19 +490,19 @@ gboolean   gtk_widget_set_scroll_adjustments (GtkWidget        *widget,
      
 void	   gtk_widget_reparent		  (GtkWidget	       *widget,
 					   GtkWidget	       *new_parent);
-void	   gtk_widget_popup		  (GtkWidget	       *widget,
-					   gint			x,
-					   gint			y);
-gint	   gtk_widget_intersect		  (GtkWidget	       *widget,
+gboolean   gtk_widget_intersect		  (GtkWidget	       *widget,
 					   GdkRectangle	       *area,
 					   GdkRectangle	       *intersection);
+GdkRegion *gtk_widget_region_intersect	  (GtkWidget	       *widget,
+					   GdkRegion	       *region);
 
+gboolean   gtk_widget_is_focus            (GtkWidget           *widget);
 void	   gtk_widget_grab_focus	  (GtkWidget	       *widget);
 void	   gtk_widget_grab_default	  (GtkWidget	       *widget);
 
 void	   gtk_widget_set_name		  (GtkWidget	       *widget,
 					   const gchar	       *name);
-gchar*	   gtk_widget_get_name		  (GtkWidget	       *widget);
+G_CONST_RETURN gchar* gtk_widget_get_name (GtkWidget	       *widget);
 void	   gtk_widget_set_state		  (GtkWidget	       *widget,
 					   GtkStateType		state);
 void	   gtk_widget_set_sensitive	  (GtkWidget	       *widget,
@@ -548,12 +550,12 @@ void	     gtk_widget_get_pointer	(GtkWidget	*widget,
 					 gint		*x,
 					 gint		*y);
 
-gint	     gtk_widget_is_ancestor	(GtkWidget	*widget,
+gboolean     gtk_widget_is_ancestor	(GtkWidget	*widget,
 					 GtkWidget	*ancestor);
 
 /* Hide widget and return TRUE.
  */
-gint	   gtk_widget_hide_on_delete	(GtkWidget	*widget);
+gboolean     gtk_widget_hide_on_delete	(GtkWidget	*widget);
 
 /* Widget styles.
  */
@@ -587,10 +589,10 @@ PangoContext *gtk_widget_get_pango_context    (GtkWidget   *widget);
 PangoLayout  *gtk_widget_create_pango_layout  (GtkWidget   *widget,
 					       const gchar *text);
 
-GdkPixbuf* gtk_widget_render_stock_icon       (GtkWidget      *widget,
-                                               const gchar    *stock_id,
-                                               const gchar    *size,
-                                               const gchar    *detail);
+GdkPixbuf    *gtk_widget_render_icon          (GtkWidget   *widget,
+                                               const gchar *stock_id,
+                                               GtkIconSize  size,
+                                               const gchar *detail);
 
 /* handle composite names for GTK_COMPOSITE_CHILD widgets,
  * the returned name is newly allocated.
@@ -613,6 +615,24 @@ void	     gtk_widget_pop_composite_child  (void);
 void	     gtk_widget_pop_style	     (void);
 void	     gtk_widget_pop_colormap	     (void);
 
+/* widget style properties
+ */
+void gtk_widget_class_install_style_property        (GtkWidgetClass     *klass,
+						     GParamSpec         *pspec);
+void gtk_widget_class_install_style_property_parser (GtkWidgetClass     *klass,
+						     GParamSpec         *pspec,
+						     GtkRcPropertyParser parser);
+void gtk_widget_style_get_property (GtkWidget	     *widget,
+				    const gchar    *property_name,
+				    GValue	     *value);
+void gtk_widget_style_get_valist   (GtkWidget	     *widget,
+				    const gchar    *first_property_name,
+				    va_list         var_args);
+void gtk_widget_style_get          (GtkWidget	     *widget,
+				    const gchar    *first_property_name,
+				    ...);
+
+
 /* Set certain default values to be used at widget creation time.
  */
 void	     gtk_widget_set_default_style    (GtkStyle	  *style);
@@ -653,12 +673,16 @@ void	     gtk_widget_class_path	   (GtkWidget *widget,
 					    gchar    **path,
 					    gchar    **path_reversed);
 
+GtkRequisition *gtk_requisition_copy (const GtkRequisition *requisition);
+void            gtk_requisition_free (GtkRequisition       *requisition);
+
 #if	defined (GTK_TRACE_OBJECTS) && defined (__GNUC__)
 #  define gtk_widget_ref gtk_object_ref
 #  define gtk_widget_unref gtk_object_unref
 #endif	/* GTK_TRACE_OBJECTS && __GNUC__ */
 
-
+GtkWidgetAuxInfo *_gtk_widget_get_aux_info (GtkWidget *widget,
+					    gboolean   create);
 
 #ifdef __cplusplus
 }
